#include <math.h>

#define INFINITY 1e64

#define max(a,b) (((a)>(b))?(a):(b))
#define min(a,b) (((a)>(b))?(b):(a))

int init_AC(double *cwt, double *freq, double **C, double *a, int Sx, int Sy)
{
	int i,j;
	for(i=0;i<Sy;i++)
	{
		a[i]=1.0/(freq[i]*Sx);
		C[i]=(double *)malloc(Sx*sizeof(double));
		if (!C[i]) return 1;
		for (j=0;j<Sx;j++) C[i][j]=log(fabs(cwt[i+j*Sy]));
	}
	return 0;
}

int Frontiere(double *cwt, double *freq, double *frontiere, int Sx, int Sy, int x, double smin, double smax, int NS, double epsilon)
{
	double **C;
	double *a;
	int i,j,is;
	int larg=floor(epsilon*Sx);
	double s=smin;
	double step=(smax-smin)/NS;
	a=(double *)malloc(Sy*sizeof(double));
	if (!a) return 1;
	C=(double **)malloc(Sy*sizeof(double *));
	if (!C) return 1;
	if (init_AC(cwt,freq,C,a,Sx,Sy)) return 1;
	for(is=0;is<NS;is++)
	{
		double s1;
		s1=-INFINITY;
		for(j=max(0,x-larg);j<min(Sx,x+larg)+1;j++)
		{
			double sb=0;
			double xxm,xym,xm,ym;
			int n;
			xxm=xym=xm=ym=0;
			n=0;
			for(i=0;i<Sy;i++)
			{
				double y=C[i][j]-(s+0.5)*log(a[i]);
				double ll=log(1+fabs(j-x)/(a[i]*Sx));
				if (y>-INFINITY)
				{
					ym+=y;
					xm+=ll;
					xxm+=ll*ll;
					xym+=ll*y;
					n++;
				}
			}
			if (n>0)
			{
				xm/=n;
				xxm/=n;
				ym/=n;
				xym/=n;
				sb=-(xym-xm*ym)/(xxm-xm*xm);
			}
			else sb=-INFINITY;
			if (sb>s1) s1=sb;
		}
		frontiere[is]=s1;
		s+=step;
	}
	free(a);
	for (i=0;i<Sy;i++) free(C[i]);
	free(C);
	return 0;
}

int Alpha_1(double *cwt, double *freq, double *alpha, int Sx, int Sy, double smin, double smax, int NS, double epsilon, int ponctuel)
{
	double **C;
	double *a;
	int i,j,is,x;
	int larg=floor(epsilon*Sx);
	double s=smin;
	double step=(smax-smin)/NS;
	a=(double *)malloc(Sy*sizeof(double));
	if (!a) return 1;
	C=(double **)malloc(Sy*sizeof(double *));
	if (!C) return 1;
	if (init_AC(cwt,freq,C,a,Sx,Sy)) return 1;
	for(x=0;x<Sx;x+=1)
	{
		double step=(smax-smin)/2.0;
		double s=smin;
		for(is=0;is<NS;is++)
		{
			double s1;
			s1=-INFINITY;
			for(j=max(0,x-larg);j<min(Sx,x+larg)+1;j++)
			{
				double sb=0;
				double xxm,xym,xm,ym;
				int n;
				xxm=xym=xm=ym=0;
				n=0;
				for(i=0;i<Sy;i++)
				{
					double y=C[i][j]-(s+0.5)*log(a[i]);
					double ll=log(1+fabs(j-x)/(a[i]*Sx));
					if (y>-INFINITY)
					{
						ym+=y;
						xm+=ll;
						xxm+=ll*ll;
						xym+=ll*y;
						n++;
					}
				}
				if (n>0)
				{
					xm/=n;
					xxm/=n;
					ym/=n;
					xym/=n;
					sb=-(xym-xm*ym)/(xxm-xm*xm);
				} else sb=-INFINITY;
				if (sb>s1) s1=sb;
			}
			if (ponctuel) s1+=s;
			if (s1<0) s-=step;
			step/=2;
			s+=step;
		}
		alpha[x]=s;
	}
	return 0;
}

int Alpha_2(double *cwt, double *freq, double *alpha, int Sx, int Sy, double smin, double smax, int NS, double epsilon, int ponctuel)
{
	double **C;
	double *a;
	int i,j,is,x;
	int larg=floor(epsilon*Sx);
	double s=smin;
	double step=(smax-smin)/NS;
	a=(double *)malloc(Sy*sizeof(double));
	if (!a) return 1;
	C=(double **)malloc(Sy*sizeof(double *));
	if (!C) return 1;
	if (init_AC(cwt,freq,C,a,Sx,Sy)) return 1;
	for(x=0;x<Sx;x+=1)
	{
		double step=(smax-smin)/2.0;
		double s=smin;
		for(is=0;is<NS;is++)
		{
			double s1;
			s1=-INFINITY;
			for(j=max(0,x-larg);j<min(Sx,x+larg)+1;j++)
			{
				double sb=0;
				double xxm,xym,xm,ym;
				int n;
				xxm=xym=xm=ym=0;
				n=0;
				for(i=0;i<Sy;i++)
				{
					double y=C[i][j]+s*log(1+fabs(j-x)/(a[i]*Sx));
					double ll=log(a[i]);
					if (y>-INFINITY)
					{
						ym+=y;
						xm+=ll;
						xxm+=ll*ll;
						xym+=ll*y;
						n++;
					}
				}
				if (n>0)
				{
					xm/=n;
					xxm/=n;
					ym/=n;
					xym/=n;
					sb=(xym-xm*ym)/(xxm-xm*xm)-0.5;
				} else sb=-INFINITY;
				if (sb>s1) s1=sb;
			}
			if (ponctuel) s1+=s;
			if (s1>0) s-=step;
			step/=2;
			s+=step;
		}
		alpha[x]=-s;
	}
	return 0;
}
