#include <math.h>
#include "C_LAB_Interf.h"
#include "Frontiere.h"

#define smin1 0.0
#define smax1 2.0
#define smin2 (-1.0)
#define smax2 1.0
#define NS 10

void LAB_alpha()
{
	double *coeff,*freq;
	Matrix *Mcoeff;
	Matrix *Mfreq;
	Matrix *Mtemp;
	int Sx,Sy,pw=0,meth=0,retour;
	double epsilon;

	double *alpha;
	Matrix *Malpha;

	if ((Interf.NbParamIn < 3) ||(Interf.NbParamIn > 5))
	{
		InterfError("Alpha: 3 to 5 parameters needed");
		return;
	}
	
	Mcoeff = Interf.Param[0];
	if (!MatrixIsNumeric(Mcoeff) || !MatrixIsReal(Mcoeff) )
	{
		InterfError("Alpha: The first parameter (cwt) must be a real matrix\n");
		return;
	}
	Sx=MatrixGetWidth(Mcoeff);
	Sy=MatrixGetHeight(Mcoeff);
	coeff=MatrixGetPr(Mcoeff);
	
	Mfreq = Interf.Param[1];
	if ( MIN(MatrixGetWidth(Mfreq),MatrixGetHeight(Mfreq)) != 1 )
	{
		InterfError("Alpha : The second parameter (freq) must be a (1xN) or (Nx1) matrix\n");
		return;
	}
	if ( MAX(MatrixGetWidth(Mfreq),MatrixGetHeight(Mfreq)) != Sy )
	{
		InterfError("Alpha : The second parameter (freq) must be of the same height than cwt\n");
		return;
	}
	freq=MatrixGetPr(Mfreq);
	
	Mtemp = Interf.Param[2];
	if ( !MatrixIsScalar(Mtemp) )
	{
		InterfError("Alpha : The third parameter (epsilon) must be a scalar\n");
		return;
	}
	epsilon=MatrixGetScalar(Mtemp);
	
	if (Interf.NbParamIn > 3)
	{
		Mtemp = Interf.Param[3];
		if ( !MatrixIsScalar(Mtemp) )
		{
			InterfError("Alpha : The fourth parameter (pointwise) must be a scalar\n");
			return;
		}
		pw=MatrixGetScalar(Mtemp)>0?1:0;
	}
	
	if (Interf.NbParamIn > 4)
	{
		Mtemp = Interf.Param[4];
		if ( !MatrixIsScalar(Mtemp) )
		{
			InterfError("Alpha : The fifth parameter (method) must be a scalar\n");
			return;
		}
		pw=MatrixGetScalar(Mtemp)>0?1:0;
	}
	
	Malpha= MatrixCreate(Sx,1,"real");
	alpha= MatrixGetPr(Malpha);
	/*printf("%d %d %lf %lf %lf %d %lf\n",Sx,Sy,floor(x*Sx),smin,smax,NS,epsilon);*/
	if (meth) retour=Alpha_2(coeff,freq,alpha,Sx,Sy,smin2,smax2,NS,epsilon,pw);
	else retour=Alpha_1(coeff,freq,alpha,Sx,Sy,smin1,smax1,NS,epsilon,pw);
	if (retour)
	{
		InterfError("Alpha : Not enough memory\n");
		return;
	}

	ReturnParam(Malpha);
	return;
}
