/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/FRACLAB-WKSP/FRACLAB/MFAS/SRC/LAB_cantor1d.c
* File type        : C source file
* Revision         : $Revision$
* RCS status       : $Id$
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Wed Sep 11 15:20:42 2002
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Thu Sep 12 12:53:53 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 2002 */

#include "C_LAB_Interf.h"
#include "MFAM_tree.h"

/************  LAB_cantor1d  *********************************************/

void LAB_cantor1d()
{ 
  register int k=0;
  /* input parameters */
  short n=12;
  short b=3;   
  int b_pow_n=pow((double)b,(double)n);
  int Nx=b_pow_n;     
  unsigned char* ucq=NULL; 
  double* q=NULL;
  unsigned char* uccantor1d=NULL; 
  double* cantor1d=NULL;
  MFAMt_tree* tree=NULL;
 
  /* Matrix */
  Matrix* Mn;
  Matrix* Mb;
  Matrix* Mq; 
  Matrix* Mcantor1d;

  /* Check the input arguments number */
  if(Interf.NbParamIn<1) 
    {
      InterfError("cantor1d: You must give at least 1 input argument\n");
      return;
    }
  
   if(Interf.NbParamIn>3) 
    {
      InterfError("cantor1d: You must give at most 3 input argument\n");
      return;
    }   
  
  /* Check the output arguments number */ 
  if(Interf.NbParamOut!=1) 
    {
      InterfError("cantor1d: You must give 1 output argument\n");
      return;
    }
 
  /* Get the input resolution: n and verify it */
  Mn=Interf.Param[0];
  if(!MatrixIsNumeric(Mn))
    {
      InterfError("cantor1d: The input resolution: n must be a numeric matrix\n");
      return;
    } 
  if(!MatrixIsReal(Mn))
    {
      InterfError("cantor1d: The input resolution: n must be a real matrix\n");
      return;
    }
  if(!MatrixIsScalar(Mn))
    {
      InterfError("cantor1d: The input resolution: n must be a scalar\n");
      return;
    }
  /* Get the input argument */ 
  n=(short)MatrixGetScalar(Mn);
 
  if(Interf.NbParamIn>1)
    {
      /* Get the input base: b and verify it */
      Mb=Interf.Param[1];
      if(!MatrixIsNumeric(Mb))
	{
	  InterfError("cantor1d: The input base: b must be a numeric matrix\n");
	  return;
	} 
      if(!MatrixIsReal(Mb))
	{
	  InterfError("cantor1d: The input base: b must be a real matrix\n");
	  return;
	}
      if(!MatrixIsScalar(Mb))
	{
	  InterfError("cantor1d: The input base: b must be a scalar\n");
	  return;
	}

      /* Get the input argument */ 
      b=(short)MatrixGetScalar(Mb);
    }
  /* else b=3 */
  
  /* dynamic memory allocation */
  if((ucq=(unsigned char*)malloc((size_t)(b*sizeof(unsigned char))))==NULL)
    {
      InterfError("cantor1d: malloc error\n");
      return;
    }

  if(Interf.NbParamIn>2)
    {
      /* Get the input weights: q and verify it */
      Mq=Interf.Param[2];
      if(!MatrixIsNumeric(Mq))
	{
	  InterfError("cantor1d: The input weights: q must be a numeric matrix\n");
	  return;
	} 
      if(!MatrixIsReal(Mq))
	{
	  InterfError("cantor1d: The input weights: q must be a real matrix\n");
	  return;
	}   
      if(MIN(MatrixGetWidth(Mq),MatrixGetHeight(Mq))!=1)
	{
	  InterfError("cantor1d: The input weights: p must be a real vector\n");
	  return;
	}
      if(MAX(MatrixGetWidth(Mq),MatrixGetHeight(Mq))!=b)
	{
	  InterfError("cantor1d: The input weights: q must be of size b\n");
	  return;
	}
      /* Get the input argument */  
      q=(double*)MatrixGetPr(Mq); 

      for(k=0;k<b;k++)
	*(ucq+k)=(unsigned char)*(q+k);
    }
  else
    {
      for(k=0;k<b;k++)
	*(ucq+k)=(unsigned char)(k%b);
    }
  
  /* cantor synthesis */
  tree=MFAM_tree_cantor(n,b,ucq); 
  uccantor1d=(unsigned char*)MFAM_tree_level_get(tree,MFAM_SET,n);
  
  
  /* dynamic memory deallocation */
  MFAM_tree_level_set(tree,MFAM_SET,n,(void*)NULL); 
  MFAM_tree_delete(tree); 
  
  /* Create the output Matrixes */   
  b_pow_n=pow((double)b,(double)n); 
  Nx=b_pow_n;
  Mcantor1d=MatrixCreate(Nx,1,"real"); 
  cantor1d=MatrixGetPr(Mcantor1d);
 
  for(k=0;k<Nx;k++)
    *(cantor1d+k)=(double)*(uccantor1d+k);
  
  /* dynamic memory deallocation */ 
  free((char*)ucq);
  free((char*)uccantor1d);
 
  /* Return the output */  
  ReturnParam(Mcantor1d); 
}
