/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /a/user/canus/MYFRACLAB/MFAL/SRC/LAB_fcfl1d.c
* File type        : C source file
* Revision         : $Revision: 1.1.1.1 $
* RCS status       : $Id: LAB_fcfl1d.c,v 1.1.1.1 1999/02/10 13:40:16 guiheneu Exp $
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sun Feb  7 16:54:41 1999
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Mon Feb  8 22:12:16 1999
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 1998 */

#include "C_LAB_Interf.h"
#include "MFAL_continuous.h"

/************  LAB_fcfl1d  ***************************************************/

void LAB_fcfl1d()
{ 
  /* input parameters */
  short J=10,Q=101,N=Q;
  int M=1024; 
  double p=1.,S_min=1,S_max=512,q_min=-10.,q_max=+10.;
  MFAMt_prog t_prog=MFAM_LOG; 
  MFAMt_osc t_osc=MFAM_OSC;
  MFAMt_lr t_lr=MFAM_LSLR;
  /* input function */
  double* x=NULL;
  double* f_x=NULL;
  /* output powers, partition sum & mass exponents functions */ 
  double* q=NULL;
  double* a=NULL;
  double* Z_a_q=NULL;
  double* t_q=NULL;
  /* output spectrum */
  double* alpha=NULL;
  double* fl_alpha=NULL;

  double* Pralpha=NULL;
  double* Prfl_alpha=NULL;
  Matrix* Mx;
  Matrix* Mf_x;
  Matrix* Mq;
  Matrix* MJ_min_max; 
  Matrix* Mprogstr;
  Matrix* Moscstr;
  Matrix* MZ_a_q;  
  Matrix* Ma;  
  Matrix* Mlrstr;
  Matrix* Mt_q;
  Matrix* Malpha; 
  Matrix* Mfl_alpha;

  /* Check the input arguments number */
  if(Interf.NbParamIn<1) 
    {
      InterfError("fcfl1d: You must give at least 1 input arguments\n");
      return;
    }
  if(Interf.NbParamIn>6) 
    {
      InterfError("fcfl1d: You must give at most 6 input arguments\n");
      return;
    }

  /* Check the output arguments number */
  if(Interf.NbParamOut<2)
    {
      InterfError("fcfl1d: You must give at least 2 output arguments\n");
      return;
    }
  if(Interf.NbParamOut>6)
    {
      InterfError("fcfl1d: You must give at most 6 output arguments\n");
      return;
    }
 
  /* Get the input function f_x and verify it */
  Mf_x=Interf.Param[0];
  if(!MatrixIsNumeric(Mf_x))
    {
      InterfError("fcfl1d: The function: f_x must be a numeric matrix\n");
      return;
    }
  if(!MatrixIsReal(Mf_x))
    {
      InterfError("fcfl1d: The function: f_x must be a real matrix\n");
      return;
    }
  M=(int)MAX(MatrixGetWidth(Mf_x),MatrixGetHeight(Mf_x));
  /* Get the input argument */ 
  f_x=MatrixGetPr(Mf_x);
 
  if(Interf.NbParamIn>1)
    {
      /* Get the input exponents q and verify it */
      Mq=Interf.Param[1];
      if(!MatrixIsNumeric(Mq))
	{
	  InterfError("fcfl1d: The exponents: q must be a numeric matrix\n");
	  return;
	}
      if(!MatrixIsReal(Mq))
	{
	  InterfError("fcfl1d: The exponents: q must be a real matrix\n");
	  return;
	} 
      Q=(short)MAX(MatrixGetWidth(Mq),MatrixGetHeight(Mq));
      /* Get the input argument */ 
      q=MatrixGetPr(Mq);
    }

  if(Interf.NbParamIn>2)
    {  
      /* Get the input vector: [J min max] and verify it */
      MJ_min_max=Interf.Param[2];
      if(!MatrixIsNumeric(MJ_min_max))
	{
	  InterfError("fcfl1d: The vector: [J min max] must be a numeric matrix\n");
	  return;
	}
      if(!MatrixIsReal(MJ_min_max))
	{
	  InterfError("fcfl1d: The vector: [J min max] must be a real matrix\n");
	  return;
	}
      if(((int)MAX(MatrixGetWidth(MJ_min_max),MatrixGetHeight(MJ_min_max)))!=3)
	{
	  InterfError("fcfl1d: The vector: [J min max] is illegal\n");
	  return;
	}  
      /* Get the input argument */ 
      J=(short)*(MatrixGetPr(MJ_min_max)); 
      S_min=*(MatrixGetPr(MJ_min_max)+1);
      S_max=*(MatrixGetPr(MJ_min_max)+2);
    }
  else
    {
      short b=0;
      if(MFAM_base(M,&b)==0)
	{
	  J=floor(log(M)/log(2.));
	  S_min=1;
	  S_max=floor(M/2.);
	}
      else
	{
	  J=floor(log(M)/log((double)b));
	  S_min=1;
	  S_max=floor(M/(double)b);
	}
    }
 
  if(Interf.NbParamIn>3)
    {  
      /* Get the input string: progstr and verify it */
      Mprogstr=Interf.Param[3];
      if(!MatrixIsString(Mprogstr))
	{
	  InterfError("fcfl1d: The string: progstr must be a string\n");
	  return;
	}
      /* Get the input argument */
      if(MFAM_progstr(MatrixReadString(Mprogstr),&t_prog)==0)
	{
	  InterfError("fcfl1d: The string: progstr is illegal\n");
	  return;
	}
    }

  if(Interf.NbParamIn>4)
    {
      /* Get the input string: oscstr and verify it */
      Moscstr=Interf.Param[4];
      if(!MatrixIsString(Moscstr))
	{
	  InterfError("fcfl1d: The string: oscstr must be a string\n");
	  return;
	}
      /* Get the input argument */
      if(MFAM_oscstr(MatrixReadString(Moscstr),&t_osc,&p)==0)
	{
	  InterfError("fcfl1d: The string: oscstr is illegal\n");
	  return;
	}
    }

  if(Interf.NbParamIn>5)
    {
      /* Get the input string: lrstr and verify it */
      Mlrstr=Interf.Param[5];
      if(!MatrixIsString(Mlrstr))
	{
	  InterfError("fcfl1d: The string: lrstr must be a string\n");
	  return;
	}
      /* Get the input argument */
      if(MFAM_lrstr(MatrixReadString(Mlrstr),&t_lr)==0)
	{
	  InterfError("fcfl1d: The string: lrstr is illegal\n");
	  return;
	}
    }

  N=Q;
 
  /* dynamic memory allocation */
  if((alpha=(double*)malloc((unsigned)(N*sizeof(double))))==NULL) 
    { 
      InterfError("fcfl1d: malloc error\n");
      return;
    }     
  if((fl_alpha=(double*)malloc((unsigned)(N*sizeof(double))))==NULL) 
    { 
      InterfError("fcfl1d: malloc error\n");
      return;
    } 
 
  /* Create the output Matrix */ 
  if(Interf.NbParamIn<2)
    {
      Mq=MatrixCreate(1,Q,"real");
      /* Get the output argument */ 
      q=MatrixGetPr(Mq);
      /* linear space */
      if(MFAM_linspace(Q,q_min,q_max,q)==0)
	{
	  fprintf(stderr,"malloc error\n"); 
	  return;
	} 
    }

  Ma=MatrixCreate(1,J,"real");
  MZ_a_q=MatrixCreate(Q,J,"real");
  Mt_q=MatrixCreate(1,Q,"real");

  /* Get the pointer on the output Matrix */  
  a=MatrixGetPr(Ma);
  Z_a_q=MatrixGetPr(MZ_a_q);
  t_q=MatrixGetPr(Mt_q);
  
  /* Call of the C function */
  if(MFAL_fcfl1d(M,x,f_x,t_prog,t_osc,p,J,S_min,S_max,a,Q,q,Z_a_q,t_lr,t_q,&N,alpha,fl_alpha)==0) 
    { 
      InterfError("fcfl1d: Call of the C function MFAL_fcfl1d failed\n"); 
      free((char*)alpha);
      free((char*)fl_alpha);
      return;
    }
 
  /* Create the output Matrix */  
  Malpha=MatrixCreate(1,N,"real");
  Mfl_alpha=MatrixCreate(1,N,"real");
 
  /* Get the pointer on the output Matrix */  
  Pralpha=MatrixGetPr(Malpha);
  Prfl_alpha=MatrixGetPr(Mfl_alpha);
  { 
    register n=0;
    for(n=0;n<N;n++)
      {   
	*(Pralpha+n)=*(alpha+n);
	*(Prfl_alpha+n)=*(fl_alpha+n);
      }
  }

  /* dynamic memory desallocation */
  free((char*)alpha);
  free((char*)fl_alpha);

  /* Transpose the output */
  MatrixTranspose(MZ_a_q);

  /* Return the output */
  ReturnParam(Malpha);
  ReturnParam(Mfl_alpha);
  if(Interf.NbParamOut>2)
    ReturnParam(MZ_a_q);
  else
    MatrixFree(MZ_a_q);
  if(Interf.NbParamOut>3)
    ReturnParam(Ma);
  else
    MatrixFree(Ma); 
  if(Interf.NbParamOut>4)
    ReturnParam(Mt_q);
  else
    MatrixFree(Mt_q); 
  if(Interf.NbParamOut>5)
    ReturnParam(Mq);
  else if(Interf.NbParamIn<2)
    MatrixFree(Mq);
}


  
