/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/FRACLAB-WKSP/FRACLAB/MFAS/SRC/LAB_inverse1d.c
* File type        : C source file
* Revision         : $Revision$
* RCS status       : $Id$
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Thu Sep 12 12:02:47 2002
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Wed Sep 12 16:05:00 2001
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 2002 */

#include "C_LAB_Interf.h"
#include "MFAM_include.h"

/************  LAB_inverse1d  *********************************************/

void LAB_inverse1d()
{ 
  register int k=0;
  /* input parameters */
  short n=12;
  short b=2;   
  int b_pow_n=(int)pow((double)b,(double)n);
  int Nx=b_pow_n;     
  double* inverse1d=NULL;

  /* Matrix */
  Matrix* Mn;
  Matrix* Mb;
  Matrix* Minverse1d;

  /* Check the input arguments number */
  if(Interf.NbParamIn<1) 
    {
      InterfError("inverse1d: You must give at least 1 input argument\n");
      return;
    }
  
   if(Interf.NbParamIn>2) 
    {
      InterfError("inverse1d: You must give at most 2 input argument\n");
      return;
    }   
  
  /* Check the output arguments number */ 
  if(Interf.NbParamOut!=1) 
    {
      InterfError("inverse1d: You must give 1 output argument\n");
      return;
    }
 
  /* Get the input resolution: n and verify it */
  Mn=Interf.Param[0];
  if(!MatrixIsNumeric(Mn))
    {
      InterfError("inverse1d: The input resolution: n must be a numeric matrix\n");
      return;
    } 
  if(!MatrixIsReal(Mn))
    {
      InterfError("inverse1d: The input resolution: n must be a real matrix\n");
      return;
    }
  if(!MatrixIsScalar(Mn))
    {
      InterfError("inverse1d: The input resolution: n must be a scalar\n");
      return;
    }
  /* Get the input argument */ 
  n=(short)MatrixGetScalar(Mn);
 
  fprintf(stderr,"n=%d\n",n);

  if(Interf.NbParamIn>1)
    {
      /* Get the input base: b and verify it */
      Mb=Interf.Param[1];
      if(!MatrixIsNumeric(Mb))
	{
	  InterfError("inverse1d: The input base: b must be a numeric matrix\n");
	  return;
	} 
      if(!MatrixIsReal(Mb))
	{
	  InterfError("inverse1d: The input base: b must be a real matrix\n");
	  return;
	}
      if(!MatrixIsScalar(Mb))
	{
	  InterfError("inverse1d: The input base: b must be a scalar\n");
	  return;
	}

      /* Get the input argument */ 
      b=(short)MatrixGetScalar(Mb);
    }
  /* else b=3 */
   
  fprintf(stderr,"b=%d\n",b);

  /* Create the output Matrixes */   
  b_pow_n=(int)pow((double)b,(double)n); 
  Nx=b_pow_n;
  Minverse1d=MatrixCreate(Nx,1,"real"); 
  inverse1d=MatrixGetPr(Minverse1d);
 
  fprintf(stderr,"Nx=%d\n",Nx);

  k=1;
  while(((1./(double)k)>(1./(double)b_pow_n))&&(k<Nx))
    {  
      int i=(int)ROUND((double)b_pow_n/(double)k);
      fprintf(stderr,"k=%d\n",k);
      fprintf(stderr,"i=%d\n",i);
      *(inverse1d+i)=1.;
      k++;
    } 

  /* Return the output */  
  ReturnParam(Minverse1d); 
}
