/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/FRACLAB-WKSP/FRACLAB/MFAG/SRC/LAB_mcfg2d.c
* File type        : C source file
* Revision         : $Revision: 1.1.1.1 $
* RCS status       : $Id: LAB_mcfg2d.c,v 1.1.1.1 1999/02/10 13:38:44 guiheneu Exp $
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sun Feb  7 16:55:56 1999
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Sun Sep 15 19:13:00 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 1998 */

#include "C_LAB_Interf.h"
#include "MFAG_continuous2d.h"

/************  LAB_mcfg2d  ***************************************************/

void LAB_mcfg2d()
{ 
  /* input parameters */
  int J=1,N=100;
  double S_min=1.,S_max=1.; 
  MFAMt_capa t_capa=MFAM_SUM; 
  MFAMt_prog t_prog=MFAM_LOG; 
  MFAMt_ball t_ball=MFAM_ASYMMETRIC; 
  MFAGt_adap t_adap=MFAG_MAXDEV;
  MFAGt_cont t_cont=MFAG_HORIKERN;
  MFAMt_kern t_kern=MFAM_GAUSSIAN;
  MFAGt_norm t_norm=MFAG_INFSUPPDF;
  /* input measure */
  int Nx=0,Ny=0; 
  double* I_nx=NULL;
  double* I_ny=NULL;
  double* mu_n2d=NULL; 
  double* nu_n2d=NULL;   
  /* input precisions */
  double* epsilon=NULL;
  /* output optimal precisions, scales, abscissa, ordinates & coarse Hoelder exponents */ 
  double* epsilon_star=NULL;
  double* eta=NULL; 
  double* x=NULL;
  double* y=NULL;
  double* alpha_eta_xy=NULL;
  /* output Hoelder exponents, pdfs & spectra */
  double* alpha=NULL;
  double* pc2d_eta_alpha=NULL;
  double* fgc2d_eta_alpha=NULL;

  /* Matrix */
  Matrix* MI_nx; 
  Matrix* MI_ny;
  Matrix* Mmu_n2d;
  Matrix* Mnu_n2d;  
  Matrix* Mprogstr;
  Matrix* Mballstr;  
  Matrix* MS_min_S_max_J;    
  Matrix* MN;
  Matrix* Mepsilon; 
  Matrix* Mcontstr;
  Matrix* Madapstr;
  Matrix* Mkernstr;
  Matrix* Mnormstr; 
  Matrix* Malpha;
  Matrix* Mpc2d_eta_alpha;
  Matrix* Mfgc2d_eta_alpha; 
  Matrix* Mepsilon_star;
  Matrix* Meta;
  Matrix* Malpha_eta_xy;

  /* Check the input arguments number */
  if(Interf.NbParamIn<1) 
    {
      InterfError("mcfg2d: You must give at least 1 input arguments\n");
      return;
    }
  if(Interf.NbParamIn>10) 
    {
      InterfError("mcfg2d: You must give at most 10 input arguments\n");
      return;
    }

  /* Check the output arguments number */ 
  if(Interf.NbParamOut<2) 
    {
      InterfError("mcfg2d: You must give at least 2 output arguments\n");
      return;
    }
  if(Interf.NbParamOut>6) 
    {
      InterfError("mcfg2d: You must give at most 6 output arguments\n");
      return;
    }
  
  /* Get the input 2d measure: mu_n2d and verify it */
  Mmu_n2d=Interf.Param[0];
  if(!MatrixIsNumeric(Mmu_n2d))
    {
      InterfError("mcfg2d: The measure: mu_n2d must be a numeric matrix\n");
      return;
    }
  if(!MatrixIsReal(Mmu_n2d))
    {
      InterfError("mcfg2d: The measure: mu_n2d must be a real matrix\n");
      return;
    }
  /* Get the input argument */  
  mu_n2d=(double*)MatrixGetPr(Mmu_n2d);
  /* Get the size of the input argument */ 
  Nx=(int)MatrixGetWidth(Mmu_n2d);
  Ny=(int)MatrixGetHeight(Mmu_n2d);
  if ((Nx<1)||(Ny<1))
    {
      InterfError("mcfg2d: The size (width,height) of the input analyzed measure: mu_n2d must be >0\n");
      return;
    }

  if(Interf.NbParamIn>1) 
    {
      /* Get the input minimal size: S_min, maximal size: S_max & # of scales: J and verify them */
      MS_min_S_max_J=Interf.Param[1];
      if(!MatrixIsNumeric(MS_min_S_max_J))
	{
	  InterfError("mcfg2d: The vector: [S_min S_max J] must be a numeric matrix\n");
	  return;
	}
      if(!MatrixIsReal(MS_min_S_max_J))
	{
	  InterfError("mcfg2d: The vector: [S_min S_max J] must be a real matrix\n");
	  return;
	}
      if(((int)MAX(MatrixGetWidth(MS_min_S_max_J),MatrixGetHeight(MS_min_S_max_J)))!=3)
	{
	  InterfError("mcfg2d: The vector: [S_min S_max J] is illegal\n");
	  return;
	} 
      /* Get the input argument */
      S_min=*(MatrixGetPr(MS_min_S_max_J));
      if(S_min<1.)
	{
	  InterfError("mcfg2d: The minimum size: S_min must be >=1.\n");
	  return;
	} 
      S_max=*(MatrixGetPr(MS_min_S_max_J)+1); 
      if(S_max<S_min)
	{
	  InterfError("mcfg2d: The maximum size: S_max must be >=S_min\n");
	  return;
	} 
      J=(short)*(MatrixGetPr(MS_min_S_max_J)+2); 
      if(J<1)
	{
	  InterfError("mcfg2d: The # of scales: J must be >=1\n");
	  return;
	} 
    }

  if(Interf.NbParamIn>2)
    {  
      /* Get the input string: progstr and verify it */
      Mprogstr=Interf.Param[2];
      if(!MatrixIsString(Mprogstr))
	{
	  InterfError("mcfg2d: The string: progstr must be a string\n");
	  return;
	}
      /* Get the input argument */
      if(MFAM_progstr(MatrixReadString(Mprogstr),&t_prog)==0)
	{
	  InterfError("mcfg2d: The string: progstr is illegal\n");
	  return;
	}
    }

  if(Interf.NbParamIn>3)
    {  
      /* Get the input string: ballstr and verify it */
      Mballstr=Interf.Param[3];
      if(!MatrixIsString(Mballstr))
	{
	  InterfError("mcfg2d: The string: ballstr must be a string\n");
	  return;
	}
      /* Get the input argument */
      if(MFAM_ballstr(MatrixReadString(Mballstr),&t_ball)==0)
	{
	  InterfError("mcfg2d: The string: ballstr is illegal\n");
	  return;
	}
    }
  
  if(Interf.NbParamIn>4)
    { 
      /* Get the input # of Hoelder exponents: N and verify it */
      MN=Interf.Param[4];
      if(!MatrixIsNumeric(MN))
	{
	  InterfError("mcfg2d: The # of Hoelder exponents: N  must be a numeric matrix\n");
	  return;
	}  
      if(!MatrixIsScalar(MN))
	{
	  InterfError("mcfg2d: The # of Hoelder exponents: N  must be a scalar\n");
	  return;
	}
      /* Get the input argument */
      N=(int)MatrixGetScalar(MN);
      if(N<1)
	{
	  InterfError("mcfg2d: The # of Hoelder exponents: N must be >=1\n");
	  return;
	}
    }

  if(Interf.NbParamIn>5)
    { 
      /* Get the input precision: epsilon and verify it */
      Mepsilon=Interf.Param[5];
      if(!MatrixIsNumeric(Mepsilon))
	{
	  InterfError("mcfg2d: The precision: epsilon must be a numeric matrix\n");
	  return;
	} 
      if(!MatrixIsReal(Mepsilon))
	{
	  InterfError("mcfg2d: The precision: epsilon must be a real matrix\n");
	  return;
	}
      /* Get the input argument */  
      epsilon=(double*)MatrixGetPr(Mepsilon);
      /* Get the size of the input argument */ 
      if(((int)MAX(MatrixGetWidth(Mepsilon),MatrixGetHeight(Mepsilon)))!=N)
	{
	  InterfError("mcfg2d: The size of the precision: epsilon must be N\n");
	  return;
	}  
    }
  else
    {
      /* dynamic memory allocation */
      if((epsilon=(double*)malloc((unsigned)(N*sizeof(double))))==NULL)
	{
	  InterfError("mcfg2d: malloc error\n"); 
	  return;
	}
    }
   
  if(Interf.NbParamIn>6)
    { 
      /* Get the input string: contstr and verify it */
      Mcontstr=Interf.Param[6];
      if(!MatrixIsString(Mcontstr))
	{
	  InterfError("mcfg2d: The string: contstr must be a string\n");
	  return;
	}
      /* Get the input argument */
      if(MFAG_contstr(MatrixReadString(Mcontstr),&t_cont)==0)
	{
	  InterfError("mcfg2d: The string: contstr is illegal\n");
	  return;
	}
    }

  if(Interf.NbParamIn>7)
    { 
      /* Get the input string: adapstr and verify it */
      Madapstr=Interf.Param[7];
      if(!MatrixIsString(Madapstr))
	{
	  InterfError("mcfg2d: The string: adapstr must be a string\n");
	  return;
	}
      /* Get the input argument */
      if(MFAG_adapstr(MatrixReadString(Madapstr),&t_adap)==0)
	{
	  InterfError("mcfg2d: The string: adapstr is illegal\n");
	  return;
	}
    }

  if(Interf.NbParamIn>8)
    { 
      /* Get the input string: kernstr and verify it */
      Mkernstr=Interf.Param[8];
      if(!MatrixIsString(Mkernstr))
	{
	  InterfError("mcfg2d: The string: kernstr must be a string\n");
	  return;
	}
      /* Get the input argument */
      if(MFAM_kernelstr(MatrixReadString(Mkernstr),&t_kern)==0)
	{
	  InterfError("mcfg2d: The string: kernstr is illegal\n");
	  return;
	}
    }

  if(Interf.NbParamIn>9)
    { 
      /* Get the input string: normstr and verify it */
      Mnormstr=Interf.Param[9];
      if(!MatrixIsString(Mnormstr))
	{
	  InterfError("mcfg2d: The string: normstr must be a string\n");
	  return;
	}
      /* Get the input argument */
      if(MFAG_normstr(MatrixReadString(Mnormstr),&t_norm)==0)
	{
	  InterfError("mcfg2d: The string: normstr is illegal\n");
	  return;
	}
    }

  if(Interf.NbParamIn>10) 
    { 
      /* Get the input reference 2d measure: nu_n and verify it */
      Mnu_n2d=Interf.Param[10];
      if(!MatrixIsNumeric(Mnu_n2d))
	{
	  InterfError("mcfg2d: The measure: nu_n2d must be a numeric matrix\n");
	  return;
	}
      if(!MatrixIsReal(Mnu_n2d))
	{
	  InterfError("mcfg2d: The measure: nu_n2d must be a real matrix\n");
	  return;
	}
      /* Get the input argument */  
      nu_n2d=(double*)MatrixGetPr(Mnu_n2d);
      /* Get the size of the input argument */ 
      if((Nx!=(int)MatrixGetWidth(Mnu_n2d))||(Ny=(int)MatrixGetHeight(Mnu_n2d)))
	{
	  InterfError("mcfg2d: The size (width,height) of the input analysed measure: mu_n2d and the input reference 2d measure: nu_n2d must be =\n");
	  return;
	}
    }

  /* Create the output Matrix */ 
  Meta=MatrixCreate(1,J,"real");  
  Malpha_eta_xy=MatrixCreate(Nx,Ny,"real");
  Malpha=MatrixCreate(1,N,"real");
  Mpc2d_eta_alpha=MatrixCreate(1,N,"real"); 
  Mfgc2d_eta_alpha=MatrixCreate(1,N,"real");
  Mepsilon_star=MatrixCreate(N,J,"real");  
    
  /* Get the pointer on the output Matrix */ 
  eta=MatrixGetPr(Meta);
  alpha_eta_xy=MatrixGetPr(Malpha_eta_xy);  
  alpha=MatrixGetPr(Malpha); 
  pc2d_eta_alpha=MatrixGetPr(Mpc2d_eta_alpha);
  fgc2d_eta_alpha=MatrixGetPr(Mfgc2d_eta_alpha);  
  epsilon_star=MatrixGetPr(Mepsilon_star);

  /* Transpose Matrix */
  MatrixTranspose(Mmu_n2d);
  *eta=1./sqrt(Nx*Ny); 
 
  /* Call of the C function */ 
  if(MFAG_mcfg2d(Nx,Ny,I_nx,I_ny,mu_n2d,nu_n2d,1,1,J,t_capa,t_prog,t_ball,eta,Nx,Ny,x,y,alpha_eta_xy,N,epsilon,t_cont,t_adap,t_kern,t_norm,alpha,pc2d_eta_alpha,fgc2d_eta_alpha,epsilon_star)==0)
    {
      InterfError("mcfg2d: Call of the C function MFAG_mcfg2d failed\n");
      return;
    }   

  /* Transpose input Matrixes */
  MatrixTranspose(Mmu_n2d);

  /* Transpose output Matrixes  */
  MatrixTranspose(Malpha_eta_xy);

  /* Return the output */
  ReturnParam(Malpha);
  ReturnParam(Mfgc2d_eta_alpha); 
  if(Interf.NbParamIn<6)
    free((char*)epsilon);
  if(Interf.NbParamOut>2) 
    ReturnParam(Mpc2d_eta_alpha);
  else
    MatrixFree(Mpc2d_eta_alpha);
 
  MatrixFree(Malpha_eta_xy);
}


