/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /a/user/canus/MYFRACLAB/MFAL/SRC/LAB_mdzq1d.c
* File type        : C source file
* Revision         : $Revision: 1.1.1.1 $
* RCS status       : $Id: LAB_mdzq1d.c,v 1.1.1.1 1999/02/10 13:40:16 guiheneu Exp $
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sun Feb  7 16:54:19 1999
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Mon Feb  8 22:42:06 1999
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 1997-98 */

#include "C_LAB_Interf.h"
#include "MFAL_partition.h"

/************  LAB_mdzq1d  ***********************************************/

void LAB_mdzq1d()
{ 
  /* input parameters */
  short J=10,Q=21;
  int N_n=1024;
  double S_min=1,S_max=512,P_min=2,P_max=1024,q_min=-10.,q_max=+10.;
  MFAMt_part t_part=MFAM_LOGPART;
  /* input measure */
  double* mu_n=NULL;
  /* output powers, scales & partition function */
  double* q=NULL;
  double* a=NULL;
  double* Z_a_q=NULL;

  /* input/output matrix */
  Matrix* Mmu_n;
  Matrix* Mq;
  Matrix* MJ_min_max;
  Matrix* Mpartstr;
  Matrix* MZ_a_q;  
  Matrix* Ma; 
 
  /* Check the input arguments number */
  if(Interf.NbParamIn<1) 
    {
      InterfError("mdzq1d: You must give at least 1 input argument\n");
      return;
    }
  if(Interf.NbParamIn>4) 
    {
      InterfError("mdzq1d: You must give at least 4 input arguments\n");
      return;
    }
  
  /* Check the output arguments number */
  if(Interf.NbParamOut<1)
    {
      InterfError("mdzq1d: You must give at most 1 output argument\n");
      return;
    }
  if(Interf.NbParamOut>3)
    {
      InterfError("mdzq1d: You must give at most 3 output arguments\n");
      return;
    } 
  
  /* Get the input measure mu_n and verify it */
  Mmu_n=Interf.Param[0];
  if(!MatrixIsNumeric(Mmu_n))
    {
      InterfError("mdzq1d: The measure: mu_n must be a numeric matrix\n");
      return;
    }
  if(!MatrixIsReal(Mmu_n))
    {
      InterfError("mdzq1d: The measure: mu_n must be a real matrix\n");
      return;
    }
  N_n=(int)MAX(MatrixGetWidth(Mmu_n),MatrixGetHeight(Mmu_n));
  /* Get the input argument */ 
  mu_n=MatrixGetPr(Mmu_n);
   
  if(Interf.NbParamIn>1)
    {
      /* Get the input exponents q and verify it */
      Mq=Interf.Param[1];
      if(!MatrixIsNumeric(Mq))
	{
	  InterfError("mdzq1d: The exponents: q must be a numeric matrix\n");
	  return;
	}
      if(!MatrixIsReal(Mq))
	{
	  InterfError("mdzq1d: The exponents: q must be a real matrix\n");
	  return;
	} 
      Q=(short)MAX(MatrixGetWidth(Mq),MatrixGetHeight(Mq));
      /* Get the input argument */ 
      q=MatrixGetPr(Mq);
    }

  if(Interf.NbParamIn>2)
    {  
      /* Get the input vector: [J min max] and verify it */
      MJ_min_max=Interf.Param[2];
      if(!MatrixIsNumeric(MJ_min_max))
	{
	  InterfError("mdzq1d: The vector: [J min max] must be a numeric matrix\n");
	  return;
	}
      if(!MatrixIsReal(MJ_min_max))
	{
	  InterfError("mdzq1d: The vector: [J min max] must be a real matrix\n");
	  return;
	}
      if(((int)MAX(MatrixGetWidth(MJ_min_max),MatrixGetHeight(MJ_min_max)))!=3)
	{
	  InterfError("mdzq1d: The vector: [J min max] is illegal\n");
	  return;
	}
      /* Get the input argument */ 
      J=(short)*(MatrixGetPr(MJ_min_max));
      if(t_part==MFAM_DECSIZE)
	{
	  S_min=*(MatrixGetPr(MJ_min_max)+1);
	  S_max=*(MatrixGetPr(MJ_min_max)+2);
	}
      else
	{ 
	  P_min=*(MatrixGetPr(MJ_min_max)+1);
	  P_max=*(MatrixGetPr(MJ_min_max)+2);
	}   
    }
  else
    {
      short b=0;
      if(MFAM_base(N_n,&b)==0)
	{
	  J=floor(log(N_n)/log(2.));
	  S_min=1;
	  S_max=floor(N_n/2.);
	  P_min=1;
	  P_max=floor(N_n/(double)2);
	}
      else
	{
	  J=floor(log(N_n)/log((double)b));
	  S_min=1;
	  S_max=floor(N_n/(double)b);
	  P_min=1;
	  P_max=floor(N_n/(double)b);
	}
    }
 
  if(Interf.NbParamIn>3)
    {  
      /* Get the input string: partstr and verify it */
      Mpartstr=Interf.Param[3];
      if(!MatrixIsString(Mpartstr))
	{
	  InterfError("mdzq1d: The string: partstr must be a string\n");
	  return;
	}
      /* Get the input argument */
      if(MFAM_partstr(MatrixReadString(Mpartstr),&t_part)==0)
	{
	  InterfError("mdzq1d: The string: partstr is illegal\n");
	  return;
	}
    }

  /* Create the output Matrix */
  if(Interf.NbParamIn<2)
    {
      Mq=MatrixCreate(1,Q,"real");
      /* Get the output argument */ 
      q=MatrixGetPr(Mq);
      /* linear space */
      if(MFAM_linspace(Q,q_min,q_max,q)==0)
	{
	  fprintf(stderr,"malloc error\n"); 
	  return;
	} 
    }
  Ma=MatrixCreate(1,J,"real");
  MZ_a_q=MatrixCreate(Q,J,"real");

  /* Get the pointer on the output Matrix */  
  a=MatrixGetPr(Ma);
  Z_a_q=MatrixGetPr(MZ_a_q);
     
  /* Call of the C function */ 
  if(MFAL_mdZq1d(N_n,mu_n,t_part,J,S_min,S_max,P_min,P_max,a,Q,q,Z_a_q)==0)
    {
      InterfError("mdzq1d: Call of the C function MFAL_mdZq1d error\n");
      return;
    } 

  /* Transpose the output */
  MatrixTranspose(MZ_a_q);

  /* Return the output */
  ReturnParam(MZ_a_q);
  if(Interf.NbParamOut>1)
    ReturnParam(Ma);
  else
    MatrixFree(Ma);
  if(Interf.NbParamOut>2)
    ReturnParam(Mq); 
  else if(Interf.NbParamIn<2)
    MatrixFree(Mq);
}


