/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/FRACLAB-WKSP/FRACLAB/MFAH/SRC/LAB_mfd1d.c
* File type        : C source file
* Revision         : $Revision$
* RCS status       : $Id$
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sun Aug 25 19:09:32 2002
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Sun Sep 15 13:43:01 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 2002 */

#include "C_LAB_Interf.h"
#include "MFAH_dimension.h"

/************  LAB_mfd1d  ****************************************************/

void LAB_mfd1d()
{
  /* input 1d capacity and measure */
  int Nx=0; 
  double* c1d=NULL;
  double* mu1d=NULL;
  /* pointwise hoelder exponents estimation parameters */
  int J=0;
  double* S=NULL;
  MFAMt_capa t_capa=MFAM_SUM;
  /* lp measure parameter */
  double P=2.;
  /* iso capacity parameter */
  int Q=50; 
  /* dimension estimation parameters */
  double epsilon=0.; 
  int K=5;
  int b_ini=1,b_fin=33;
  double* B=NULL; 
  MFADt_dim t_dim=MFAD_BOX; 
  /* output pointwise hoelder exponents */
  double* p_alpha1d=NULL;
  /* ouput iso-hoelder set */
  unsigned char* iso_alpha1d=NULL;
  /* output spectrum */
  int N=100;
  double* alpha=NULL;
  double* fd1d_alpha=NULL;

  /* Matrix */
  Matrix* Mc1d;  
  Matrix* Mmu1d; 
  Matrix* MS;
  Matrix* Mcapa; 
  Matrix* Mepsilon; 
  Matrix* MB; 
  Matrix* Mdim;
  Matrix* MN;
  Matrix* Mp_alpha1d;
  Matrix* Malpha;
  Matrix* Mfd1d_alpha;

  /* Check the input arguments number */
  if(Interf.NbParamIn<1) 
    {
      InterfError("mfd1d: You must give at least 1 input argument\n");
      return;
    }  
  if(Interf.NbParamIn>8) 
    {
      InterfError("mfd1d: You must give at most 8 input arguments\n");
      return;
    }

  /* Check the output arguments number */ 
  if(Interf.NbParamOut<2) 
    {
      InterfError("mfd1d: You must give at least 2 output arguments\n");
      return;
    }
  if(Interf.NbParamOut>3) 
    {
      InterfError("mfd1d: You must give at most 3 output arguments\n");
      return;
    }

  /* Get the input capacity: c1d and verify it */
  Mc1d=Interf.Param[0];
  if(!MatrixIsNumeric(Mc1d)) 
    {
      InterfError("mfd1d: The input capacity: c1d must be a numeric matrix\n");
      return;
    }

  if(!MatrixIsReal(Mc1d)) 
    {
      InterfError("mfd1d: The input capacity: c1d must be a real matrix\n");
      return;
    }

  /* Get the input argument */  
  c1d=(double*)MatrixGetPr(Mc1d);
 
  /* Get the size of the input argument */ 
  Nx=(int)MAX(MatrixGetWidth(Mc1d),MatrixGetHeight(Mc1d));
  if (Nx<=0)
    {
      InterfError("mfd1d: The size of the input capacity: c1d must be >0\n");
      return;
    }

  if(Interf.NbParamIn>1) 
    {
      /* Get the capacity and verify it */
      Mcapa=Interf.Param[1];
      if (!MatrixIsString(Mcapa)) 
	{
	  InterfError("mfd1d: the input capacity: capa must be a string\n");
	  MatrixTranspose(Mc1d);
	  return;
	}

      /* Get the input argument */
      if(MFAM_capastr(MatrixReadString(Mcapa),&t_capa)==0) 
	{
	  InterfError("mfd1d: invalid input capacity\n");
	  return;
	}
    }
  /* else t_capa=MFAM_SUM */
 
  if(Interf.NbParamIn>2) 
    {
      /* Get the input sizes: S and verify it */
      MS=Interf.Param[2];
      if(!MatrixIsNumeric(MS))
	{
	  InterfError("mfd1d: The input sizes: S must be a numeric matrix\n");
	  return;
	}
      if(!MatrixIsReal(MS))
	{
	  InterfError("mfd1d: The input sizes: S must be a real matrix\n");
	  return;
	}
      if(((int)MIN(MatrixGetWidth(MS),MatrixGetHeight(MS)))!=1)
	{
	  InterfError("mfd1d: The input sizes: S must be a vector\n");
	  return;
	} 
      /* Get the size of the input argument */ 
      J=(int)MAX(MatrixGetWidth(MS),MatrixGetHeight(MS)); 
      if(J<1)
	{
	  InterfError("mfd1d: The length of the input sizes vector: S must be >0\n");
	  return;
	} 
      /* Get the input argument */
      S=MatrixGetPr(MS);
      if(*S>*(S+J-1))
	{
	  InterfError("mfd1d: The input sizes vector: S must be increasing\n");
	  return;
	}
    }
  else
    { 
      register int j=0;
      int s_ini=1,s_fin=5;   
      J=(s_fin-s_ini)/2+1; 
      /* dynamic memory allocation */
      if((S=(double*)malloc((size_t)(J*sizeof(double))))==NULL)
	{
	  InterfError("mfd1d: malloc error\n");
	  return;
	}
      for (j=0;j<J;j++)
	*(S+j)=s_ini+j*2;
    }

  if(Interf.NbParamIn>3) 
    { 
      /* Get the input precision: epsilon and verify it */
      Mepsilon=Interf.Param[3];
      if(!MatrixIsNumeric(Mepsilon)) 
	{
	  InterfError("mfd1d : The input precision: epsilon must be a numeric matrix\n");
	  return;
	}
      if(!MatrixIsReal(Mepsilon)) 
	{
	  InterfError("mfd1d : The input precision: epsilon must be a real matrix\n");
	  return;
	}
      if(!MatrixIsScalar(Mepsilon))
	{
	  InterfError("mfd1d: The input precision: epsilon must be a scalar\n");
	  return;
	}

      /* Get the input argument */  
      epsilon=(double)MatrixGetScalar(Mepsilon);
    }
  /* else epsilon=.1; */

  if(Interf.NbParamIn>4) 
    {
      /* Get the input box sizes: B and verify it */
      MB=Interf.Param[4];

      if(!MatrixIsNumeric(MB))
	{
	  InterfError("mfd1d: The input box sizes: B must be a numeric matrix\n");
	  return;
	}

      if(!MatrixIsReal(MB))
	{
	  InterfError("mfd1d: The input box sizes: B must be a real matrix\n");
	  return;
	}

      if(((int)MIN(MatrixGetWidth(MB),MatrixGetHeight(MB)))!=1)
	{
	  InterfError("mfd1d: The input box sizes: B must be a vector\n");
	  return;
	} 

      /* Get the size of the input argument */ 
      K=(int)MAX(MatrixGetWidth(MB),MatrixGetHeight(MB)); 
      if(K<1)
	{
	  InterfError("mfd1d: The length of the input box sizes vector: B must be >0\n");
	  return;
	} 
 
      /* Get the input argument */
      B=MatrixGetPr(MB);
      if(*B>*(B+K-1))
	{
	  InterfError("mfd1d: The input box sizes vector: B must be increasing\n");
	  return;
	}
    }
  else
    { 
      register int k=0;
      int b_ini=1,b_fin=33;   
      K=(b_fin-b_ini)/2+1; 
      /* dynamic memory allocation */
      if((B=(double*)malloc((size_t)(K*sizeof(double))))==NULL)
	{
	  InterfError("mfd1d: malloc error\n");
	  return;
	}
      for (k=0;k<K;k++)
	*(B+k)=b_ini+k*2;
    }

  if(Interf.NbParamIn>5) 
    { 
      /* Get the dimension and verify it */
      Mdim=Interf.Param[5];
      if (!MatrixIsString(Mdim)) 
	{
	  InterfError("mfd1d: the input capacity: capa must be a string\n");
	  MatrixTranspose(Mc1d);
	  return;
	}

      /* Get the input argument */
      if(MFAD_dimstr(MatrixReadString(Mdim),&t_dim)==0) 
	{
	  InterfError("mfd1d: invalid input capacity\n");
	  return;
	}
    } 
  /* else t_dim=MFAH_BOX */
  
  if(Interf.NbParamIn>6) 
    { 
      /* Get the input reference measure: mu1d */
      Mmu1d=Interf.Param[6];
      if(!MatrixIsNumeric(Mmu1d)) 
	{
	  InterfError("mfd1d: The input reference measure: mu1d must be a numeric matrix\n");
	  return;
	}
    
      if(!MatrixIsReal(Mmu1d)) 
	{
	  InterfError("mfd1d: The input reference measure: mu1d must be a real matrix\n");
	  return;
	}
  
      if(Nx!=(int)MAX(MatrixGetWidth(Mmu1d),(int)MatrixGetHeight(Mmu1d)))
	{
	  InterfError("mfd1d: The size of the input capacity: c1d and reference measure: mu1d must be equal\n");
	  return;
	}

      /* Get the input argument */  
      mu1d=MatrixGetPr(Mmu1d);
    }
  /* else mu1d=NULL */

  if(Interf.NbParamIn>7) 
  { 
    /* Get the input # of Hlder exponents: N and verify it */
    MN=Interf.Param[7];
    if(!MatrixIsNumeric(MN)) 
    {
      InterfError("mfd1d: The input # of Hoelder exponents: N must be a numeric matrix\n");
      return;
    }
    if(!MatrixIsReal(MN)) 
    {
      InterfError("mfd1d : The input # of Hoelder exponents: N must be a real matrix\n");
      return;
    }
    if(!MatrixIsScalar(MN))
      {
	InterfError("mfd1d: The input # of Hoelder exponents: N must be a scalar\n");
	return;
      }  

    /* Get the input argument */  
    N=(int)MatrixGetScalar(MN);
  } 
  /* else N=100 */

  /* Create the output Matrix */  
  Mp_alpha1d=MatrixCreate(Nx,1,"real"); 
  p_alpha1d=MatrixGetPr(Mp_alpha1d); 
  Malpha=MatrixCreate(N,1,"real"); 
  alpha=MatrixGetPr(Malpha); 
  Mfd1d_alpha=MatrixCreate(N,1,"real"); 
  fd1d_alpha=MatrixGetPr(Mfd1d_alpha);

  /* dynamic memory allocation */
  if((iso_alpha1d=(unsigned char*)malloc((size_t)(Nx*sizeof(unsigned char))))==NULL)
    {
      InterfError("mfd1d: malloc error\n");
      return;
    }

  /* Call of the C function */
  if(MFAH_mfd1d(Nx,c1d,mu1d,J,S,t_capa,P,Q,p_alpha1d,epsilon,iso_alpha1d,K,B,t_dim,N,alpha,fd1d_alpha)==0)
    { 
      InterfError("mfd1d: Call of the C function MFAH_mfd1d failed\n"); 
      return; 
    }

  /* dynamic memory deallocation */
  free((char*)iso_alpha1d);

  if (Interf.NbParamIn<=2) 
    free((char*)S); 

  if (Interf.NbParamIn<=4) 
    free((char*)B);

  /* Return the output */  
  ReturnParam(Malpha); 
  ReturnParam(Mfd1d_alpha);  

  if(Interf.NbParamOut>2) 
    ReturnParam(Mp_alpha1d);
  else
    MatrixFree(Mp_alpha1d); 
}
