/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/FRACLAB-WKSP/FRACLAB/MFAH/SRC/LAB_mfdls1d.c
* File type        : C source file
* Revision         : $Revision$
* RCS status       : $Id$
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Wed Aug 21 23:44:31 2002
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Sun Sep 15 16:09:51 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 2002 */

#include "C_LAB_Interf.h"
#include "MFAH_mfdlimsup1d.h"

/************  LAB_mfdls1d  ****************************************************/

void LAB_mfdls1d()
{ 
  /* input 1d capacity and measure */
  int Nx=0;
  double* c1d=NULL;
  double* mu1d=NULL;
  /* input  arguments */ 
  short n_ini=0;
  short n_fin=0;
  MFAMt_capa t_capa=MFAM_SUM;  
  /* lp measure parameter */
  double P=2.;
  /* iso capacity parameter */
  int Q=50; 
  /* dimension estimation parameters */ 
  double epsilon=.1;
  double* B=NULL;
  int K=11;
  int b_ini=1,b_fin=33;  
  MFADt_dim t_dim=MFAD_BOX; 
  /* output spectrum */ 
  int N=100;
  double* alpha=NULL;
  double* fdls1d_alpha=NULL;

  /* Matrix */
  Matrix* Mc1d; 
  Matrix* Mmu1d; 
  Matrix* Mn;  
  double* n; 
  Matrix* Mcapa;
  Matrix* Mepsilon; 
  Matrix* MB; 
  Matrix* Mdim;
  Matrix* MN;
  Matrix* Malpha;
  Matrix* Mfdls1d_alpha;

  /* Check the input arguments number */
  if(Interf.NbParamIn<1) 
    {
      InterfError("mfdls1d: You must give at least 1 input argument\n");
      return;
    }  
  if(Interf.NbParamIn>8) 
    {
      InterfError("mfdls1d: You must give at most 8 input arguments\n");
      return;
    }

  /* Check the output arguments number */ 
  if(Interf.NbParamOut!=2) 
    {
      InterfError("mfdls1d: You must give 2 output argument\n");
      return;
    }

  /* Get the input capacity: c1d and verify it */
  Mc1d=Interf.Param[0];
  if(!MatrixIsNumeric(Mc1d)) 
    {
      InterfError("mfdls1d: The input capacity: c1d must be a numeric matrix\n");
      return;
    }

  if(!MatrixIsReal(Mc1d)) 
    {
      InterfError("mfdls1d: The input capacity: c1d must be a real matrix\n");
      return;
    }

  /* Get the input argument */  
  c1d=(double*)MatrixGetPr(Mc1d);
 
  /* Get the size of the input argument */ 
  Nx=(int)MAX(MatrixGetWidth(Mc1d),MatrixGetHeight(Mc1d));
  if(Nx<=0)
    {
      InterfError("mfdls1d: The size of the input capacity: c1d must be >0\n");
      return;
    }

  if(Interf.NbParamIn>1) 
    {
      /* Get the input resolutions: n and verify it */
      Mn=Interf.Param[1];

      if(!MatrixIsNumeric(Mn))
	{
	  InterfError("mfdls1d: The input resolutions: n must be a numeric matrix\n");
	  return;
	}

      if(!MatrixIsReal(Mn))
	{
	  InterfError("mfdls1d: The input resolutions: n must be a real matrix\n");
	  return;
	}

      if(((int)MIN(MatrixGetWidth(Mn),MatrixGetHeight(Mn)))!=1)
	{
	  InterfError("mfdls1d: The input resolutions: n must be a vector\n");
	  return;
	} 

      /* Get the size of the input argument */ 
      K=(int)MAX(MatrixGetWidth(Mn),MatrixGetHeight(Mn)); 
      if(K!=2)
	{
	  InterfError("mfdls1d: The length of the input resolutions: n vector must be = 2\n");
	  return;
	} 
 
      /* Get the input argument */
      n=MatrixGetPr(Mn);
      if(*n>*(n+1))
	{
	  InterfError("mfdls1d: The input box sizes vector: B must be increasing\n");
	  return;
	}

      /* Get the input argument */  
      n_ini=(int)*n;
      n_fin=(int)*(n+1);
    }
  /* else n_ini=0; n_fin=0; */

  if(Interf.NbParamIn>2) 
    { 
      /* Get the capacity and verify it */
      Mcapa=Interf.Param[2];
      if (!MatrixIsString(Mcapa)) 
	{
	  InterfError("mfdls1d: the input capacity: capa must be a string\n");
	  MatrixTranspose(Mc1d);
	  return;
	}

      /* Get the input argument */
      if(MFAM_capastr(MatrixReadString(Mcapa),&t_capa)==0) 
	{
	  InterfError("mfdls1d: invalid input capacity\n");
	  return;
	}
    }
  /* else t_capa=MFAM_SUM */
 
  if(Interf.NbParamIn>3) 
    { 
      /* Get the input precision: epsilon and verify it */
      Mepsilon=Interf.Param[3];
      if(!MatrixIsNumeric(Mepsilon)) 
	{
	  InterfError("mfdls1d: The input precision: epsilon must be a numeric matrix\n");
	  return;
	}
      if(!MatrixIsReal(Mepsilon)) 
	{
	  InterfError("mfdls1d: The input precision: epsilon must be a real matrix\n");
	  return;
	}
      if(!MatrixIsScalar(Mepsilon))
	{
	  InterfError("mfdls1d: The input precision: epsilon must be a scalar\n");
	  return;
	}

      /* Get the input argument */  
      epsilon=(double)MatrixGetScalar(Mepsilon);
    }
  /* else epsilon=.1; */

  if(Interf.NbParamIn>4) 
    {
      /* Get the input box sizes: B and verify it */
      MB=Interf.Param[4];

      if(!MatrixIsNumeric(MB))
	{
	  InterfError("mfdls1d: The input box sizes: B must be a numeric matrix\n");
	  return;
	}

      if(!MatrixIsReal(MB))
	{
	  InterfError("mfdls1d: The input box sizes: B must be a real matrix\n");
	  return;
	}

      if(((int)MIN(MatrixGetWidth(MB),MatrixGetHeight(MB)))!=1)
	{
	  InterfError("mfdls1d: The input box sizes: B must be a vector\n");
	  return;
	} 

      /* Get the size of the input argument */ 
      K=(int)MAX(MatrixGetWidth(MB),MatrixGetHeight(MB)); 
      if(K<1)
	{
	  InterfError("mfdls1d: The length of the input box sizes vector: B must be >0\n");
	  return;
	} 
 
      /* Get the input argument */
      B=MatrixGetPr(MB);
      if(*B>*(B+K-1))
	{
	  InterfError("mfdls1d: The input box sizes vector: B must be increasing\n");
	  return;
	}
    }
  else
    { 
      register int k=0;
      int b_ini=1,b_fin=33;   
      K=(b_fin-b_ini)/2+1;
      /* dynamic memory allocation */
      if((B=(double*)malloc((unsigned int)(K*sizeof(double))))==NULL)
	{
	  InterfError("mfdls1d: malloc error\n");
	  return;
	}
      for (k=0;k<K;k++)
	*(B+k)=b_ini+k*2;
    }

  if(Interf.NbParamIn>5) 
    { 
      /* Get the dimension and verify it */
      Mdim=Interf.Param[5];
      if (!MatrixIsString(Mdim)) 
	{
	  InterfError("mfdls1d: the input capacity: capa must be a string\n");
	  MatrixTranspose(Mc1d);
	  return;
	}

      /* Get the input argument */
      if(MFAD_dimstr(MatrixReadString(Mdim),&t_dim)==0) 
	{
	  InterfError("mfdls1d: invalid input capacity\n");
	  return;
	}
    } 
  /* else t_dim=MFAH_BOX */

  if(Interf.NbParamIn>6) 
    { 
      /* Get the input reference image: mu1d */
      Mmu1d=Interf.Param[6];
      if(!MatrixIsNumeric(Mmu1d)) 
	{
	  InterfError("mfdls1d: The input reference image: mu1d must be a numeric matrix\n");
	  return;
	}
    
      if(!MatrixIsReal(Mmu1d)) 
	{
	  InterfError("mfdls1d: The input reference image: mu1d must be a real matrix\n");
	  return;
	}
  
      if(Nx!=(int)MAX(MatrixGetWidth(Mmu1d),(int)MatrixGetHeight(Mmu1d)))
	{
	  InterfError("mfdls1d: The size of the input capacity: c1d and reference measure: mu1d must be equal\n");
	  return;
	}

      /* Get the input argument */  
      mu1d=MatrixGetPr(Mmu1d);
    }
  /* else mu1d=NULL */

  if(Interf.NbParamIn>7) 
    { 
      /* Get the input # of Hlder exponents: N and verify it */
      MN=Interf.Param[7];
      if(!MatrixIsNumeric(MN)) 
	{
	  InterfError("mfdls1d: The input # of Hlder exponents: N must be a numeric matrix\n");
	  return;
	}
      if(!MatrixIsReal(MN)) 
	{
	  InterfError("mfdls1d: The input # of Hlder exponents: N must be a real matrix\n");
	  return;
	}
      if(!MatrixIsScalar(MN))
	{
	  InterfError("mfdls1d: The input # of Hlder exponents: N must be a scalar\n");
	  return;
	}  

      /* Get the input argument */  
      N=(int)MatrixGetScalar(MN);
    } 
  /* else N=100 */

  /* Create the output Matrixes */ 
  Malpha=MatrixCreate(N,1,"real"); 
  alpha=MatrixGetPr(Malpha);
  Mfdls1d_alpha=MatrixCreate(N,1,"real"); 
  fdls1d_alpha=MatrixGetPr(Mfdls1d_alpha);
  
  /* Transpose input Matrixes */
  MatrixTranspose(Mc1d);
 
  if(Interf.NbParamIn>6) 
    MatrixTranspose(Mmu1d);

  /* Call of the C function */
  if(MFAH_mfdls1d(Nx,c1d,mu1d,n_ini,n_ini,t_capa,epsilon,K,B,t_dim,N,alpha,fdls1d_alpha)==0)
    { 
      InterfError("mfdls1d: Call of the C function MFAH_mfdls1d failed\n"); 
      return; 
    }

  /* Transpose input Matrixes */
  MatrixTranspose(Mc1d);
  
  if(Interf.NbParamIn>6)
    MatrixTranspose(Mmu1d);

  /* dynamic memory deallocation */
  if (Interf.NbParamIn<=3) 
    free((char*)B);

  /* Return the output */  
  ReturnParam(Malpha); 
  ReturnParam(Mfdls1d_alpha);

}

