/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/FRACLAB-WKSP/FRACLAB/MFAH/SRC/LAB_mph1d.c
* File type        : C source file
* Revision         : $Revision$
* RCS status       : $Id$
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sun Sep  1 14:52:50 2002
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Sun Sep 15 16:15:06 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 2002 */

#include "C_LAB_Interf.h"
#include "MFAH_hoelder1d.h"

void LAB_mph1d()
{ 
  /* input capacity and reference measure */
  int Nx=0;  
  double* c1d=NULL;
  double* mu1d=NULL; 
  /* Pointwise hoelder exponents estimation parameters */ 
  int J=0;
  double* S=NULL; 
  MFAMt_capa t_capa=MFAM_SUM;
  /* lp measure parameter */
  double P=2.;
  /* iso capacity parameter */
  int Q=50;
 
  /* ouput hoelder exponents */
  double* p_alpha1d=NULL;

  /* Matrix */
  Matrix* Mc1d; 
  Matrix* Mmu1d;
  Matrix* MS;
  Matrix* Mcapa;
  Matrix* Mp_alpha1d;

  /* Check the input arguments number */
  if(Interf.NbParamIn<1) 
  {
    InterfError("mph1d: You must give at least 1 input argument\n");
    return;
  }  
  if(Interf.NbParamIn>4) 
  {
    InterfError("mph1d: You must give at most 4 input arguments\n");
    return;
  }

  /* Check the output arguments number */ 
  if(Interf.NbParamOut!=1) 
  {
    InterfError("mph1d: You must give 1 output argument\n");
    return;
  }

  /* Get the input capacity: c1d and verify it */
  Mc1d=Interf.Param[0];
  if(!MatrixIsNumeric(Mc1d)) 
  {
    InterfError("mph1d: The input capacity: c1d must be a numeric matrix\n");
    return;
  }

  if(!MatrixIsReal(Mc1d)) 
  {
    InterfError("mph1d: The input capacity: c1d must be a real matrix\n");
    return;
  }

  /* Get the input argument */  
  c1d=(double*)MatrixGetPr(Mc1d);
 
  /* Get the size of the input argument */ 
  Nx=(int)MatrixGetWidth(Mc1d);
  if (Nx<=0)
  {
    InterfError("mph1d: The size of the input capacity: c1d must be >0\n");
    return;
  }

  if(Interf.NbParamIn>1) 
  {
    /* Get the capacity and verify it */
    Mcapa=Interf.Param[1];
    if(!MatrixIsString(Mcapa)) 
    {
      InterfError("mph1d: the input capacity: capa must be a string\n");
      return;
    }

    /* Get the input argument */
    if(MFAM_capastr(MatrixReadString(Mcapa),&t_capa)==0) 
      {
	InterfError("mph1d: invalid input capacity\n");
	return;
      }
  }
  /* else t_capa=MFAM_SUM */

  if(Interf.NbParamIn>2) 
  {
    /* Get the input sizes: S and verify it */
    MS=Interf.Param[2];
    if(!MatrixIsNumeric(MS))
    {
      InterfError("mph1d: The input sizes: S must be a numeric matrix\n");
      return;
    }
    if(!MatrixIsReal(MS))
    {
      InterfError("mph1d: The input sizes: S must be a real matrix\n");
      return;
    }
    if(((int)MIN(MatrixGetWidth(MS),MatrixGetHeight(MS)))!=1)
    {
      InterfError("mph1d: The input sizes: S must be a vector\n");
      return;
    } 
    /* Get the size of the input argument */ 
    J=(int)MAX(MatrixGetWidth(MS),MatrixGetHeight(MS)); 
    if(J<1)
    {
      InterfError("mph1d: The length of the input sizes vector: S must be >0\n");
      return;
    } 
    /* Get the input argument */
    S=MatrixGetPr(MS);
    if(*S>*(S+J-1))
    {
      InterfError("mph1d: The input sizes vector: S must be increasing\n");
      return;
    }
  }
  else
    { 
      register int j=0;
      int s_ini=1,s_fin=5;   
      J=(s_fin-s_ini)/2+1; 
      /* dynamic memory allocation */
      if((S=(double*)malloc((unsigned int)(J*sizeof(double))))==NULL)
	{
	  InterfError("mfd1d: malloc error\n");
	  return;
	}
      for (j=0;j<J;j++)
	*(S+j)=s_ini+j*2;
    }

  if(Interf.NbParamIn>3) 
  { 
    /* Get the input reference measure: mu1d */
    Mmu1d=Interf.Param[3];
    if(!MatrixIsNumeric(Mmu1d)) 
    {
      InterfError("mph1d: The input reference measure: mu1d must be a numeric matrix\n");
      return;
    }
    
    if(!MatrixIsReal(Mmu1d)) 
    {
      InterfError("mph1d: The input reference measure: mu1d must be a real matrix\n");
      return;
    }
  
    if (Nx!=(int)MatrixGetWidth(Mmu1d))
    {
      InterfError("mph1d: The size of the input capacity: c1d and reference measure: mu1d must be =\n");
      return;
    }

    /* Get the input argument */  
    mu1d=MatrixGetPr(Mmu1d);
  }
  /* else mu2d=NULL */

  /* Create the output Matrix */  
  Mp_alpha1d=MatrixCreate(Nx,1,"real"); 
  p_alpha1d=MatrixGetPr(Mp_alpha1d);
 
  /* Transpose Matrix */
  MatrixTranspose(Mc1d);
  if(Interf.NbParamIn>3) 
    MatrixTranspose(Mmu1d); 
  
  /* Call of the C function */
  if(MFAH_mph1d(Nx,c1d,mu1d,J,S,t_capa,P,Q,p_alpha1d)==0)
  { 
    InterfError("mph1d: Call of the C function MFAH_mph1d failed\n");
    return; 
  }

  /* dynamic memory deallocation */
  if (Interf.NbParamIn<=2) 
    free((char*)S); 

  /* Transpose input Matrixes */
  MatrixTranspose(Mc1d);
  if(Interf.NbParamIn>3) 
    MatrixTranspose(Mmu1d);

  /* Transpose output Matrix */
  MatrixTranspose(Mp_alpha1d);
  
  /* Return the output */
  ReturnParam(Mp_alpha1d);
}
