/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/FRACLAB-WKSP/FRACLAB/MFAD/SRC/LAB_sboxdim2d.c
* File type        : C source file
* Revision         : $Revision$
* RCS status       : $Id$
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sun Feb  7 16:53:00 1999
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Wed Sep 11 14:05:00 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 1998 */

#include "C_LAB_Interf.h"
#include "MFAD_box.h"

void LAB_sboxdim2d()
{
  register int k=0;
  /* input parameters */
  int K=1;
  double* B=NULL;
  /* input set2d image */
  int Nx=0,Ny=0;
  unsigned char* ucset2d=0;
  double* set2d=NULL;
  /* ouput sboxdim2d */
  double* d_box2d=NULL;
  double* log_epsilon=NULL;
  double* log_N_epsilon=NULL;

  /* Matrix */
  Matrix* Mset2d;
  Matrix* MB;
  Matrix* Mlog_epsilon;
  Matrix* Mlog_N_epsilon;
  Matrix* Md_box2d; 

  /* Check the input arguments number */
  if(Interf.NbParamIn!=2) 
  {
    InterfError("sboxdim2d: You must 2 input arguments\n");
    return;
  }   
  
  /* Check the output arguments number */ 
  if(Interf.NbParamOut<1) 
  {
    InterfError("sboxdim2d: You must give at least 1 output argument\n");
    return;
  }

  if(Interf.NbParamOut==2) 
  {
    InterfError("sboxdim2d: You can give 2 output arguments\n");
    return;
  }

  if(Interf.NbParamOut>3) 
  {
    InterfError("sboxdim2d: You must give at most 3 output arguments\n");
    return;
  }

  /* Get the input 2d set image: set2d and verify it */
  Mset2d=Interf.Param[0];
  if(!MatrixIsNumeric(Mset2d)) 
  {
    InterfError("sboxdim2d : The input 2d set image: set2d must be a numeric matrix\n");
    return;
  }

  if(!MatrixIsReal(Mset2d)) 
  {
    InterfError("sboxdim2d : The input 2d set image: set2d must be a real matrix\n");
    return;
  }

  /* Get the input argument */  
  set2d=(double*)MatrixGetPr(Mset2d);
 
  /* Get the size of the input argument */ 
  Nx=(int)MatrixGetWidth(Mset2d);
  Ny=(int)MatrixGetHeight(Mset2d);
  if ((Nx<=0)||(Ny<=0))
  {
    InterfError("sboxdim2d : The size (width,height) of the input 2d set image: set2d must be >0\n");
    return;
  }

  /* Get the input spectrum graph abscissa: alpha and verify it */
  MB=Interf.Param[1];
  if(!MatrixIsNumeric(MB))
  {
    InterfError("sboxdim2d: The input spectrum graph abscissa: B must be a numeric matrix\n");
    return;
  }

  if(!MatrixIsReal(MB))
  {
    InterfError("sboxdim2d: The input spectrum graph abscissa: B must be a real matrix\n");
    return;
  } 

  if(((int)MIN(MatrixGetWidth(MB),MatrixGetHeight(MB)))!=1)
  {
    InterfError("sboxdim2d: The input spectrum graph abscissa: B must be a vector\n");
    return;
  } 

  /* Get the size of the input argument */ 
  K=(int)MAX(MatrixGetWidth(MB),MatrixGetHeight(MB)); 
  if(K<1)
  {
    InterfError("sboxdim2d: The length of the input spectrum graph abscissa: B must be >0\n");
    return;
  }
 
  /* Get the input argument */  
  B=(double*)MatrixGetPr(MB);
 
  /* Create the outputs */  
  Md_box2d=MatrixCreate(1,1,"real"); 
  d_box2d=MatrixGetPr(Md_box2d);
  Mlog_epsilon=MatrixCreate(1,K,"real"); 
  log_epsilon=MatrixGetPr(Mlog_epsilon);
  Mlog_N_epsilon=MatrixCreate(1,K,"real"); 
  log_N_epsilon=MatrixGetPr(Mlog_N_epsilon);
 
  /* Transpose input Matrix */
  MatrixTranspose(Mset2d);
  
  /* memory allocation */
  if((ucset2d=(unsigned char*)malloc((unsigned int)(Nx*Ny*sizeof(unsigned char))))==NULL)
  {
    InterfError("sboxdim2d: malloc error\n");
    return;
  }
  for(k=0;k<Nx*Ny;k++)
    *(ucset2d+k)=(unsigned char)*(set2d+k);
	  
  /* Call of the C function */
  if(MFAD_box2d(Nx,Ny,ucset2d,K,B,log_epsilon,log_N_epsilon,d_box2d)==0)
  { 
    InterfError("sboxdim2d: Call of the C function MFAD_box2d failed\n"); 
    return; 
  }
  
  /* memory de-allocation */
  free((char*)ucset2d);

  /* Transpose input Matrix */
  MatrixTranspose(Mset2d);
 
  /* Return the output */
  ReturnParam(Md_box2d);
  if(Interf.NbParamOut>2)
  {
    ReturnParam(Mlog_epsilon);
    ReturnParam(Mlog_N_epsilon);
  }
  else
  {
    MatrixFree(Mlog_epsilon);
    MatrixFree(Mlog_N_epsilon);
  }
}
