/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/MFA.CMP/MFAD.CL/src/MFAD_box.c
* File type        : C source file
* Revision         : $Revision: 1.1.1.1 $
* RCS status       : $Id: MFAD_box.c,v 1.1.1.1 1999/02/10 13:38:21 guiheneu Exp $
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sun Feb  7 14:46:45 1999
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Sun Aug 25 13:25:37 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 1996-98 */

#include "MFAD_box.h"

/*****************************************************************************/

double MFAD_box(unsigned char* set,unsigned N,unsigned n_min,unsigned n_max,double v_e)
{   
  register int i=0;
  int n=n_min; 
  double* log_omega=NULL;   
  double* minus_n_log_two=NULL; 
  double* e_y=NULL;  
  double d=0.;
  int two_pow_N=(int)pow(2.,(double)N);   
  log_omega=(double*)malloc(sizeof(double)*(n_max-n_min+1));   
  minus_n_log_two=(double*)malloc(sizeof(double)*(n_max-n_min+1));   
  for(i=0;i<n_max-n_min+1;i++)
    {         
      int x=0;        
      int omega=0;	       
      int two_pow_n=(int)pow(2.,(double)n);      
      while(x<=two_pow_N-two_pow_n)      
	{	 
	  register int k=0;	 
	  for(k=0;(k<two_pow_n)&&(*(set+x+k)!=1);k++);     
	  if (k!=two_pow_n)	    
	    omega++;	 
	  x+=two_pow_n;      
	}       
      *(minus_n_log_two+i)=-(double)n*log(2.);
      if (omega!=0)	 
	*(log_omega+i)=log((double)omega); 
      else
	*(log_omega+i)=0.;
      n++;   
    }
  d=MFAD_slope(minus_n_log_two,log_omega,n_max-n_min+1,v_e,&e_y); 
#ifdef MFADDEBUG 
 {     
    FILE* fp=NULL;
    char* name=NULL;
    const char* cfname="box.%d.%d.%d.xg",*cname1="cluster.%d.%d.%d",*cname2="linear.%d.%d.%d";
    name=(char*)malloc((unsigned)(strlen(cfname)+4)*sizeof(char));
    sprintf(name,cfname,1,n_max-n_min+1,N);	
    if ((fp=fopen(name,"w"))==NULL)
      {
	fprintf(stderr,"fopen error\n");
	exit(1);
      }  
    realloc(name,(unsigned)(strlen(cname1)+4)*sizeof(char));
    sprintf(name,cname1,1,n_max-n_min+1,N);	
    fprintf(fp,"\"%s\n",name);  
    for(i=0;i<n_max-n_min+1;i++)
      fprintf(fp,"%f\t%f\n",*(minus_n_log_two+i),*(log_omega+i));
    fprintf(fp,"\n");
    realloc(name,(unsigned)(strlen(cname2)+4)*sizeof(char));
    sprintf(name,cname2,1,n_max-n_min+1,N);	
    fprintf(fp,"\"%s\n",name);  
    for(i=0;i<n_max-n_min+1;i++)
      fprintf(fp,"%f\t%f\n",*(minus_n_log_two+i),*(e_y+i));
    if (fclose(fp)==EOF)  
      {
	fprintf(stderr,"fclose error\n");
	exit(1);
      }  
    free((char*)e_y);  
    free(name);
  }     
#endif /* MFADDEBUG */
  free((char*)minus_n_log_two);    
  free((char*)log_omega);
  return d;
}  
 
/*****************************************************************************/
	
double MFAD_slope(double* x,double* y,int n,double v_e,double* *e_y)
{       
  register int i=0;
  int flg=0;
  double m_x=0.,m_y=0.,xc=0.,sum_xc2=0.,sum_xcy=0.,a=0.,b=0.,e=0.,m_e2=0.;    
  do   
    {      
      i=0;      
      m_x=m_y=xc=sum_xc2=sum_xcy=m_e2=0.;        
      while(i<n)      
	{	
	  m_x+=*(x+i);	 
	  m_y+=*(y+i);	 
	  i++;      
	}      
      m_x/=(double)n;     
      m_y/=(double)n;       
      i=0;      
      while(i<n)      
	{	
	  xc=*(x+i)-m_x;	 
	  sum_xc2+=xc*xc;	 
	  sum_xcy+=xc**(y+i);	 
	  i++;      
	}        
      a=sum_xcy/sum_xc2;      
      b=m_y-a*m_x;
      if (flg==0)
	{
	  *e_y=(double*)malloc((unsigned)(n*sizeof(double)));  
	  i=0;
	  while(i<n)
	    {
	      *(*e_y+i)=a**(x+i)+b;
	      i++;
	    } 
	  flg++;
	}
      i=0;      
      while(i<n)          
	{	 
	  e=*(y+i)-a**(x+i)-b;	 
	  m_e2+=e*e;  	 
	  i++;            
	}             
      n--;        
    }   
  while(m_e2/(double)n>v_e);
  return a;
}  

/*****************************************************************************/
	
int MFAD_box1d(int Nx,unsigned char* set1d,int K,double* B,double* log_epsilon,double* log_N_epsilon,double* d_box1d)
{
  if((Nx>0)&&(set1d!=NULL)&&(K>0)&&(B!=NULL)&&(log_epsilon!=NULL)&&(log_N_epsilon!=NULL)&&(d_box1d!=NULL))
    {
      register int k=0;
      for(k=0;k<K;k++)
	{
	  register int x=0;
	  register int i=0;
	  unsigned int N_epsilon=0;
	  int Nx_f=Nx-(int)*(B+k);
	  short non_empty=0;
	  for(x=0;x<Nx_f;x+=(int)*(B+k))
	    {
	      for(i=0;(i<(int)*(B+k))&&(x+i<Nx);i++)
		{
		  if((non_empty=*(set1d+x+i)))
		    break;
		}
	      if(non_empty)
		N_epsilon++;
	    }
	  *(log_N_epsilon+k)=(N_epsilon==0?0.:log((double)N_epsilon));
	  *(log_epsilon+k)=log(*(B+k)/(double)(Nx));
	}
      if(K>1)
	{
	  double b=0.,sigma2_e=0.;
	  if(MFAM_lslr(K,log_epsilon,log_N_epsilon,d_box1d,&b,NULL,NULL,&sigma2_e)==0)
	    {
	      fprintf(stderr,"MFAM_lslr function call error\n");
	      return 0;
	    }
	}
      else
	{
	  if(*B!=1)
	    *log_epsilon=log(*B);
	  else
	    *log_epsilon=1.; 
	  *d_box1d=*log_N_epsilon/(*log_epsilon);      
	}
      *d_box1d=(*d_box1d<0.?-(*d_box1d):0.);
      return 1;
    }
  else
    {
      fprintf(stderr,"MFAD_box1d arguments error\n"); 
      fprintf(stderr,"\tNx=[7m%d[m\n",Nx);
      fprintf(stderr,"\tset1d=[7m%p[m\n",set1d); 
      fprintf(stderr,"\tK=[7m%d[m\n",K);
      fprintf(stderr,"\tlog_epsilon=[7m%p[m\n",log_epsilon);  
      fprintf(stderr,"\tlog_N_epsilon=[7m%p[m\n",log_N_epsilon); 
      fprintf(stderr,"\td_box1d=[7m%p[m\n",d_box1d); 
      return 0;
    }
}
/*****************************************************************************/
	
int MFAD_box2d(int Nx,int Ny,unsigned char* set2d,int K,double* B,double* log_epsilon,double* log_N_epsilon,double* d_box2d)
{
  if((Nx>0)&&(Ny>0)&&(set2d!=NULL)&&(K>0)&&(B!=NULL)&&(log_epsilon!=NULL)&&(log_N_epsilon!=NULL)&&(d_box2d!=NULL))
    {
      register int k=0;
      for(k=0;k<K;k++)
	{
	  register int x=0,y=0;
	  register int i=0,j=0;
	  unsigned int N_epsilon=0;
	  int Nx_f=Nx-(int)*(B+k),Ny_f=Ny-(int)*(B+k);
	  short non_empty=0;
	  for(y=0;y<Ny_f;y+=(int)*(B+k))
	    {
	      for(x=0;x<Nx_f;x+=(int)*(B+k))
		{
		  for(j=0;(j<(int)*(B+k))&&(x+j<Ny);j++)
		    {
		      for(i=0;(i<(int)*(B+k))&&(x+i<Nx);i++)
			{
			  if((non_empty=*(set2d+Nx*(y+j)+x+i)))
			    break;
			}
		      if(non_empty)
			break;
		    }
		  if(non_empty)
		    N_epsilon++;
		}
	    }
	  *(log_N_epsilon+k)=(N_epsilon==0?0.:log((double)N_epsilon));
	  *(log_epsilon+k)=log(*(B+k)/(double)(Nx*Ny));
	}
      if(K>1)
	{
	  double b=0.,sigma2_e=0.;
	  if(MFAM_lslr(K,log_epsilon,log_N_epsilon,d_box2d,&b,NULL,NULL,&sigma2_e)==0)
	    {
	      fprintf(stderr,"MFAM_lslr function call error\n");
	      return 0;
	    }
	}
      else
	{
	  if(*B!=1)
	    *log_epsilon=log(*B);
	  else
	    *log_epsilon=1.; 
	  *d_box2d=*log_N_epsilon/(*log_epsilon);      
	}
      *d_box2d=(*d_box2d<0.?-(*d_box2d):0.);
      return 1;
    }
  else
    {
      fprintf(stderr,"MFAD_box2d arguments error\n"); 
      fprintf(stderr,"\tNx=[7m%d[m\n",Nx);  
      fprintf(stderr,"\tNx=[7m%d[m\n",Nx);
      fprintf(stderr,"\tset2d=[7m%p[m\n",set2d); 
      fprintf(stderr,"\tK=[7m%d[m\n",K);  
      fprintf(stderr,"\tB=[7m%p[m\n",B);
      fprintf(stderr,"\tlog_epsilon=[7m%p[m\n",log_epsilon);  
      fprintf(stderr,"\tlog_N_epsilon=[7m%p[m\n",log_N_epsilon); 
      fprintf(stderr,"\td_box2d=[7m%p[m\n",d_box2d); 
      return 0;
    }
}



