/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/MFA.CMP/MFAD.CL/src/MFAD_dimension.c
* File type        : C source file
* Revision         : $Revision$
* RCS status       : $Id$
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sat Aug 24 15:48:28 2002
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Sat Sep 21 14:46:29 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 2002 */

#include "MFAD_dimension.h"

/*****************************************************************************/

int MFAD_dimstr(char* dimstr,MFADt_dim* t_dim)
{
  if(strcmp(MFAD_BOXSTR,dimstr)==0)
    *t_dim=MFAD_BOX;
  else if(strcmp(MFAD_VOSSSTR,dimstr)==0)
    *t_dim=MFAD_VOSS; 
  else if(strcmp(MFAD_BRANCHINGSTR,dimstr)==0)
    *t_dim=MFAD_BRANCHING;
  else if(strcmp(MFAD_HAUSDORFFSTR,dimstr)==0)
    *t_dim=MFAD_HAUSDORFF;
  else 
    return 0;
  return 1;
}

/*****************************************************************************/

int MFAD_dimension1d(int Nx,unsigned char* set1d,int K,double* B,MFADt_dim t_dim,double* log_epsilon,double* log_N_epsilon,double* dim1d,void* tree)
{
  if((Nx>0)&&(set1d!=NULL)&&(K>0)&&(B!=NULL)&&(dim1d!=NULL))
    {
      short internal=0;  
      short n_opt=0;
      short b_opt=0;
      MFAMt_tree* dim_tree=NULL;
 
      if((t_dim==MFAD_BOX)||(t_dim==MFAD_VOSS))
	{
	  if((log_epsilon==NULL)&&(log_N_epsilon==NULL))
	    {  
	      if((log_epsilon=(double*)malloc((size_t)(K*sizeof(double))))==NULL)
		{
		  fprintf(stderr,"malloc error\n");
		  return 0;
		}
	      memset(log_epsilon,0,K*sizeof(double));
	      if((log_N_epsilon=(double*)malloc((size_t)(K*sizeof(double))))==NULL)
		{
		  fprintf(stderr,"malloc error\n");
		  return 0;
		}     
	      memset(log_N_epsilon,0,K*sizeof(double)); 
	      internal=1;
	    }
	}	  
      else if((t_dim==MFAD_BRANCHING)||(t_dim==MFAD_HAUSDORFF))
	dim_tree=MFAM_tree_s_level_init(Nx,set1d,tree,&n_opt,&b_opt);

      switch(t_dim)
	{
	case MFAD_BOX:
	  if(MFAD_box1d(Nx,set1d,K,B,log_epsilon,log_N_epsilon,dim1d)==0) 
	    { 
	      fprintf(stderr,"MFAD_box1d function call error\n"); 
	      return 0; 
	    } 
	  break;
	case MFAD_VOSS:
	  if(MFAD_voss1d(Nx,set1d,K,B,log_epsilon,log_N_epsilon,dim1d)==0) 
	    { 
	      fprintf(stderr,"MFAD_voss1d function call error\n"); 
	      return 0; 
	    } 
	  break;	
	case MFAD_BRANCHING:
	  if(MFAD_branching1d(Nx,set1d,0,0,dim1d,dim_tree,&n_opt,&b_opt)==0) 
	    { 
	      fprintf(stderr,"MFAD_branching1d function call error\n"); 
	      return 0; 
	    }
	  break;
	case MFAD_HAUSDORFF: 
	  {
	    int P=21;
	    double* p=NULL;
	    double p_ini=0.; 
	    double p_fin=1.;
	    double p_o=0.;
	    int M=10000;
	    p_o=1./(double)b_opt;
	    if((p=(double*)malloc((size_t)(P*sizeof(double))))==NULL)
	    {
	      fprintf(stderr,"malloc error\n");
	      return 0;
	    }   
	    memset(p,0,P*sizeof(double));
	    if(1)
	      p_ini=p_o;
	    (void)MFAM_linspace(P,p_ini,p_fin,p);
	    if(MFAD_hausdorff1d(Nx,set1d,0,0,M,P,p,dim1d,dim_tree,&n_opt,&b_opt)==0) 
	      { 
		fprintf(stderr,"MFAD_hausdorff1d function call error\n"); 
		return 0; 
	      }     
	    free((char*)p);
	  }
	  break;
	default:
	  return 0;
	} 
      if((t_dim==MFAD_BOX)||(t_dim==MFAD_VOSS))
	{
	  if(internal)
	    {
	      free((char*)log_epsilon);
	      free((char*)log_N_epsilon);
	    }
	}
      else if((t_dim==MFAD_BRANCHING)||(t_dim==MFAD_HAUSDORFF))
	MFAM_tree_s_level_clear(Nx,set1d,dim_tree,tree,&n_opt);
      return 1;
    }
  else
    {
      fprintf(stderr,"MFAD_dimension1d arguments error\n"); 
      fprintf(stderr,"\tNx=[7m%d[m\n",Nx);  
      fprintf(stderr,"\tset1d=[7m%p[m\n",set1d); 
      fprintf(stderr,"\tK=[7m%d[m\n",K);
      fprintf(stderr,"\tB=[7m%p[m\n",B);  
      fprintf(stderr,"\tt_dim=[7m%d[m\n",t_dim); 
      fprintf(stderr,"\tlog_epsilon=[7m%p[m\n",log_epsilon);  
      fprintf(stderr,"\tlog_N_epsilon=[7m%p[m\n",log_N_epsilon); 
      fprintf(stderr,"\tdim1d=[7m%p[m\n",dim1d); 
      fprintf(stderr,"\ttree=[7m%p[m\n",tree);
      return 0;
    }
}

/*****************************************************************************/

int MFAD_dimension2d(int Nx,int Ny,unsigned char* set2d,int K,double* B,MFADt_dim t_dim,double* log_epsilon,double* log_N_epsilon,double* dim2d,void* tree)
{
  if((Nx>0)&&(Ny>0)&&(set2d!=NULL)&&(K>0)&&(B!=NULL)&&(dim2d!=NULL))
    {   
      short internal=0; 
      short n_opt=0;
      MFAMt_qtree* dim_tree=NULL;

      if((t_dim==MFAD_BOX)||(t_dim==MFAD_VOSS))
	{
   	  if((log_epsilon==NULL)&&(log_N_epsilon==NULL))
	    {  
	      if((log_epsilon=(double*)malloc((size_t)(K*sizeof(double))))==NULL)
		{
		  fprintf(stderr,"malloc error\n");
		  return 0;
		} 
	      memset(log_epsilon,0.,K*sizeof(double));
	      if((log_N_epsilon=(double*)malloc((size_t)(K*sizeof(double))))==NULL)
		{
		  fprintf(stderr,"malloc error\n");
		  return 0;
		}     
	      memset(log_N_epsilon,0.,K*sizeof(double)); 
	      internal=1;
	    }
	}
      else if((t_dim==MFAD_BRANCHING)||(t_dim==MFAD_HAUSDORFF))
	dim_tree=MFAM_qtree_s_level_init(Nx,Ny,set2d,tree,&n_opt); 

      switch(t_dim)
	{
	case MFAD_BOX:
	  if(MFAD_box2d(Nx,Ny,set2d,K,B,log_epsilon,log_N_epsilon,dim2d)==0) 
	    { 
	      fprintf(stderr,"MFAD_box2d function call error\n"); 
	      return 0; 
	    } 
	  break;
	case MFAD_VOSS: 
	  if(MFAD_voss2d(Nx,Ny,set2d,K,B,log_epsilon,log_N_epsilon,dim2d)==0) 
	    { 
	      fprintf(stderr,"MFAD_voss2d function call error\n"); 
	      return 0; 
	    } 
	  break;
	case MFAD_BRANCHING:  
	  if(MFAD_branching2d(Nx,Ny,set2d,0,0,dim2d,dim_tree,&n_opt)==0) 
	    { 
	      fprintf(stderr,"MFAD_branching2d function call error\n"); 
	      return 0; 
	    }
	  break;
	case MFAD_HAUSDORFF:  
	  {
	    int P=21;
	    double* p=NULL;
	    double p_ini=0.; 
	    double p_fin=1.;
	    double p_o=0.;
	    int M=10000; 
	    p_o=1./(double)2.;
	    if((p=(double*)malloc((size_t)(P*sizeof(double))))==NULL)
	    {
	      fprintf(stderr,"malloc error\n");
	      return 0;
	    }   
	    memset(p,0,P*sizeof(double));
	    if(1)
	      p_ini=p_o;
	    (void)MFAM_linspace(P,p_ini,p_fin,p);
	    if(MFAD_hausdorff2d(Nx,Ny,set2d,0,0,M,P,p,dim2d,dim_tree,&n_opt)==0) 
	      { 
		fprintf(stderr,"MFAD_hausdorff2d function call error\n"); 
		return 0; 
	      } 
	    free((char*)p);
	  }
	  break;
	default:
	  return 0;
	} 
      if((t_dim==MFAD_BOX)||(t_dim==MFAD_VOSS))
	{
	  if(internal)
	    {
	      free((char*)log_epsilon);
	      free((char*)log_N_epsilon);
	    }
	} 
      else if((t_dim==MFAD_BRANCHING)||(t_dim==MFAD_HAUSDORFF))
	MFAM_qtree_s_level_clear(Nx,Ny,set2d,dim_tree,tree,&n_opt);

      return 1;
    }
  else
    {
      fprintf(stderr,"MFAD_dimension2d arguments error\n"); 
      fprintf(stderr,"\tNx=[7m%d[m\n",Nx); 
      fprintf(stderr,"\tNy=[7m%d[m\n",Ny);  
      fprintf(stderr,"\tset2d=[7m%p[m\n",set2d); 
      fprintf(stderr,"\tK=[7m%d[m\n",K);
      fprintf(stderr,"\tB=[7m%p[m\n",B);  
      fprintf(stderr,"\tt_dim=[7m%d[m\n",t_dim); 
      fprintf(stderr,"\tlog_epsilon=[7m%p[m\n",log_epsilon);  
      fprintf(stderr,"\tlog_N_epsilon=[7m%p[m\n",log_N_epsilon); 
      fprintf(stderr,"\tdim2d=[7m%p[m\n",dim2d); 
      fprintf(stderr,"\ttree=[7m%p[m\n",tree);
      return 0;
    }
}
