/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/MFA.CMP/MFAD.CL/src/MFAD_voss.c
* File type        : C source file
* Revision         : $Revision$
* RCS status       : $Id$
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Wed Sep 11 22:54:21 2002
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Sun Sep 15 15:49:18 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 2002 */

#include "MFAD_voss.h"

/*****************************************************************************/
	
int MFAD_voss1d(int Nx,unsigned char* set1d,int K,double* B,double* log_epsilon,double* log_N_epsilon,double* vossdim)
{
  if((Nx>0)&&(set1d!=NULL)&&(K>0)&&(B!=NULL)&&(log_epsilon!=NULL)&&(log_N_epsilon!=NULL)&&(vossdim!=NULL))
    {
      register int k=0; 
      int f_B=*(B+K-1);
      int N=0;
      for(k=0;k<Nx;k++)
	N+=(int)*(set1d+k);
      if(N)
	{
	  double* h_m_epsilon=NULL;
	  if((h_m_epsilon=(double*)malloc((size_t)(f_B+1)*sizeof(double)))==NULL)
	    {
	      fprintf(stderr,"malloc error\n");
	      return 0;
	    } 
	  memset(h_m_epsilon,0,(size_t)(f_B+1)*sizeof(double));
	  for(k=0;k<K;k++)
	    {
	      register int x=0;
	      register int i=0; 
	      int c_B=(int)*(B+k);
	      int m=0;
	      int m_max=c_B+1;
	      int Nx_f=Nx-c_B;
	      int sum_h_m_epsilon=0.; 
	      double N_epsilon=0; 
	      memset(h_m_epsilon,0,(size_t)(f_B+1)*sizeof(double));
	      for(x=0;x<Nx_f;x+=c_B)
		{  
		  m=0; 
		  if(*(set1d+x+(c_B-1)/2)==1)
		    {
		      for(i=0;(i<c_B)&&(x+i<Nx);i++)
			m+=(int)*(set1d+x+i);
		    }
		  if(m!=0)
		    {
		      *(h_m_epsilon+m)+=1.;
		      sum_h_m_epsilon+=m;
		    }
		}
	      if(sum_h_m_epsilon!=0)
		{
		  for(m=0;m<m_max;m++)
		    *(h_m_epsilon+m)/=(double)sum_h_m_epsilon;
		  for(m=1;m<m_max;m++)
		    N_epsilon+=N**(h_m_epsilon+m)/(double)m;
		}
	      *(log_N_epsilon+k)=(N_epsilon==0?0.:log((double)N_epsilon));
	      *(log_epsilon+k)=log((double)c_B/(double)Nx);
	    }
	  if(K>1)
	    {
	      double b=0.,sigma2_e=0.;
	      if(MFAM_lslr(K,log_epsilon,log_N_epsilon,vossdim,&b,NULL,NULL,&sigma2_e)==0)
		{
		  fprintf(stderr,"MFAM_lslr function call error\n");
		  return 0;
		}
	    }
	  else
	    {
	      if(*B!=1)
		*log_epsilon=log(*B);
	      else
		*log_epsilon=1.; 
	      *vossdim=*log_N_epsilon/(*log_epsilon);      
	    }
	  free((char*)h_m_epsilon);
	}
      *vossdim=(*vossdim<0.?-(*vossdim):0.);
      return 1;
    }
  else
    {
      fprintf(stderr,"MFAD_voss1d arguments error\n"); 
      fprintf(stderr,"\tNx=[7m%d[m\n",Nx);
      fprintf(stderr,"\tset1d=[7m%p[m\n",set1d); 
      fprintf(stderr,"\tK=[7m%d[m\n",K);  
      fprintf(stderr,"\tB=[7m%p[m\n",B);
      fprintf(stderr,"\tlog_epsilon=[7m%p[m\n",log_epsilon);  
      fprintf(stderr,"\tlog_N_epsilon=[7m%p[m\n",log_N_epsilon); 
      fprintf(stderr,"\tvossdim=[7m%p[m\n",vossdim); 
      return 0;
    }
}

/*****************************************************************************/
	
int MFAD_voss2d(int Nx,int Ny,unsigned char* set2d,int K,double* B,double* log_epsilon,double* log_N_epsilon,double* vossdim)
{
  if((Nx>0)&&(Ny>0)&&(set2d!=NULL)&&(K>0)&&(B!=NULL)&&(log_epsilon!=NULL)&&(log_N_epsilon!=NULL)&&(vossdim!=NULL))
    {
      register int k=0;
      int f_B=*(B+K-1);
      int N=0;
      for(k=0;k<Nx*Ny;k++)
	N+=(int)*(set2d+k);
      if(N)
	{
	  double* h_m_epsilon=NULL;
	  if((h_m_epsilon=(double*)malloc((size_t)(f_B*f_B+1)*sizeof(double)))==NULL)
	    {
	      fprintf(stderr,"malloc error\n");
	      return 0;
	    } 
	  memset(h_m_epsilon,0,(size_t)(f_B*f_B+1)*sizeof(double));
	  for(k=0;k<K;k++)
	    {
	      register int x=0; 
	      register int y=0;
	      register int i=0; 
	      register int j=0;
	      int c_B=(int)*(B+k);
	      int B_pow_2=c_B*c_B;
	      int m_max=B_pow_2+1;
	      int Nx_f=Nx-c_B;  
	      int Ny_f=Ny-c_B;
	      int m=0;
	      int sum_h_m_epsilon=0.; 
	      double N_epsilon=0;  
	      memset(h_m_epsilon,0,(size_t)(f_B*f_B+1)*sizeof(double)); 
	      for(y=0;y<Ny_f;y+=c_B)
		for(x=0;x<Nx_f;x+=c_B)
		  {  
		    m=0; 
		    if(*(set2d+Nx*(y+(c_B-1)/2)+x+(c_B-1)/2)==1)
		      {
			for(j=0;(j<c_B)&&(y+j<Ny);j++)
			  for(i=0;(i<c_B)&&(x+i<Nx);i++)
			    m+=(int)*(set2d+Nx*(y+j)+x+i);
		      }
		    if(m!=0)
		      {
			*(h_m_epsilon+m)+=1.;
			sum_h_m_epsilon+=m;
		      }
		  }
	      if(sum_h_m_epsilon!=0)
		{
		  for(m=0;m<m_max;m++)
		    *(h_m_epsilon+m)/=(double)sum_h_m_epsilon;
		  for(m=1;m<m_max;m++)
		    N_epsilon+=N**(h_m_epsilon+m)/(double)m;
		}
	      *(log_N_epsilon+k)=(N_epsilon==0?0.:log((double)N_epsilon));
	      *(log_epsilon+k)=log((double)c_B/(double)(Nx*Ny));
	    }
	  if(K>1)
	    {
	      double b=0.,sigma2_e=0.;
	      if(MFAM_lslr(K,log_epsilon,log_N_epsilon,vossdim,&b,NULL,NULL,&sigma2_e)==0)
		{
		  fprintf(stderr,"MFAM_lslr function call error\n");
		  return 0;
		}
	    }
	  else
	    {
	      if(*B!=1)
		*log_epsilon=log(*B);
	      else
		*log_epsilon=1.; 
	      *vossdim=*log_N_epsilon/(*log_epsilon);      
	    }
	  free((char*)h_m_epsilon);
	}
      *vossdim=(*vossdim<0.?-(*vossdim):0.);
      return 1;
    }
  else
    {
      fprintf(stderr,"MFAD_voss2d arguments error\n"); 
      fprintf(stderr,"\tNx=[7m%d[m\n",Nx);  
      fprintf(stderr,"\tNy=[7m%d[m\n",Ny);
      fprintf(stderr,"\tset2d=[7m%p[m\n",set2d); 
      fprintf(stderr,"\tK=[7m%d[m\n",K);  
      fprintf(stderr,"\tB=[7m%p[m\n",B);
      fprintf(stderr,"\tlog_epsilon=[7m%p[m\n",log_epsilon);  
      fprintf(stderr,"\tlog_N_epsilon=[7m%p[m\n",log_N_epsilon); 
      fprintf(stderr,"\tvossdim=[7m%p[m\n",vossdim); 
      return 0;
    }
}
