/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/MFA.CMP/MFAH.CL/src/MFAH_btree.c
* File type        : C source file
* Revision         : $Revision$
* RCS status       : $Id$
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sat Aug  3 19:24:35 2002
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Sat Aug 24 13:16:48 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 02 */

#include "MFAH_btree.h"

/*****************************************************************************/

int MFAH_btree_h_level_compute(MFAMt_btree *tree,short n_min,short n_max)
{
   if(tree!=NULL)
   {
      short n=n_min;
      while(n<=n_max)
      {
	 int two_pow_n=(int)pow(2.,(double)n);
	 int k=0;
	 double c_n_k=0.;
	 double alpha_n_k=0.;
#ifdef MFAHDEBUG
	 fprintf(stderr,"computing hoelder exponents of level # %d\n",n);
#endif /* MFAHDEBUG */
	 while(k<two_pow_n)
	 {  
	   c_n_k=*(*(tree->c_level+n)+k);
	   if((c_n_k!=0.)&&(c_n_k!=1.))
	     alpha_n_k=log(c_n_k)/(-n*log(2.));
	   else
	     alpha_n_k=0.;
	   *(*(tree->h_level+n)+k)=alpha_n_k;
	   k++;
	 }
	 n++;
      }
      return 1;
   }
   else
   {
      fprintf(stderr,"MFAH_btree_h_level_compute argument error\n");
      return 0;
   } 
}

/*****************************************************************************/

int MFAH_btree_mma_level_compute(MFAMt_btree* tree,short n_min,short n_max)
{
   if(tree!=NULL)
   {
      short n=n_min;
      while(n<=n_max)
      {
	 int two_pow_n=(int)pow(2.,(double)n);
	 int k=0;
	 double c_n_k=0.;
	 double mu_n_k=0.;
	 double alpha_n_k=0.;
#ifdef MFAHDEBUG
	 fprintf(stderr,"computing hoelder exponents of level # %d\n",n);
#endif /* MFAHDEBUG */ 
	 while(k<two_pow_n)
	 { 
	   c_n_k=*(*(tree->c_level+n)+k);
	   mu_n_k=*(*(tree->r_level+n)+k);
	   if((c_n_k!=0.)&&(c_n_k!=1.)&&(mu_n_k!=0.)&&(mu_n_k!=1.))
	     alpha_n_k=log(c_n_k)/log(mu_n_k);
	   else
	     alpha_n_k=0.;
	   *(*(tree->h_level+n)+k)=alpha_n_k;
	   k++;
	 }
	 n++;
      }
      return 1;
   }
   else
   {
      fprintf(stderr,"MFAH_btree_mma_level_compute arguments error\n");
      return 0;
   } 
}

/*****************************************************************************/

int MFAH_btree_E_epsilon_alpha(MFAMt_btree* tree,short n,double epsilon,double alpha)
{
   if((tree!=NULL)&&(n>0)&&(epsilon>0.))
   {   
     register int k=0;
     int two_pow_n=(int)pow(2.,(double)n);
     while(k<two_pow_n)
       { 
	 *(*(tree->s_level+n)+k)=(unsigned char)(fabs(*(*(tree->h_level+n)+k)-alpha)<epsilon);
	 k++;
       }
      return 1;
   }  
   else
   {
      fprintf(stderr,"MFAH_btree_E_epsilon_alpha error\n");
      return 0;
   } 
}

/*****************************************************************************/

int MFAH_btree_E_n_epsilon_alpha(MFAMt_btree* tree,short n,double epsilon,double alpha)
{
   if(tree!=NULL)
   {
      MFAH_bnode_E_n_epsilon_alpha(tree->root,tree,n,epsilon,alpha);
      return 1;
   }
   else
   {
      fprintf(stderr,"MFAH_btree_E_n_epsilon_alpha arguments error\n");
      return 0;
   }  
}

/*****************************************************************************/

int MFAH_bnode_E_n_epsilon_alpha(MFAMt_bnode* node,MFAMt_btree* tree,short n,double epsilon,double alpha)
{ 
   if(node!=NULL)
   {
      int s=0;
      int k=node->n;
#ifdef MFAHDEBUG
      MFAM_bnode_print(node,0);
#endif /* MFAHDEBUG */
      if (k<n)
      {
#ifdef MFAHDEBUG
	  fprintf(stderr,"[7mk<n[m\n");
#endif /* MFAHDEBUG */
	 MFAH_bnode_E_n_epsilon_alpha(node->lnode,tree,n,epsilon,alpha);
	 MFAH_bnode_E_n_epsilon_alpha(node->rnode,tree,n,epsilon,alpha);
      }
      else if(k>=n)
      {
	double h=0.;
	h=*(*(tree->h_level+node->n)+node->k); 
	s=(int)(fabs(h-alpha)<epsilon);
	if(s&&(k<tree->N))
	    {
	      s=MFAH_bnode_E_n_epsilon_alpha(node->lnode,tree,n,epsilon,alpha)
		&&
		MFAH_bnode_E_n_epsilon_alpha(node->rnode,tree,n,epsilon,alpha);
	    }
	if(k==n)
	  *(*(tree->s_level+node->n)+node->k)=(unsigned char)s;
      }
      return s;
   }
   else
     return 0;
}
