/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/MFA.CMP/MFAH.CL/src/MFAH_dimension.c
* File type        : C source file
* Revision         : $Revision$
* RCS status       : $Id$
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sun Feb  7 16:24:56 1999
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Fri Sep 20 17:11:04 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 1998 */

#include "MFAH_dimension.h"

/*****************************************************************************/

int MFAH_fd1d(int Nx,double* p_alpha1d,double epsilon,unsigned char* E_epsilon_alpha1d,int K,double* B,MFADt_dim t_dim,int N,double* alpha,double* fd1d_alpha)
{
  if((Nx>0)&&(p_alpha1d!=NULL)&&(epsilon>=0.)&&(E_epsilon_alpha1d!=NULL)&&(K>0)&&(B!=NULL)&&(N>0)&&(alpha!=NULL)&&(fd1d_alpha!=NULL))
    {
      register int k=0;
      double* log_epsilon=NULL;
      double* log_N_epsilon=NULL;
      short n_opt=0;
      short b_opt=0; 
      MFAMt_tree* dim_tree=NULL;
      void* keep_tree=NULL;

      if(MFAH_hoelder(Nx,p_alpha1d,N,alpha,&epsilon)==0)
	{  
	  fprintf(stderr,"MFAH_hoelder function call error\n");
	  return 0;
	}

      for(k=0;k<N;k++)
	{   
	  double alpha_min=*(alpha+k)-epsilon/2;
  	  double alpha_max=*(alpha+k)+epsilon/2;

	  if(alpha_min<0.)
	    alpha_min=0.;
 
#ifndef MFAHNOPROGESS
	  MFAM_progress(k,N);
#endif /* MFAHNOPROGESS */

	  /* segmentation */
	  if(MFAM_segmentation1d(Nx,p_alpha1d,1,&alpha_min,&alpha_max,E_epsilon_alpha1d)==0)
	    {  
	      fprintf(stderr,"MFAM_segmentation1d function call error\n");
	      return 0;
	    }

	  /* dynamic memory allocation */
	  if((t_dim==MFAD_BOX)||(t_dim==MFAD_VOSS))
	    {  
	      if(k==0)
		{
		  if((log_N_epsilon=(double*)malloc((size_t)(K*sizeof(double))))==NULL)
		    {
		      fprintf(stderr,"malloc error\n");
		      return 0;
		    }  
		  memset(log_N_epsilon,0,K*sizeof(double));
		  if((log_epsilon=(double*)malloc((size_t)(K*sizeof(double))))==NULL)
		    {
		      fprintf(stderr,"malloc error\n");
		      return 0;
		    } 
		  memset(log_N_epsilon,0,K*sizeof(double));
		}  
	    }
	  else if((t_dim==MFAD_BRANCHING)||(t_dim==MFAD_HAUSDORFF))
	    {
	      if(k==0)
		keep_tree=(void*)(dim_tree=MFAM_tree_s_level_init(Nx,E_epsilon_alpha1d,NULL,&n_opt,&b_opt)); 
	      else
		MFAM_tree_s_level_update(Nx,E_epsilon_alpha1d,dim_tree,&n_opt);
	    } 
	     
	  if(MFAM_isempty(Nx,E_epsilon_alpha1d)==0)
	    { 
	      if(MFAD_dimension1d(Nx,E_epsilon_alpha1d,K,B,t_dim,log_epsilon,log_N_epsilon,fd1d_alpha+k,keep_tree)==0)
		{
		  fprintf(stderr,"MFAD_dimension1d function call error\n");
		  return 0;
		}
	    } 

#ifdef MFAHDEBUG
	  fprintf(stderr,"alpha=%g\n",*(alpha+k));
	  fprintf(stderr,"fd1d_alpha=%g\n",*(fd1d_alpha+k));
#endif /* MFAHDEBUG */

	}
  
#ifndef MFAHNOPROGESS
      MFAM_progress(N,N);
#endif /* MFAHNOPROGESS */
     
      /* dynamic memory deallocation */
      if((t_dim==MFAD_BOX)||(t_dim==MFAD_VOSS))
	{
	  free((char*)log_N_epsilon);
	  free((char*)log_epsilon);
	}
      else if((t_dim==MFAD_BRANCHING)||(t_dim==MFAD_HAUSDORFF))
	MFAM_tree_s_level_clear(Nx,E_epsilon_alpha1d,dim_tree,NULL,&n_opt);

      /* spectrum normalization */
      if(MFAM_norm(N,fd1d_alpha,NULL,1)==0)
	{
	  fprintf(stderr,"MFAM_norm function call error\n");
	  return 0;
	} 
 
      return 1;  
    } 
  else
    {
      fprintf(stderr,"MFAH_fd1d arguments error\n");
      fprintf(stderr,"\tNx=[7m%d[m\n",Nx);
      fprintf(stderr,"\tp_alpha1d=[7m%p[m\n",p_alpha1d);  
      fprintf(stderr,"\tepsilon=[7m%f[m\n",epsilon);  
      fprintf(stderr,"\tE_epsilon_alpha1d=[7m%p[m\n",E_epsilon_alpha1d);
      fprintf(stderr,"\tK=[7m%d[m\n",K);
      fprintf(stderr,"\tB=[7m%p[m\n",B);  
      fprintf(stderr,"\tt_dim=[7m%d[m\n",t_dim);
      fprintf(stderr,"\tN=[7m%d[m\n",N);
      fprintf(stderr,"\talpha=[7m%p[m\n",alpha); 
      fprintf(stderr,"\tfd1d_alpha=[7m%p[m\n",fd1d_alpha);
      return 0;
    }
}

/*****************************************************************************/

int MFAH_fd2d(int Nx,int Ny,double* p_alpha2d,double epsilon,unsigned char* E_epsilon_alpha2d,int K,double* B,MFADt_dim t_dim,int N,double* alpha,double* fd2d_alpha)
{
  if((Nx>0)&&(Ny>0)&&(p_alpha2d!=NULL)&&(epsilon>=0.)&&(E_epsilon_alpha2d!=NULL)&&(K>0)&&(B!=NULL)&&(N>0)&&(alpha!=NULL)&&(fd2d_alpha!=NULL))
    {
      register int k=0;
      double* log_epsilon=NULL;
      double* log_N_epsilon=NULL; 
      short n_opt=0;
      MFAMt_qtree* dim_tree=NULL;
      void* keep_tree=NULL;
      
      if(MFAH_hoelder(Nx*Ny,p_alpha2d,N,alpha,&epsilon)==0)
	{  
	  fprintf(stderr,"MFAH_hoelder function call error\n");
	  return 0;
	}

      for(k=0;k<N;k++)
	{   
	  double alpha_min=*(alpha+k)-epsilon/2;
  	  double alpha_max=*(alpha+k)+epsilon/2;

	  if(alpha_min<0.)
	    alpha_min=0.;

#ifndef MFAHNOPROGESS
	  MFAM_progress(k,N);
#endif /* MFAHNOPROGESS */
	 
	  /* segmentation */
	  if(MFAM_segmentation2d(Nx,Ny,p_alpha2d,1,&alpha_min,&alpha_max,E_epsilon_alpha2d)==0)
	    { 
	      fprintf(stderr,"MFAM_segmentation2d function call error\n");
	      return 0;
	    }   

	  /* dynamic memory allocation */
	  if((t_dim==MFAD_BOX)||(t_dim==MFAD_VOSS))
	    {  
	      if(k==0)
		{
		  if((log_N_epsilon=(double*)malloc((size_t)(K*sizeof(double))))==NULL)
		    {
		      fprintf(stderr,"malloc error\n");
		      return 0;
		    }  
		  memset(log_N_epsilon,0,K*sizeof(double));
		  if((log_epsilon=(double*)malloc((size_t)(K*sizeof(double))))==NULL)
		    {
		      fprintf(stderr,"malloc error\n");
		      return 0;
		    } 
		  memset(log_N_epsilon,0,K*sizeof(double));
		}  
	    }
	  else if((t_dim==MFAD_BRANCHING)||(t_dim==MFAD_HAUSDORFF))
	    {
	      if(k==0)
		keep_tree=(void*)(dim_tree=MFAM_qtree_s_level_init(Nx,Ny,E_epsilon_alpha2d,NULL,&n_opt)); 
	      else
		MFAM_qtree_s_level_update(Nx,Ny,E_epsilon_alpha2d,dim_tree,&n_opt);
	    } 
	     
	  if(MFAM_isempty(Nx*Ny,E_epsilon_alpha2d)==0)
	    { 
	      if(MFAD_dimension2d(Nx,Ny,E_epsilon_alpha2d,K,B,t_dim,log_epsilon,log_N_epsilon,fd2d_alpha+k,keep_tree)==0)
		{
		  fprintf(stderr,"MFAD_dimension2d function call error\n");
		  return 0;
		}
	    }

#ifdef MFAHDEBUG
	  fprintf(stderr,"alpha=%g\n",*(alpha+k));
	  fprintf(stderr,"fd2d_alpha=%g\n",*(fd2d_alpha+k));
#endif /* MFAHDEBUG */

	}
  
#ifndef MFAHNOPROGESS
      MFAM_progress(N,N);
#endif /* MFAHNOPROGESS */
   
      /* dynamic memory deallocation */
      if((t_dim==MFAD_BOX)||(t_dim==MFAD_VOSS))
	{
	  free((char*)log_N_epsilon);
	  free((char*)log_epsilon);
	}
      else if((t_dim==MFAD_BRANCHING)||(t_dim==MFAD_HAUSDORFF))
	MFAM_qtree_s_level_clear(Nx,Ny,E_epsilon_alpha2d,dim_tree,NULL,&n_opt);

      /* spectrum normalization */
      if(MFAM_norm(N,fd2d_alpha,NULL,2)==0)
	{
	  fprintf(stderr,"MFAM_norm function call error\n");
	  return 0;
	}
   
      return 1;  
    } 
  else
    {
      fprintf(stderr,"MFAH_fd2d arguments error\n");
      fprintf(stderr,"\tNx=[7m%d[m\n",Nx);
      fprintf(stderr,"\tNy=[7m%d[m\n",Ny);
      fprintf(stderr,"\tp_alpha2d=[7m%p[m\n",p_alpha2d);  
      fprintf(stderr,"\tepsilon=[7m%f[m\n",epsilon);  
      fprintf(stderr,"\tE_epsilon_alpha2d=[7m%p[m\n",E_epsilon_alpha2d);
      fprintf(stderr,"\tK=[7m%d[m\n",K);
      fprintf(stderr,"\tB=[7m%p[m\n",B);  
      fprintf(stderr,"\tt_dim=[7m%d[m\n",t_dim);
      fprintf(stderr,"\tN=[7m%d[m\n",N);
      fprintf(stderr,"\talpha=[7m%p[m\n",alpha); 
      fprintf(stderr,"\tfd2d_alpha=[7m%p[m\n",fd2d_alpha);
      return 0;
    }
}

/*****************************************************************************/

int MFAH_mfd1d(int Nx,double* c1d,double* mu1d,int J,double* S,MFAMt_capa t_capa,double P,short Q,double* p_alpha1d,double epsilon,unsigned char* E_epsilon_alpha1d,int K,double* B,MFADt_dim t_dim,int N,double* alpha,double* fd1d_alpha)
{
  if((Nx>0)&&(c1d!=NULL)&&(J>0)&&(S!=NULL)&&(P>1.)&&(Q>0)&&(p_alpha1d!=NULL)&&(epsilon>=0.)&&(E_epsilon_alpha1d!=NULL)&&(K>0)&&(B!=NULL)&&(N>0)&&(alpha!=NULL)&&(fd1d_alpha!=NULL))
    {
      if(MFAH_mph1d(Nx,c1d,mu1d,J,S,t_capa,P,Q,p_alpha1d)==0)
	{
	  fprintf(stderr,"MFAH_mph1d function call error\n");
	  return 0;
	}
      if(MFAH_fd1d(Nx,p_alpha1d,epsilon,E_epsilon_alpha1d,K,B,t_dim,N,alpha,fd1d_alpha)==0)
	{
	  fprintf(stderr,"MFAH_fd1d function call eerror\n");
	  return 0;
	}
      return 1;  
    } 
  else
    {
      fprintf(stderr,"MFAH_mfd1d arguments error\n"); 
      fprintf(stderr,"\tNx=[7m%d[m\n",Nx); 
      fprintf(stderr,"\tc1d=[7m%p[m\n",c1d); 
      fprintf(stderr,"\tmu1d=[7m%p[m\n",mu1d); 
      fprintf(stderr,"\tJ=[7m%d[m\n",J);
      fprintf(stderr,"\tS=[7m%p[m\n",S);
      fprintf(stderr,"\tt_capa=[7m%d[m\n",t_capa);  
      fprintf(stderr,"\tP=[7m%f[m\n",P);  
      fprintf(stderr,"\tQ=[7m%d[m\n",Q);
      fprintf(stderr,"\tp_alpha1d=[7m%p[m\n",p_alpha1d); 
      fprintf(stderr,"\tepsilon=[7m%f[m\n",epsilon); 
      fprintf(stderr,"\tE_epsilon_alpha1d=[7m%p[m\n",E_epsilon_alpha1d);
      fprintf(stderr,"\tK=[7m%d[m\n",K);
      fprintf(stderr,"\tB=[7m%p[m\n",B);
      fprintf(stderr,"\tt_dim=[7m%d[m\n",t_dim);
      fprintf(stderr,"\tN=[7m%d[m\n",N);
      fprintf(stderr,"\talpha=[7m%p[m\n",alpha); 
      fprintf(stderr,"\tfd1d_alpha=[7m%p[m\n",fd1d_alpha);
      return 0;
    }
}


/*****************************************************************************/

int MFAH_mfd2d(int Nx,int Ny,double* c2d,double* mu2d,int J,double* S,MFAMt_capa t_capa,double P,short Q,double* p_alpha2d,double epsilon,unsigned char* E_epsilon_alpha2d,int K,double* B,MFADt_dim t_dim,int N,double* alpha,double* fd2d_alpha)
{
  if((Nx>0)&&(Ny>0)&&(c2d!=NULL)&&(J>0)&&(S!=NULL)&&(P>1.)&&(Q>0)&&(p_alpha2d!=NULL)&&(epsilon>=0.)&&(E_epsilon_alpha2d!=NULL)&&(K>0)&&(B!=NULL)&&(N>0)&&(alpha!=NULL)&&(fd2d_alpha!=NULL))
    {
      if(MFAH_mph2d(Nx,Ny,c2d,mu2d,J,S,t_capa,P,Q,p_alpha2d)==0)
	{
	  fprintf(stderr,"MFAH_mph2d function call error\n");
	  return 0;
	}
      if(MFAH_fd2d(Nx,Ny,p_alpha2d,epsilon,E_epsilon_alpha2d,K,B,t_dim,N,alpha,fd2d_alpha)==0)
	{
	  fprintf(stderr,"MFAH_fd2d function call eerror\n");
	  return 0;
	}
      return 1;  
    } 
  else
    {
      fprintf(stderr,"MFAH_mfd2d arguments error\n"); 
      fprintf(stderr,"\tNx=[7m%d[m\n",Nx);
      fprintf(stderr,"\tNy=[7m%d[m\n",Ny);  
      fprintf(stderr,"\tc2d=[7m%p[m\n",c2d); 
      fprintf(stderr,"\tmu2d=[7m%p[m\n",mu2d); 
      fprintf(stderr,"\tJ=[7m%d[m\n",J);
      fprintf(stderr,"\tS=[7m%p[m\n",S);
      fprintf(stderr,"\tt_capa=[7m%d[m\n",t_capa);  
      fprintf(stderr,"\tP=[7m%f[m\n",P);  
      fprintf(stderr,"\tQ=[7m%d[m\n",Q);
      fprintf(stderr,"\tp_alpha2d=[7m%p[m\n",p_alpha2d); 
      fprintf(stderr,"\tepsilon=[7m%f[m\n",epsilon); 
      fprintf(stderr,"\tE_epsilon_alpha2d=[7m%p[m\n",E_epsilon_alpha2d);
      fprintf(stderr,"\tK=[7m%d[m\n",K);
      fprintf(stderr,"\tB=[7m%p[m\n",B);
      fprintf(stderr,"\tt_dim=[7m%d[m\n",t_dim);
      fprintf(stderr,"\tN=[7m%d[m\n",N);
      fprintf(stderr,"\talpha=[7m%p[m\n",alpha); 
      fprintf(stderr,"\tfd2d_alpha=[7m%p[m\n",fd2d_alpha);
      return 0;
    }
}

