/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/MFA.CMP/MFAH.CL/src/MFAH_hoelder1d.c
* File type        : C source file
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sat Aug 17 19:20:59 2002
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Sun Sep 15 20:09:20 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 2002 */

#include "MFAH_hoelder1d.h"

extern double norm_meas;
extern double p;
extern double c_ref;
extern double q;
extern double sigma2;

double i_c_delta_1d;
double norm_mu_1d;
double norm_c_1d;

/*****************************************************************************/

int MFAH_capa1d(double* c1d,short J,double* S,double (*capa)(double,double),double* log_c_delta)
{
  if((c1d!=NULL)&&(J>0)&&(S!=NULL)&&(capa!=NULL)&&(log_c_delta!=NULL))
    {
      register int k=0,j=0;
      int xc=0,xs=0,s=0;
      double c_delta=i_c_delta_1d;
      xc=((int)*(S+J-1)-1)/2;
      c_ref=*(c1d+xc);
      xs=((int)(*(S+J-1)-*S+1))/2;
      for(k=0;k<(int)*S;k++)
	c_delta=capa(c_delta,*(c1d+xs+k));
      *log_c_delta=(c_delta!=0.?log(c_delta):0.);
      j=1;
      for(s=(int)*S+2;s<=(int)*(S+J-1);s+=2)
	{ 
	  xs=((int)*(S+J-1)-s)/2;
	  c_delta=capa(c_delta,*(c1d+xs));
	  xs+=s-1;
	  c_delta=capa(c_delta,*(c1d+xs));
	  if(s==*(S+j))
	    {
	      *(log_c_delta+j)=(c_delta!=0.?log(c_delta):0.);
	      j++;
	    }
	}
      return 1;
    }
  else
    {  
      fprintf(stderr,"MFAH_capa1d arguments error\n");
      fprintf(stderr,"\tc1d=[7m%p[m\n",c1d);
      fprintf(stderr,"\tJ=[7m%d[m\n",J);
      fprintf(stderr,"\tS=[7m%p[m\n",S);
      fprintf(stderr,"\tcapa=[7m%p[m\n",capa);
      fprintf(stderr,"\tlog_c_delta=[7m%p[m\n",log_c_delta);
      return 0;
    }
}

/*****************************************************************************/

int MFAH_mma1d(double* c1d,double* mu1d,short J,double* S,double (*capa)(double,double),double* log_c_delta,double* log_mu_delta)
{
  if((c1d!=NULL)&&(mu1d!=NULL)&&(J>0)&&(S!=NULL)&&(capa!=NULL)&&(log_c_delta!=NULL)&&(log_mu_delta!=NULL))
    {
      register int k=0,j=0;
      int xc=0,xs=0,s=0;
      double c_delta=i_c_delta_1d,mu_delta=0.;
      xc=((int)*(S+J-1)-1)/2;
      c_ref=*(c1d+xc);
      xs=((int)(*(S+J-1)-*S+1))/2;
      for(k=0;k<(int)*S;k++)
	{
	  norm_meas=norm_mu_1d;
	  mu_delta=MFAM_sum(mu_delta,*(mu1d+xs+k)); 
	  norm_meas=norm_c_1d;
	  c_delta=capa(c_delta,*(c1d+xs+k));
	}
      *log_mu_delta=(mu_delta!=0.?log(mu_delta):0.);
      *log_c_delta=(c_delta!=0.?log(c_delta):0.);
      j=1;
      for(s=(int)*S+2;s<=(int)*(S+J-1);s+=2)
	{ 
	  xs=((int)*(S+J-1)-s)/2;
	  norm_meas=norm_mu_1d;
	  mu_delta=MFAM_sum(mu_delta,*(mu1d+xs));
	  norm_meas=norm_c_1d;
	  c_delta=capa(c_delta,*(c1d+xs));
	  xs+=s-1;
	  norm_meas=norm_mu_1d;
	  mu_delta=MFAM_sum(mu_delta,*(mu1d+xs)); 
	  norm_meas=norm_c_1d;
	  c_delta=capa(c_delta,*(c1d+xs));
	  if(s==*(S+j))
	    {
	      if(capa==MFAM_sum)
		{
		  *log_mu_delta=(mu_delta!=0.?log(mu_delta):0.);
		  *log_c_delta=(c_delta!=0.?log(c_delta):0.);
		}
	      else
		{
		  *(log_mu_delta+j)=(mu_delta!=0.?log(mu_delta):0.);
		  *(log_c_delta+j)=(c_delta!=0.?log(c_delta):0.);
		}
	      j++;
	    }
	}
      return 1;
    }
  else
    {  
      fprintf(stderr,"MFAH_mma1d arguments error\n");
      fprintf(stderr,"\tc1d=[7m%p[m\n",c1d); 
      fprintf(stderr,"\tmu1d=[7m%p[m\n",mu1d);
      fprintf(stderr,"\tJ=[7m%d[m\n",J);
      fprintf(stderr,"\tS=[7m%p[m\n",S);
      fprintf(stderr,"\tcapa=[7m%p[m\n",capa);
      fprintf(stderr,"\tlog_c_delta=[7m%p[m\n",log_c_delta); 
      fprintf(stderr,"\tlog_mu_delta=[7m%p[m\n",log_mu_delta);
      return 0;
    }
}

/*****************************************************************************/

int MFAH_mph1d(int Nx,double* c1d,double* mu1d,int J,double* S,MFAMt_capa t_capa,double P,short Q,double* p_alpha1d)
{ 
  if((Nx>0)&&(c1d!=NULL)&&(J>0)&&(S!=NULL)&&(P>1.)&&(Q>0)&&(p_alpha1d!=NULL))
    {
      register int x=0;
      int Nx_f=Nx-(int)*(S+J-1)+1;
      double p_alpha=0.;
      double* log_c_delta=NULL;
      double* log_mu_delta=NULL;
      double (*capa)(double,double);
      switch(t_capa)
	{
	case MFAM_SUM:
	  i_c_delta_1d=0.;
	  norm_c_1d=0.;
	  for(x=0;x<Nx;x++) 
	    norm_c_1d+=*(c1d+x);
	  capa=MFAM_sum;
	  break;
	case MFAM_LPSUM:
	  i_c_delta_1d=0.;
	  norm_c_1d=0.;
	  for(x=0;x<Nx;x++) 
	    norm_c_1d+=*(c1d+x);
	  p=P;
	  capa=MFAM_lpsum;
	  break; 
	case MFAM_MIN:
	  i_c_delta_1d=HUGE;
	  capa=MFAM_min;
	  break; 
	case MFAM_MAX:
	  i_c_delta_1d=-HUGE;
	  capa=MFAM_max;
	  break;
	case MFAM_ISO:
	  {
	    double c_min=0.,c_max=-HUGE;
	    i_c_delta_1d=0.;
	    if(MFAM_min_max(Nx,c1d,&c_min,&c_max)==0)
	      { 
		fprintf(stderr,"MFAM_min_max error\n"); 
		return 0; 
	      }
	    q=(c_max-c_min)/Q;
	    capa=MFAM_iso;
	    break;
	  }
	case MFAM_ADAPISO:
	  i_c_delta_1d=0.;
	  if((sigma2=MFAM_variance(Nx,c1d,MFAM_mean(Nx,c1d)))==HUGE)
	    {
	      fprintf(stderr,"MFAM_variance error\n"); 
	      return 0; 
	    }
	  capa=MFAM_adapiso;
	  break; 
	default:
	  return 0;
	} 
      if((log_mu_delta=(double*)malloc((size_t)(J*sizeof(double))))==0)
	{
	  fprintf(stderr,"malloc error\n");
	  return 0;
	}
      memset(log_mu_delta,0,J*sizeof(double));

      if((log_c_delta=(double*)malloc((size_t)(J*sizeof(double))))==0)
	{
	  fprintf(stderr,"malloc error\n");
	  return 0;
	}
      memset(log_c_delta,0,J*sizeof(double));

      if(mu1d==NULL)
	{
	  for(x=0;x<J;x++) 
	    {
	      if(*(S+x)<=0)
		*(log_mu_delta+x)=0;
	      else
		*(log_mu_delta+x)=log(*(S+x));
	    }
	  norm_meas=norm_c_1d;
	}
      else
	{
	  norm_mu_1d=0.;
	  for(x=0;x<Nx;x++) 
	    norm_mu_1d+=*(mu1d+x);
	}

      for(x=0;x<Nx_f;x++)
	{ 
	  p_alpha=0;
	  if(mu1d==NULL)
	    {
	      if(MFAH_capa1d(c1d+x,J,S,capa,log_c_delta)==0)
		{
		  fprintf(stderr,"MFAH_capa1d error\n");
		  return 0;
		}
	    }
	  else
	    { 
	      if(MFAH_mma1d(c1d+x,mu1d+x,J,S,capa,log_c_delta,log_mu_delta)==0)
		{
		  fprintf(stderr,"MFAH_mma1d error\n");
		  return 0;
		}
	    }
	  if(J>1)
	    {
	      double b=0.,sigma2_e=0.;   
	      if(MFAM_lslr(J,log_mu_delta,log_c_delta,&p_alpha,&b,NULL,NULL,&sigma2_e)==0)
		{
		  fprintf(stderr,"MFAM_lslr error\n");
		  return 0;
		} 
#ifdef MFAHDEBUG   
	      {
		register int j=0;
		for(j=0;j<J;j++)
		  fprintf(stderr,"%d %f %f\n",j,*(log_mu_delta+j),*(log_c_delta+j));
		fprintf(stderr,"alpha=%f\n",p_alpha);
	      }
#endif /* MFAHDEBUG */
	   
	    }                       
	  else /* J==1 */            
	    {
	      if(*log_c_delta<0.) 
		{
		  if(mu1d==NULL)
		    *log_mu_delta=-log((double)(Nx));
		}
	      else /* *log_c_delta>=0. */
		{
		  if(*S!=1) 
		    {
		      if(*S<=0) 
			*log_mu_delta=0;
		      else
			*log_mu_delta=log(*S);
		    }
		  else
		    *log_mu_delta=1.; 
		}

	      if(*log_mu_delta==0) 
		*log_mu_delta=0.00001;
	      p_alpha=*log_c_delta/(*log_mu_delta);
	    }
	  *(p_alpha1d+x+((int)*(S+J-1)-1)/2)=(((t_capa==MFAM_MIN)||(t_capa==MFAM_MAX))?fabs(p_alpha):p_alpha);
	}  
      free((char*)log_mu_delta);
      free((char*)log_c_delta);
      return 1;
    }
  else
    {
      fprintf(stderr,"MFAH_mph1d arguments error\n"); 
      fprintf(stderr,"\tNx=[7m%d[m\n",Nx); 
      fprintf(stderr,"\tc1d=[7m%p[m\n",c1d); 
      fprintf(stderr,"\tJ=[7m%d[m\n",J);
      fprintf(stderr,"\tS=[7m%p[m\n",S); 
      fprintf(stderr,"\tt_capa=[7m%d[m\n",t_capa);
      fprintf(stderr,"\tP=[7m%f[m\n",P);  
      fprintf(stderr,"\tQ=[7m%d[m\n",Q);
      fprintf(stderr,"\tp_alpha1d=[7m%p[m\n",p_alpha1d);
      return 0;
    }
} 
