/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/MFA.CMP/MFAH.CL/src/MFAH_mfdlimsup2d.c
* File type        : C source file
* Revision         : $Revision$
* RCS status       : $Id$
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Mon Apr  1 17:11:04 2002
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Wed Oct 30 15:40:17 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

#include "MFAH_mfdlimsup2d.h"

/*****************************************************************************/

int MFAH_mfdls2d(int Nx,int Ny,double* c2d,double* mu2d,short n_ini,short n_fin,MFAMt_capa t_capa,double epsilon,int K,double* B,MFADt_dim t_dim,int N,double* alpha,double* fdls2d_alpha)
{
  if((Nx>0)&&(Ny>0)&&(c2d!=NULL)&&(n_ini>=0)&&(n_fin>=0)&&(t_capa<6)&&(epsilon>=0.)&&(K>0)&&(B!=NULL)&&(t_dim<4)&&(N>0)&&(alpha!=NULL)&(fdls2d_alpha!=NULL))
    {
      register int k=0;
      short n=0;
      short n_opt=0;
      int two_pow_n=0;
      int four_pow_n=0; 
      double d_E_n_epsilon_alpha=0.;
      double sup_d_E_n_epsilon_alpha=0;
      double* log_epsilon=NULL;
      double* log_N_epsilon=NULL;
      double* alpha_n_i_j=NULL; 
      unsigned char* E_n_epsilon_alpha=NULL;
      unsigned char* sup_E_n_epsilon_alpha=NULL;

      MFAMt_qtree* tree=0;
      MFAMt_qtree* dim_tree=NULL;
 
      n_opt=(short)ceil(log((double)MAX(Nx,Ny))/log(2.)); 

      if(n_fin==0)
	n_fin=n_opt;
      if(n_ini==0)
	n_ini=n_opt-1;
      else if(n_ini>n_opt-1)
	{  
	  fprintf(stderr,"n_ini>n_opt-1\n");
	  fprintf(stderr,"MFAH_mfdls2d arguments error\n");
	  return 0;
	}
  
#ifdef MFAHDEBUG 
      fprintf(stderr,"Nx=%d,Ny=%d,n_opt=%d,n_ini=%d,n_fin=%d\n",Nx,Ny,n_opt,n_ini,n_fin);
#endif /* MFAHDEBUG */ 
 
      four_pow_n=(int)pow(2.,(double)n_fin);
      if((sup_E_n_epsilon_alpha=(unsigned char*)malloc((size_t)(four_pow_n*(sizeof(unsigned char)))))==NULL) 
	{ 
 	  fprintf(stderr,"malloc error\n");
	  return 0; 
 	}  
      memset(sup_E_n_epsilon_alpha,0,(size_t)(four_pow_n*sizeof(unsigned char)));
	     
      /* tree creation with n_opt levels */
      tree=MFAM_qtree_create(n_opt);
      
      two_pow_n=(int)pow(2.,(double)n_opt);
 
      /* capacities levels creation and computation
	 by attachment and decimation */
      if((Nx==two_pow_n)||(Ny==two_pow_n)) 
 	{ 
 	  MFAM_qtree_level_create(tree,MFAM_CAPACITY,n_ini,n_opt-1);  
 	  MFAM_qtree_level_set(tree,MFAM_CAPACITY,n_opt,(void*)c2d); 
 	} 
      else 
 	{
 	  MFAM_qtree_level_create(tree,MFAM_CAPACITY,n_ini,n_opt); 
 	  MFAM_qtree_level_convert(tree,MFAM_CAPACITY,n_opt,Nx,Ny,(void*)c2d); 
 	} 

      MFAM_qtree_c_level_decimate(tree,t_capa,n_ini,n_opt);
  	 
#ifdef MFAHDEBUG
      MFAM_qtree_level_print(tree,MFAM_CAPACITY,n_ini,n_opt);
#endif /* MFAHDEBUG */

      /* reference measures levels creation and computation
	 by attachment and decimation */
      if(mu2d)
	{ 
	  if((Nx==two_pow_n)||(Ny==two_pow_n)) 
	    { 
	      MFAM_qtree_level_create(tree,MFAM_REFERENCE,n_ini,n_opt-1);
	      MFAM_qtree_level_set(tree,MFAM_REFERENCE,n_opt,(void*)mu2d); 
	    }
	  else
	    {  
	      MFAM_qtree_level_create(tree,MFAM_REFERENCE,n_ini,n_opt); 
	      MFAM_qtree_level_convert(tree,MFAM_REFERENCE,n_opt,Nx,Ny,(void*)mu2d); 
	    }
	  
	  MFAM_qtree_r_level_decimate(tree,n_ini,n_opt);

#ifdef MFAHDEBUG
	  MFAM_qtree_level_print(tree,MFAM_REFERENCE,n_ini,n_opt);
#endif /* MFAHDEBUG */

	}

      /* coarse-grain hoelder exponents levels creation and computation */
      MFAM_qtree_level_create(tree,MFAM_HOELDER,n_ini,n_opt);
      if(mu2d) 
	MFAH_qtree_mma_level_compute(tree,n_ini,n_opt); 
      else
	MFAH_qtree_h_level_compute(tree,n_ini,n_opt); 

#ifdef MFAHDEBUG
      MFAM_qtree_level_print(tree,MFAM_HOELDER,n_ini,n_opt);
#endif /* MFAHDEBUG */ 

      /* coarse-grain hoelder exponents statistics computation */
      alpha_n_i_j=(double*)MFAM_qtree_level_get(tree,MFAM_HOELDER,n_ini);
      four_pow_n=(int)pow(4.,(double)n_ini);
      MFAH_hoelder(four_pow_n,alpha_n_i_j,N,alpha,&epsilon);

#ifdef MFAHDEBUG
      fprintf(stderr,"[alpha_min,alpha_max]=[7m[%g,%g][m\n",alpha_min,alpha_max);
#endif /* MFAHDEBUG */     

      /* iso-hoelder sets level creation */
      MFAM_qtree_level_create(tree,MFAM_SET,n_ini,n_fin); 
	     
      /* dynamic memory allocation */
      if((t_dim==MFAD_BOX)||(t_dim==MFAD_VOSS))
	{ 
	  if((log_N_epsilon=(double*)malloc((size_t)(K*sizeof(double))))==NULL)
	    {
	      fprintf(stderr,"malloc error\n");
	      return 0;
	    }  
	  memset(log_N_epsilon,0,K*sizeof(double));
	  if((log_epsilon=(double*)malloc((size_t)(K*sizeof(double))))==NULL)
	    {
	      fprintf(stderr,"malloc error\n");
	      return 0;
	    } 
	  memset(log_N_epsilon,0,K*sizeof(double));
	}  
      else if((t_dim==MFAD_BRANCHING)||(t_dim==MFAD_HAUSDORFF))
	{  	
	  /* dim tree creation */
	  dim_tree=MFAM_qtree_create(n_fin); 
	  /* dim tree set levels creation */
      	  MFAM_qtree_level_create(dim_tree,MFAM_SET,0,n_fin);
	} 
	      
      for(k=0;k<N;k++) 
	{

#ifndef MFAHNOPROGESS
	  MFAM_progress(k,N);
#endif /* MFAHNOPROGESS */

	  n=n_fin;
	  sup_d_E_n_epsilon_alpha=0.;
	  while(n>=n_ini)  
	    {  
	      two_pow_n=(int)pow(2.,(double)n);
	      MFAH_qtree_E_n_epsilon_alpha(tree,n,epsilon,*(alpha+k));
	      E_n_epsilon_alpha=(unsigned char*)MFAM_qtree_level_get(tree,MFAM_SET,n);
	      if((t_dim==MFAD_BRANCHING)||(t_dim==MFAD_HAUSDORFF))
		{ 
		  /* dim tree set level conversion */
		  MFAM_qtree_level_convert(dim_tree,MFAM_SET,n,two_pow_n,two_pow_n,(void*)E_n_epsilon_alpha); 
		  /* dim tree set levels decimation */
		  MFAM_qtree_s_level_decimate(dim_tree,0,n); 
		}
	      if(MFAM_isempty(two_pow_n*two_pow_n,E_n_epsilon_alpha)==0)
		{    
		  if(MFAD_dimension2d(two_pow_n,two_pow_n,E_n_epsilon_alpha,K,B,t_dim,log_epsilon,log_N_epsilon,&d_E_n_epsilon_alpha,(void*)dim_tree)==0) 
		    { 
		      fprintf(stderr,"MFAD_dimension2d function call error\n"); 
		      return 0; 
		    }
		}
	      sup_d_E_n_epsilon_alpha=(sup_d_E_n_epsilon_alpha>d_E_n_epsilon_alpha?sup_d_E_n_epsilon_alpha:d_E_n_epsilon_alpha);

#ifdef MFAHDEBUG 
	      fprintf(stderr,"alpha=%f, d_E_n_epsilon_alpha=%f\n",*(alpha+k),d_E_n_epsilon_alpha);
	      fprintf(stderr,"alpha=%f, sup_d_E_n_epsilon_alpha=%f\n",*(alpha+k),sup_d_E_n_epsilon_alpha);  
#endif /* MFAHDEBUG */

	      d_E_n_epsilon_alpha=0;
	      n--;
	    } 
	  *(fdls2d_alpha+k)=sup_d_E_n_epsilon_alpha;

#ifdef MFAHDEBUG 
	  fprintf(stderr,"alpha=%f, fdls2d_alpha=%f\n",*(alpha+k),*(fdls2d_alpha+k));
#endif /* MFAHDEBUG */

	} 

#ifndef MFAHNOPROGESS
      MFAM_progress(N,N);
#endif /* MFAHNOPROGESS */
  
      /* dynamic memory deallocation */
      if((t_dim==MFAD_BOX)||(t_dim==MFAD_VOSS))
	{
	  free((char*)log_N_epsilon);
	  free((char*)log_epsilon);
	}
      else if((t_dim==MFAD_BRANCHING)||(t_dim==MFAD_HAUSDORFF))
	{  
	  /* dim tree set levels deletion */
	  MFAM_qtree_level_delete(dim_tree,MFAM_SET,0,n_fin);   
	  /* tree deletion */
	  MFAM_qtree_delete(dim_tree);
	}
 
      two_pow_n=(int)pow(2.,(double)n_opt);

      /* capacities levels deletion */  
      if((Nx==two_pow_n)||(Ny==two_pow_n)) 
	{
	  MFAM_qtree_level_set(tree,MFAM_CAPACITY,n_opt,(void*)NULL); 
	  MFAM_qtree_level_delete(tree,MFAM_CAPACITY,n_ini,n_opt-1);  
	}
      else
	MFAM_qtree_level_delete(tree,MFAM_CAPACITY,n_ini,n_opt);

      /* reference measures levels deletion */  
      if(mu2d)
	{ 
	  if((Nx==two_pow_n)||(Ny==two_pow_n)) 
	    {
	      MFAM_qtree_level_set(tree,MFAM_REFERENCE,n_opt,(void*)NULL); 
	      MFAM_qtree_level_delete(tree,MFAM_REFERENCE,n_ini,n_opt-1);
	    } 
	  else
	    MFAM_qtree_level_delete(tree,MFAM_REFERENCE,n_ini,n_opt);
	} 

      /* coarse-grain hoelder exponents levels deletion */
      MFAM_qtree_level_delete(tree,MFAM_HOELDER,n_ini,n_opt); 

      /* iso-hoelder sets levels deletion */
      MFAM_qtree_level_delete(tree,MFAM_SET,n_ini,n_fin);

      /* tree deletion */
      MFAM_qtree_delete(tree);
    
      /* spectrum normalization */
      if(MFAM_norm(N,fdls2d_alpha,NULL,2)==0)
	{
	  fprintf(stderr,"MFAM_norm error\n");
	  return 0;
	}
    
      return 1;
    }
  else
    {
      fprintf(stderr,"MFAH_mfdls2d arguments error\n");
      fprintf(stderr,"\tNx=[7m%d[m\n",Nx); 
      fprintf(stderr,"\tNy=[7m%d[m\n",Ny);  
      fprintf(stderr,"\tc2d=[7m%p[m\n",c2d);  
      fprintf(stderr,"\tmu2d=[7m%p[m\n",mu2d); 
      fprintf(stderr,"\tn_ini=[7m%d[m\n",n_ini); 
      fprintf(stderr,"\tn_fin=[7m%d[m\n",n_fin); 
      fprintf(stderr,"\tt_capa=[7m%d[m\n",t_capa); 
      fprintf(stderr,"\tepsilon=[7m%f[m\n",epsilon); 
      fprintf(stderr,"\tK=[7m%d[m\n",K);   
      fprintf(stderr,"\tB=[7m%p[m\n",B);  
      fprintf(stderr,"\tt_dim=[7m%d[m\n",t_dim); 
      fprintf(stderr,"\tN=[7m%d[m\n",N);   
      fprintf(stderr,"\talpha=[7m%p[m\n",alpha);
      fprintf(stderr,"\tfdls2d_alpha=[7m%p[m\n",fdls2d_alpha); 
      return 0;
    }
}
