/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/MFA.CMP/MFAH.CL/src/MFAH_misc.c
* File type        : C source file
* Revision         : $Revision$
* RCS status       : $Id$
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Thu Sep 19 11:40:31 2002
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Sat Sep 21 12:12:53 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

#include "MFAH_misc.h"

/*****************************************************************************/

int MFAH_min_max(int n,double* n_sample,double* min,double* max)
{
  if((n>0)&&(n_sample!=NULL)&&(min!=NULL)&&(max!=NULL))
  {
    register int k=0; 
    *min=HUGE;
    *max=-HUGE;
    while(k<n)
    {
      if((*(n_sample+k)!=HUGE)&&(*(n_sample+k)!=0.)) /* to avoid zeros */
      {
	*min=(*(n_sample+k)<*min?*(n_sample+k):*min);
	*max=(*(n_sample+k)>*max?*(n_sample+k):*max);
      }
      k++;
    }  
    return 1;
  }
  else
  { 
    fprintf(stderr,"MFAH_min_max arguments error\n");
    fprintf(stderr,"\tn=[7m%d[m\n",n);
    fprintf(stderr,"\tn_sample=[7m%p[m\n",n_sample);  
    fprintf(stderr,"\tmin=[7m%p[m\n",min);  
    fprintf(stderr,"\tmax=[7m%p[m\n",max);
    return 0;
  }   
}

/*****************************************************************************/

int MFAH_hoelder(int Nx,double* p_alpha,int N,double* alpha, double* epsilon)
{ 
  if((Nx>0)&&(p_alpha!=NULL)&&(N>0)&&(alpha!=NULL)&&(epsilon!=NULL))
  {
    double p_alpha_min=HUGE;
    double p_alpha_max=-HUGE;
    double d_alpha=0.;

    if(MFAH_min_max(Nx,p_alpha,&p_alpha_min,&p_alpha_max)==0)
      { 
	fprintf(stderr,"MFAH_min_max function call error\n");
	return 0;
      }
    
#ifdef MFAHDEBUG
    fprintf(stderr,"p_alpha_min=%f,p_alpha_max=%f\n",p_alpha_min,p_alpha_max);
#endif /* MFAHDEBUG */
    
    d_alpha=(p_alpha_max-p_alpha_min)/((double)N-1.);
    if(*epsilon<d_alpha)
      {
	fprintf(stderr,"\n[7m Forced epsilon (%f) to be greater than d_alpha (%f)[m\n",*epsilon,d_alpha);
	*epsilon=d_alpha; 
      }
    else if(*epsilon>10*d_alpha)
      {	
	fprintf(stderr,"\n[7m Minimum value of epsilon (%f) is d_alpha (%f)[m\n",*epsilon,d_alpha);
      }

    if(MFAM_linspace(N,p_alpha_min,p_alpha_max,alpha)==0)
      { 
	fprintf(stderr,"MFAM_linspace function call error\n");
	return 0;
      } 
    return 1;
  }  
  else
  { 
    fprintf(stderr,"MFAH_hoelder arguments error\n");
    fprintf(stderr,"\tNx=[7m%d[m\n",Nx);
    fprintf(stderr,"\tp_alpha=[7m%p[m\n",p_alpha);  
    fprintf(stderr,"\tN=[7m%d[m\n",N);  
    fprintf(stderr,"\talpha=[7m%p[m\n",alpha);
    fprintf(stderr,"\tepsilon=[7m%p[m\n",epsilon); 
    return 0;
  }   
}

/*****************************************************************************/

int MFAH_sup_E_n_epsilon_alpha(int N_E_n_epsilon_alpha,unsigned char* E_n_epsilon_alpha,int N_sup_E_n_epsilon_alpha,unsigned char* sup_E_n_epsilon_alpha)
{
  if((N_E_n_epsilon_alpha>0)&&(E_n_epsilon_alpha!=NULL)&&(N_E_n_epsilon_alpha<N_sup_E_n_epsilon_alpha)&&(sup_E_n_epsilon_alpha!=NULL))
  {
    register int k=0;   
    register int l=0; 
    int ratio=N_sup_E_n_epsilon_alpha/N_E_n_epsilon_alpha;
    for(k=0;k<N_E_n_epsilon_alpha;k++)
      if(*(E_n_epsilon_alpha+k)==(unsigned char)1)
	for(l=0;k<ratio;k++)
	  *(sup_E_n_epsilon_alpha+k)=(unsigned char)1;
    return 1;
  }
  else
  { 
    fprintf(stderr,"MFAH_sup_E_n_epsilon_alphaarguments error\n");
    fprintf(stderr,"\tN_E_n_epsilon_alpha=[7m%d[m\n",N_E_n_epsilon_alpha);
    fprintf(stderr,"\tE_n_epsilon_alpha=[7m%p[m\n",E_n_epsilon_alpha);  
    fprintf(stderr,"\tN_sup_E_n_epsilon_alpha=[7m%d[m\n",N_sup_E_n_epsilon_alpha);  
    fprintf(stderr,"\tsup_E_n_epsilon_alpha=[7m%p[m\n",sup_E_n_epsilon_alpha);
    return 0;
  }   
}
