/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/MFA.CMP/MFAH.CL/src/MFAH_qtree.c
* File type        : C source file
* Revision         : $Revision$
* RCS status       : $Id$
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sun Mar 31 17:52:14 2002
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Tue Sep 17 15:08:44 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 02 */

#include "MFAH_qtree.h"

/*****************************************************************************/

int MFAH_qtree_h_level_compute(MFAMt_qtree* tree,short n_min,short n_max)
{
   if(tree!=NULL)
   {
      short n=n_min;
      while(n<=n_max)
      {
	 int four_pow_n=(int)pow(4.,(double)n);
	 int k=0;
	 double c_n_i_j=0.;
	 double alpha_n_i_j=0.;
#ifdef MFAHDEBUG
	 fprintf(stderr,"computing hoelder exponents of level # %d\n",n);
#endif /* MFAHDEBUG */
	 while(k<four_pow_n)
	 { 
	   c_n_i_j=*(*(tree->c_level+n)+k);
	   if(c_n_i_j!=0.)
	     alpha_n_i_j=log(c_n_i_j)/(-n*log(2.));
	   else
	     alpha_n_i_j=HUGE;
	   *(*(tree->h_level+n)+k)=alpha_n_i_j;
	   k++;
	 }
	 n++;
      }
      return 1;
   }
   else
   {
      fprintf(stderr,"MFAH_qtree_h_level_compute arguments error\n");
      return 0;
   } 
}

/*****************************************************************************/

int MFAH_qtree_mma_level_compute(MFAMt_qtree* tree,short n_min,short n_max)
{
   if(tree!=NULL)
   {
      short n=n_min;
      while(n<=n_max)
      {
	 int four_pow_n=(int)pow(4.,(double)n);
	 int k=0;
	 double c_n_i_j=0.;
	 double mu_n_i_j=0.;
	 double alpha_n_i_j=0.;
#ifdef MFAHDEBUG
	 fprintf(stderr,"computing hoelder exponents of level # %d\n",n);
#endif /* MFAHDEBUG */
	 while(k<four_pow_n)
	 { 
	   c_n_i_j=*(*(tree->c_level+n)+k);
	   mu_n_i_j=*(*(tree->r_level+n)+k);
	   if((c_n_i_j!=0.)&&(mu_n_i_j!=0.)&&(mu_n_i_j!=1.))
	     alpha_n_i_j=log(c_n_i_j)/log(mu_n_i_j);
	   else
	     alpha_n_i_j=HUGE;
	   *(*(tree->h_level+n)+k)=alpha_n_i_j;
	   k++;
	 }
	 n++;
      }
      return 1;
   }
   else
   {
      fprintf(stderr,"MFAH_qtree_mma_level_compute arguments error\n");
      return 0;
   } 
}

/*****************************************************************************/

int MFAH_qtree_E_n_epsilon_alpha(MFAMt_qtree* tree,short n,double epsilon,double alpha)
{  
  if(tree!=NULL)
    {
      MFAH_qnode_E_n_epsilon_alpha(tree->root,tree,n,epsilon,alpha);
      return 1;
    }
  else
    {  
      fprintf(stderr,"MFAH_qtree_E_n_epsilon_alpha arguments error\n");
      return 0;
    }
}

/*****************************************************************************/

int MFAH_qnode_E_n_epsilon_alpha(MFAMt_qnode* node,MFAMt_qtree* tree,short n,double epsilon,double alpha)
{ 
  if(node!=NULL)
    {
      int s=0;
      int k=node->n; 
#ifdef MFAHDEBUG
      MFAM_qnode_print(node,0);
#endif /* MFAHDEBUG */
      if(k<n)
	{
#ifdef MFAHDEBUG
	  fprintf(stderr,"[7mk<n[m\n");
#endif /* MFAHDEBUG */
	  MFAH_qnode_E_n_epsilon_alpha(node->cnode,tree,n,epsilon,alpha);
	  MFAH_qnode_E_n_epsilon_alpha(node->hnode,tree,n,epsilon,alpha);
	  MFAH_qnode_E_n_epsilon_alpha(node->vnode,tree,n,epsilon,alpha);
	  MFAH_qnode_E_n_epsilon_alpha(node->hvnode,tree,n,epsilon,alpha);
	}
      else if(k>=n)
	{ 
	  double h=0.;
	  h=MFAM_qnode_hget(node,tree);
	  s=(int)(fabs(h-alpha)<epsilon);
#ifdef MFAHDEBUG  
	  fprintf(stderr,"[7mk>=n[m\n");
	  fprintf(stderr,"k=%d h=%f alpha=%f epsilon=%f ==> s=[7m%d[m\n",k,h,alpha,epsilon,s);
#endif /* MFAHDEBUG */
	  if(s&&(k<tree->N))
	    {
	      s&=MFAH_qnode_E_n_epsilon_alpha(node->cnode,tree,n,epsilon,alpha)
		&&
		MFAH_qnode_E_n_epsilon_alpha(node->hnode,tree,n,epsilon,alpha)
		&&
		MFAH_qnode_E_n_epsilon_alpha(node->vnode,tree,n,epsilon,alpha)
		&&
		MFAH_qnode_E_n_epsilon_alpha(node->hvnode,tree,n,epsilon,alpha);
	    }
	  if(k==n)
	    { 
#ifdef MFAHDEBUG
	      fprintf(stderr,"[7mk=n[m\n"); 
	      fprintf(stderr,"s=%d k=%d\n",s,k);
#endif /* MFAHDEBUG */
	      MFAM_qnode_sset(node,tree,(unsigned char)s);
	    }	
	  if((h==HUGE)&&(k==tree->N))
	    { 
	      s=1;
#ifdef MFAHDEBUG
	      fprintf(stderr,"HUGE ==> s=%d\n",s);
#endif /* MFAHDEBUG */
	    }
	} 
      return s;
    }
  else
    return 0;
}
