/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/MFA.CMP/MFAM.CL/src/MFAM_adaption.c
* File type        : C source file
* Revision         : $Revision$
* RCS status       : $Id$
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sun Feb  7 15:56:49 1999
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Mon Apr  1 17:51:04 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 1997-98 */

#include "MFAM_adaption.h" 

static double mfam_stddev(int,double*);

/*****************************************************************************/

static double mfam_stddev(int n,double* n_sample)
{
  register int k=0;
  double mean_n=0.,sigma2_n=0.;
  for(k=0;k<n;k++)
    mean_n+=*(n_sample+k);
  mean_n/=(double)n;
  for(k=0;k<n;k++) 
    sigma2_n+=(*(n_sample+k)-mean_n)*(*(n_sample+k)-mean_n);
  sigma2_n/=(double)n;
  return sqrt(sigma2_n);
}

/*****************************************************************************/

int MFAM_widthstr(char* widthstr,MFAMt_width* t_width)
{
  if((widthstr!=NULL)&&(t_width!=NULL))
  {
    if(strcmp(MFAM_HNSSTR,widthstr)==0)
      *t_width=MFAM_HNS;
    else if(strcmp(MFAM_MINDEVSTR,widthstr)==0)
      *t_width=MFAM_MINDEV;
    else if(strcmp(MFAM_MAXDEVSTR,widthstr)==0)
      *t_width=MFAM_MAXDEV;
    else if(strcmp(MFAM_MEANDEVSTR,widthstr)==0)
      *t_width=MFAM_MEANDEV;
    else if(strcmp(MFAM_LOCALDEVSTR,widthstr)==0)
      *t_width=MFAM_LOCALDEV;  
    else if(strcmp(MFAM_LAPHSTR,widthstr)==0)
      *t_width=MFAM_LAPH;
    else
      return 0;
    return 1;
  }
  else
  {
    fprintf(stderr,"MFAM_widthstr arguments error\n");
    return 0; 
  }
}
 
/*****************************************************************************/

#ifndef __STDC__
int MFAM_adaption(n,n_sample,M,dev_x, kern,N,h,t_width,x,p_x)
     int n;
     double* n_sample;
     int M;
     double* dev_x;
     double (*kern)();
     int N;
     double* h;
     MFAMt_width t_width;
     double* x;
     double* p_x;
#else /* __STDC__ */
int MFAM_adaption(int n,double* n_sample,int M,double* dev_x,double(*kern)(double),int N,double* h,MFAMt_width t_width,double* x,double* p_x)
#endif /* __STDC__ */
{
  if((n>0)&&(n_sample!=NULL)&&(N>0)&&(h!=NULL)&&(x!=NULL)&&(p_x!=NULL)) 
  {
    register int i=0,k=0;
    double h_n=0.,s_n=0.,dev=0.,dev_x_min=HUGE,dev_x_max=-HUGE,dev_x_mean=0.;
    switch(t_width)
    {
      case MFAM_HNS:
	s_n=mfam_stddev(n,n_sample)/pow(n,.2);
	if(kern==MFAM_gauss)
	  h_n=GAUSSIANH*s_n;
	else if(kern==MFAM_epa)
	  h_n=EPANECHNIKOVH*s_n;
	else if(kern==MFAM_quartic)
	  h_n=QUARTICH*s_n;
	else
	  h_n=s_n;
	for(k=0;k<N;k++)
	  *(h+k)=h_n;
	break;
      case MFAM_MINDEV: 
	h_n=HUGE;
	if((M>0)&&(dev_x!=NULL))
 	  for(k=1;k<M;k++) 
 	    h_n=(*(dev_x+k)<h_n?*(dev_x+k):h_n); 
 	else 
	{
	  if(MFAM_min_dev(n,n_sample,&h_n)==0)
	  {
	    fprintf(stderr,"MFAM_min_dev error\n");
	    return 0;
	  }
	}  
	for(k=0;k<N;k++)
	  *(h+k)=h_n;
	break;
      case MFAM_MAXDEV: 
	h_n=-HUGE;
	if((M>0)&&(dev_x!=NULL)) 
	  for(k=1;k<M;k++)
	    h_n=(*(dev_x+k)>h_n?*(dev_x+k):h_n);
	else
	{
	  if(MFAM_max_dev(n,n_sample,&h_n)==0)
	  {
	    fprintf(stderr,"MFAM_max_dev error\n");
	    return 0;
	  } 
	}
	for(k=0;k<N;k++)
	  *(h+k)=h_n;
	break;
      case MFAM_MEANDEV:
	h_n=0.;
	if((M>0)&&(dev_x!=NULL))
	{
	  for(k=1;k<M;k++)
	    h_n+=*(dev_x+k);
	  h_n/=(M-1);
	}
	else
	{
	  if(MFAM_mean_dev(n,n_sample,&h_n)==0)
	  {
	    fprintf(stderr,"MFAM_mean_dev error\n");
	    return 0;
	  }
	}  
	for(k=0;k<N;k++)
	  *(h+k)=h_n;
	break;
      case MFAM_LOCALDEV:
	if((M>0)&&(dev_x!=NULL))
	{
	  if(MFAM_min_max(M-1,dev_x+1,&dev_x_min,&dev_x_max)==0)
	  {
	    fprintf(stderr,"MFAM_min_max error\n");
	    return 0;   
	  } 
	  dev_x_mean=(dev_x_max+dev_x_min)/2.;
	  for(k=0;k<M;k++)
	  {
	    if(*(dev_x+k)>dev_x_mean)
	      *(dev_x+k)=dev_x_max;
	    else
	      *(dev_x+k)=dev_x_min;
	  } 
	}
	else
	{
	  for(i=1;i<n;i++)
	  {
	    dev=*(n_sample+i)-*(n_sample+i-1);
	    if(dev)
	    {
	      dev_x_max=(dev>dev_x_max?dev:dev_x_max);
	      dev_x_min=(dev<dev_x_min?dev:dev_x_min);
	    }
	  }
	  dev_x_mean=(dev_x_max+dev_x_min)/2.;
	  i=0;
	  do
	  {
	    i++;
	    dev=*(n_sample+i)-*(n_sample+i-1);   
	  }
	  while((i<n)&&(dev<=dev_x_mean));
	  for(k=0;k<N;k++)
	  {
	    if(*(x+k)<*(n_sample+i))
	      *(h+k)=dev_x_max;
	    else
	      *(h+k)=dev_x_min;
	  }
	}
	break;
      case MFAM_LAPH:
	{
	  if(MFAM_laph(n,n_sample,kern,N,x,h)==0)
	  {
	    fprintf(stderr,"MFAM_laph arguments error\n");
	    return 0;
	  }
	  break;
	}
      default: 
	fprintf(stderr,"MFAM_adaption arguments error\n");
	return 0;
    }
    return 1;
  }
  else
  {
    fprintf(stderr,"MFAM_adaption arguments error\n");
    return 0; 
  }
}

/*****************************************************************************/

#ifndef __STDC__
int MFAM_laph(n,Y_t,kern,N,t,h_hat_t)
     int n;
     double* Y_t;
     double(*kern)();
     int N;
     double* t;
     double* h_hat_t;
#else /* __STDC__ */
int MFAM_laph(int n,double* Y_t,double(*kern)(double),int N,double* t,double* h_hat_t)
#endif /* __STDC__ */
{
  if((n>0)&&(Y_t!=NULL)&&(kern!=NULL)&&(N>0)&&(t!=NULL)&&(h_hat_t!=NULL))
  {
    register int k=0,j=0,l=0;
    int boolean=1;
    double* H_sigma=NULL;
    double* f_hat_h_t=NULL;
    double sigma2=1./n;
    double s=1.;
    double D3=1.;
    double a=2.;
    double h_star_sigma=(*(t+N-1)-*t)/2.;
    short J=floor(log(h_star_sigma/sigma2))/log(a); 
    if((H_sigma=(double*)malloc((unsigned)(J*sizeof(double))))==NULL)
    {
      fprintf(stderr,"malloc error\n");
      return 0;
    } 
    if((f_hat_h_t=(double*)malloc((unsigned)(J*sizeof(double))))==NULL)
    {
      fprintf(stderr,"malloc error\n");
      return 0;
    }
    for(j=0;j<J;j++)
      *(H_sigma+j)=pow(a,-(double)j)*h_star_sigma;
    for(k=0;k<N;k++)
    { 
    /*   for(j=0;j<J;j++) */
/* 	*(H_sigma+j)=pow(a,-(double)j)**(h_hat_t+k); */
      j=J-1;
      boolean=1;
      do
      {
	if(MFAM_kernelconvolve(n,Y_t,*(H_sigma+j),kern,N,k,t+k,f_hat_h_t+j)==0)
	{
	  fprintf(stderr,"MFAM_kernelconvolve error\n");
	  return 0;
	}  
	/* fprintf(stderr,"h_t=%f,f_hat_h_t=%f\n",*(H_sigma+j),*(f_hat_h_t+j)); */
	l=J-1;
	while((l>j)&&boolean)
        {
	/*   fprintf(stderr,"|f_hat_h_t-f_hat_eta_t|=%f\n",fabs(*(f_hat_h_t+j)-*(f_hat_h_t+l))); */
/* 	  fprintf(stderr,"maj=%f\n",D3*s/sqrt(n**(H_sigma+l))*sqrt(1+log(h_star_sigma/(*(H_sigma+l))))); */
	  boolean=(fabs(*(f_hat_h_t+j)-*(f_hat_h_t+l))<=
		   D3*s/sqrt(n**(H_sigma+l))*
		   sqrt(1+log(h_star_sigma/(*(H_sigma+l)))));
	  l--; 
	}
	j--; 
      }
      while((j>=0)&&boolean);
    /*   fprintf(stderr,"********** %d,%d,%f\n",k,l,*(H_sigma+l)); */
      *(h_hat_t+k)=*(H_sigma+l);
    }  
    free((char*)H_sigma); 
    free((char*)f_hat_h_t);
    return 1;
  }
  else
  { 
    fprintf(stderr,"MFAM_laph arguments error\n");
    return 0;
  }
}
