/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/canus/MFA.CMP/MFAM.CL/src/MFAM_kernel.c
* File type        : C source file
* Revision         : $Revision: 1.1.1.1 $
* RCS status       : $Id: MFAM_kernel.c,v 1.1.1.1 1999/02/10 13:29:04 guiheneu Exp $
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sun Feb  7 15:56:25 1999
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Sun Feb  7 15:56:27 1999
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 1997-98 */

#include "MFAM_kernel.h"

/*****************************************************************************/
/* compact support                                                           */
/*****************************************************************************/

/* plus */
double MFAM_plus(double x)
{
  if(x>0)
    return x;
  else
    return 0.;   
}

/* boxcar kernel */
double MFAM_boxcar(double x)
{
  if(fabs(x)>.5)
    return 0.;
  else
    return 1.;
}

/* triangle kernel */
double MFAM_triangle(double x)
{
  if(fabs(x)>.5)
    return 0.;
  else if(x>0.)
    return 1-2.*x;
  else 
    return 1+2.*x;
}

/* Mollifier kernel */
double MFAM_mol(double x)
{
  if(fabs(x)>.5)
    return 0.;
  else
    return exp(-1/fabs(1-x*x))/MOLLIFIER;
}

/* Epanechnikov kernel */
double MFAM_epa(double x)
{
  if(fabs(x)>SQRTFIVE)
    return 0.;
  else
    return EPANECHNIKOV*MFAM_plus(1-x*x/5.);
}

/* Quartic kernel */
double MFAM_quartic(double x)
{
  if(fabs(x)>.5)
    return 0.;
  else
    return QUARTIC*MFAM_plus(1-x*x)*MFAM_plus(1-x*x);
}

/*****************************************************************************/
/* non-compact support                                                       */
/*****************************************************************************/

/* normalized reduced Gaussian kernel */
double MFAM_gauss(double x)
{
  return 1/SQRTTWOPI*exp(-x*x/2.);
}

/*****************************************************************************/

int MFAM_kernelstr(char* kernelstr,MFAMt_kern* t_kern)
{
  if(strcmp(MFAM_INTEGRATEDSTR,kernelstr)==0)
    *t_kern=MFAM_INTEGRATED;
  else if(strcmp(MFAM_BOXCARSTR,kernelstr)==0)
    *t_kern=MFAM_BOXCAR;
  else if(strcmp(MFAM_TRIANGLESTR,kernelstr)==0)
    *t_kern=MFAM_TRIANGLE;
  else if(strcmp(MFAM_GAUSSIANSTR,kernelstr)==0)
    *t_kern=MFAM_GAUSSIAN;   
  else if(strcmp(MFAM_MOLLIFIERSTR,kernelstr)==0)
    *t_kern=MFAM_MOLLIFIER;
  else if(strcmp(MFAM_EPANECHNIKOVSTR,kernelstr)==0)
    *t_kern=MFAM_EPANECHNIKOV; 
  else if(strcmp(MFAM_QUARTICSTR,kernelstr)==0)
    *t_kern=MFAM_QUARTIC;  
  else 
    return 0;
  return 1;
}

/*****************************************************************************/

int MFAM_kernelconvolve(int n,double* n_sample,double h,double(*kern)(double),int N,int k,double* x,double* p_x)
{
  if((n>0)&&(n_sample!=NULL)&&(h>0.)&&(kern!=NULL)&&(N>0)&&(k<N)&&(x!=NULL)&&(p_x!=NULL))
  {
    register int i=0; 
    *p_x=0.;
    if((k==0)||(k==N-1))
    {
      for(i=0;i<n;i++)
	*p_x+=kern((*(n_sample+i)-*x)/h);
      *p_x/=h*(double)n;
    }
    else
    {
      if(*x-*n_sample<h)
      {
	for(i=0;i<n;i++)
	  if(*(n_sample+i)>*x)  
	    *p_x+=kern((*(n_sample+i)-*x)/h);
	  else
	    *p_x+=kern((*(n_sample+i)-*x)/(*x-*n_sample));
      }
      else if(*(n_sample+n-1)-*x<h)
      {
	for(i=0;i<n;i++)
	  if(*(n_sample+i)<*x)  
	    *p_x+=kern((*(n_sample+i)-*x)/h);
	  else
	    *p_x+=kern((*(n_sample+i)-*x)/(*(n_sample+n-1)-*x));
      }
      else
	for(i=0;i<n;i++)
	  *p_x+=kern((*(n_sample+i)-*x)/h);
      *p_x/=h*(double)n;
    }
    return 1;
  }
  else
  {
    fprintf(stderr,"MFAM_kernelconvolve arguments error\n");
    return 0;
  }
}
