/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/FRACLAB-WKSP/FRACLAB/MFAM/SRC/MFAM_qtree.c
* File type        : C source file
* Revision         : $Revision$
* RCS status       : $Id$
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sat Mar 30 18:57:08 2002
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Wed Nov 20 23:21:34 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 2002 */

#include "MFAM_qtree.h"

extern double norm_meas;
extern double p;
extern double c_ref;
extern double q;
extern double sigma2;

/*****************************************************************************/

MFAMt_qnode* MFAM_qnode_create(MFAMt_qnode* node,char* x,short n,int i,int j,short n_fin)
{
  if (node==NULL)
    {	
      if((node=(MFAMt_qnode*)malloc(sizeof(MFAMt_qnode)))==NULL)
	{ 
	  fprintf(stderr,"MFAM_qnode_create: malloc error\n");
	  return NULL;
	}
      node->x=x;
      node->n=n;
      node->i=i;
      node->j=j;
      node->cnode=NULL;
      node->hnode=NULL;
      node->vnode=NULL;
      node->hvnode=NULL;
    }
  if(n!=n_fin)
    {
      char* xc=NULL;
      char* xh=NULL;
      char* xv=NULL;
      char* xhv=NULL;
      xc=(char*)malloc((strlen(x)+2)*sizeof(char));
      sprintf(xc,"%s%d",x,0); 
      node->cnode=MFAM_qnode_create(node->cnode,xc,n+1,2*i,2*j,n_fin);
      xh=(char*)malloc((strlen(x)+2)*sizeof(char));
      sprintf(xh,"%s%d",x,1); 
      node->hnode=MFAM_qnode_create(node->hnode,xh,n+1,2*i+1,2*j,n_fin);
      xv=(char*)malloc((strlen(x)+2)*sizeof(char));
      sprintf(xv,"%s%d",x,2); 
      node->vnode=MFAM_qnode_create(node->vnode,xv,n+1,2*i,2*j+1,n_fin); 
      xhv=(char*)malloc((strlen(x)+2)*sizeof(char));
      sprintf(xhv,"%s%d",x,3); 
      node->hvnode=MFAM_qnode_create(node->hvnode,xhv,n+1,2*i+1,2*j+1,n_fin);
    }
  return node;
}

/*****************************************************************************/

void MFAM_qnode_print(MFAMt_qnode* node,int recursive_print)
{ 
  if (node!=NULL)
    {
      fprintf(stderr,"qnode of type: ");
      if (!(node->i%2))
	{
	  if (!(node->j%2))
	    fprintf(stderr,"c");
	  else
	    fprintf(stderr,"v");
	}
      else
	{
	  if (!(node->j%2))
	    fprintf(stderr,"h");
	  else
	    fprintf(stderr,"hv");
	}
      fprintf(stderr," level k=%d, x=%s, i=%d,j=%d\n",node->n,node->x,node->i,node->j);
      if (recursive_print)
	{
	  MFAM_qnode_print(node->cnode,recursive_print);
	  MFAM_qnode_print(node->hnode,recursive_print); 
	  MFAM_qnode_print(node->vnode,recursive_print);
	  MFAM_qnode_print(node->hvnode,recursive_print);
	}
    }
}
/*****************************************************************************/

void MFAM_qnode_delete(MFAMt_qnode* node)
{ 
  if (node!=NULL)  
    { 
      MFAM_qnode_delete(node->cnode);
      node->cnode=NULL;
      MFAM_qnode_delete(node->hnode);
      node->hnode=NULL; 
      MFAM_qnode_delete(node->vnode);   
      node->vnode=NULL; 
      MFAM_qnode_delete(node->hvnode); 
      node->hvnode=NULL;
      free(node->x);
      free((char*)node);
    }
}

/*****************************************************************************/

MFAMt_qtree* MFAM_qtree_create(short N)
{
  MFAMt_qtree* tree=NULL; 
  char* x=NULL;
  if ((tree=(MFAMt_qtree*)malloc(sizeof(MFAMt_qtree)))==NULL) 
    { 
      fprintf(stderr,"MFAM_qtree_create: malloc error\n");
      return NULL;
    }
  tree->N=N;
  tree->root=NULL;
  x=(char*)strdup("0.");
  if ((tree->root=MFAM_qnode_create(tree->root,x,0,0,0,N))==NULL)
    { 
      fprintf(stderr,"root node creation error\n");
      return NULL;
    }  
  if((tree->c_level=(double**)malloc((N+1)*sizeof(double*)))==NULL)
    { 
      fprintf(stderr,"malloc error\n");
      return 0;
    }  
  memset(tree->c_level,0,(N+1)*sizeof(double*));
  
  if((tree->r_level=(double**)malloc((N+1)*sizeof(double*)))==NULL)
    { 
      fprintf(stderr,"malloc error\n");
      return 0;
    }  
  memset(tree->r_level,0,(N+1)*sizeof(double*));
 
  if((tree->h_level=(double**)malloc((N+1)*sizeof(double*)))==NULL)
    { 
      fprintf(stderr,"malloc error\n");
      return 0;
    }  
  memset(tree->h_level,0,(N+1)*sizeof(double*));

  if((tree->s_level=(unsigned char**)malloc((N+1)*sizeof(unsigned char*)))==NULL)
    { 
      fprintf(stderr,"malloc error\n");
      return 0;
    } 
  memset(tree->s_level,0,(N+1)*sizeof(unsigned char*)); 

  return tree;
}

/*****************************************************************************/

void MFAM_qtree_delete(MFAMt_qtree* tree)
{
  if (tree!=NULL)
    {
      int n=0;
      MFAM_qnode_delete(tree->root);
      while(n<=tree->N)
	{
	  if (*(tree->c_level+n)!=NULL)
	    {
	      free((char*)*(tree->c_level+n));
	      *(tree->c_level+n)=NULL;
	    } 
	  if (*(tree->r_level+n)!=NULL)
	    {
	      free((char*)*(tree->c_level+n));
	      *(tree->r_level+n)=NULL;
	    }
	  if (*(tree->h_level+n)!=NULL)
	    {
	      free((char*)*(tree->h_level+n));
	      *(tree->h_level+n)=NULL;
	    }
	  if (*(tree->s_level+n)!=NULL)  
	    {
	      free((char*)*(tree->s_level+n));
	      *(tree->s_level+n)=NULL;
	    }
	  n++;
	}
      free((char*)tree->c_level);  
      free((char*)tree->r_level);   
      free((char*)tree->h_level);
      free((char*)tree->s_level);
      free((char*)tree);
    }
}

/*****************************************************************************/

void MFAM_qtree_print(MFAMt_qtree* tree)
{ 
  if (tree!=NULL)
    {
      fprintf(stderr,"qtree with # N=%d levels\n",tree->N);
      MFAM_qnode_print(tree->root,1);
    }
}

/*****************************************************************************/

int MFAM_qtree_level_create(MFAMt_qtree* tree,MFAMt_level level_type,short n_ini,short n_fin)
{
  if (tree!=NULL)
    {
      short n=n_ini;
      switch (level_type)
	{	
	case MFAM_CAPACITY:
	  while(n<=n_fin)
	    {
	      int four_pow_n=(int)pow(4.,(double)n);   
	      if((*(tree->c_level+n)=(double*)malloc((size_t)(four_pow_n*sizeof(double))))==NULL) 
		{ 
		  fprintf(stderr,"MFAM_qtree_level_create malloc error\n");
		  return 0;
		}
	      memset(*(tree->c_level+n),0,(size_t)(four_pow_n*sizeof(double)));
	      n++;
	    }
	  break;    
	case MFAM_REFERENCE:
	  while(n<=n_fin)
	    {
	      int four_pow_n=(int)pow(4.,(double)n);   
	      if((*(tree->r_level+n)=(double*)malloc((size_t)(four_pow_n*sizeof(double))))==NULL) 
		{ 
		  fprintf(stderr,"MFAM_qtree_level_create malloc error\n");
		  return 0;
		} 
	      memset(*(tree->r_level+n),0,(size_t)(four_pow_n*sizeof(double)));
	      n++;
	    }
	  break;    
	case MFAM_HOELDER:
	  while(n<=n_fin)
	    {
	      int four_pow_n=(int)pow(4.,(double)n);
	      if((*(tree->h_level+n)=(double*)malloc((size_t)(four_pow_n*sizeof(double))))==NULL) 
		{ 
		  fprintf(stderr,"MFAM_qtree_level_create malloc error\n");
		  return 0;
		} 
	      memset(*(tree->h_level+n),0,(size_t)(four_pow_n*sizeof(double)));
	      n++;
	    }
	  break;   
	case MFAM_SET:
	  while(n<=n_fin)
	    {
	      int four_pow_n=(int)pow(4.,(double)n);
	      if((*(tree->s_level+n)=(unsigned char*)malloc((size_t)(four_pow_n*sizeof(unsigned char))))==NULL) 
		{ 
		  fprintf(stderr,"MFAM_qtree_level_create malloc error\n");
		  return 0;
		}
	      memset(*(tree->s_level+n),0,(size_t)(four_pow_n*sizeof(unsigned char)));
	      n++;
	    }
	  break;
	default:
	  fprintf(stderr,"MFAM_qtree_level_create arguments error\n");
	  return 0;
	  break;
	}
      return 1;
    }
  else
    {  
      fprintf(stderr,"MFAM_qtree_level_create arguments error\n");
      return 0;
    }
}

/*****************************************************************************/

int MFAM_qtree_level_delete(MFAMt_qtree* tree,MFAMt_level level_type,short n_ini,short n_fin)
{
  if (tree!=NULL)
    {
      short n=n_ini;
      switch (level_type)
	{	
	case MFAM_CAPACITY:
	  while(n<=n_fin)
	    {
	      if(*(tree->c_level+n)!=NULL)
		{  
		  free((char*)*(tree->c_level+n));
		  *(tree->c_level+n)=NULL;
		}
	      else
		{ 
		  fprintf(stderr,"MFAM_qtree_level_delete # %d free error\n",n);
		  return 0;
		}
	      n++;
	    }
	  break; 
	case MFAM_REFERENCE:
	  while(n<=n_fin)
	    {
	      if(*(tree->r_level+n)!=NULL)
		{  
		  free((char*)*(tree->r_level+n));
		  *(tree->r_level+n)=NULL;
		}
	      else
		{ 
		  fprintf(stderr,"MFAM_qtree_level_delete # %d free error\n",n);
		  return 0;
		}
	      n++;
	    }
	  break;      
	case MFAM_HOELDER:
	  while(n<=n_fin)
	    {
	      if(*(tree->h_level+n)!=NULL)
		{  
		  free((char*)*(tree->h_level+n));
		  *(tree->h_level+n)=NULL;
		}
	      else
		{ 
		  fprintf(stderr,"MFAM_qtree_level_delete # %d free error\n",n);
		  return 0;
		}
	      n++;
	    }
	  break;   
	case MFAM_SET:
	  while(n<=n_fin)
	    {
	      if(*(tree->s_level+n)!=NULL)
		{      
		  free((char*)*(tree->s_level+n));
		  *(tree->s_level+n)=NULL;
		}
	      else
		{ 
		  fprintf(stderr,"MFAM_qtree_level_delete # %d free error\n",n);
		  return 0;
		}
	      n++;
	    }
	  break;
	default:
	  fprintf(stderr,"MFAM_qtree_level_delete arguments error\n");
	  return 0;
	  break;
	}
      return 1;
    }
  else
    {  
      fprintf(stderr,"MFAM_qtree_level_delete arguments error\n");
      return 0;
    }
}

/*****************************************************************************/

int MFAM_qtree_level_set(MFAMt_qtree* tree,MFAMt_level level_type,short n,void* level)
{
  if((tree!=NULL)&&(n>0))
    {
      switch (level_type)
	{	
	case MFAM_CAPACITY:
	  if(((*(tree->c_level+n)==NULL)&&(level!=NULL))||
	     ((*(tree->c_level+n)!=NULL)&&(level==NULL)))
  	    { 
#ifdef MFAMDEBUG
	      fprintf(stderr,"qtree capacity level set n=%d\n",n);
#endif /* MFAMDEBUG */
	      *(tree->c_level+n)=(double*)level;
	    }
	  break; 
	case MFAM_REFERENCE:
	  if(((*(tree->r_level+n)==NULL)&&(level!=NULL))||
	     ((*(tree->r_level+n)!=NULL)&&(level==NULL)))
  	    { 
#ifdef MFAMDEBUG
	      fprintf(stderr,"qtree reference level set n=%d\n",n);
#endif /* MFAMDEBUG */
	      *(tree->r_level+n)=(double*)level;
	    }
	  break;     
	case MFAM_HOELDER:
	  *(tree->h_level+n)=(double*)level;
	  break;    
	case MFAM_SET:  
	  *(tree->s_level+n)=(unsigned char*)level;
	  break;   
	default:
	  fprintf(stderr,"MFAM_qtree_level_set arguments error\n");
	  break;
	}
      return 1;
    }
  else
    {
      fprintf(stderr,"MFAM_qtree_level_set arguments error\n");
      return 0;
   } 
}

/*****************************************************************************/

int MFAM_qtree_level_convert(MFAMt_qtree* tree,MFAMt_level level_type,short n,int Nx,int Ny,void* level)
{
  if((tree!=NULL)&&(n>0)&&(Nx>0)&&(Ny>0)&&(level!=NULL))
    { 
      register int x=0;
      register int y=0;
      int two_pow_n=(int)pow(2.,(double)n);
      switch (level_type)
	{	
	case MFAM_CAPACITY:
	  if((*(tree->c_level+n)!=NULL)&&(level!=NULL))
  	    {  
	      double* tmp_c2d=NULL;
	      double* tmp_level2d=NULL;
	      tmp_c2d=(double*)MFAM_qtree_level_get(tree,MFAM_CAPACITY,n);
	      tmp_level2d=(double*)level;
	      memset(tmp_c2d,0,(size_t)(two_pow_n*two_pow_n*sizeof(double)));
	      for(y=0;y<Ny;y++)
		for(x=0;x<Nx;x++)
		  *(tmp_c2d+two_pow_n*y+x)=(double)*(tmp_level2d+Nx*y+x);
	    }
	  break; 
	case MFAM_REFERENCE:
	  if((*(tree->r_level+n)!=NULL)&&(level!=NULL))
  	    {  
	      double* tmp_mu2d=NULL;
	      double* tmp_level2d=NULL;
	      tmp_mu2d=(double*)MFAM_qtree_level_get(tree,MFAM_REFERENCE,n);
	      tmp_level2d=(double*)level;
	      memset(tmp_mu2d,0,(size_t)(two_pow_n*two_pow_n*sizeof(double)));
	      for(y=0;y<Ny;y++)
		for(x=0;x<Nx;x++)
		  *(tmp_mu2d+two_pow_n*y+x)=(double)*(tmp_level2d+Nx*y+x);
	    }
	  break;     
	case MFAM_HOELDER:
	  *(tree->h_level+n)=(double*)level;
	  break;    
	case MFAM_SET: 
	  if((*(tree->s_level+n)!=NULL)&&(level!=NULL))
  	    {     
	      unsigned char* tmp_set2d=NULL;
	      unsigned char* tmp_level2d=NULL;	    
	      tmp_set2d=(unsigned char*)MFAM_qtree_level_get(tree,MFAM_SET,n);
	      tmp_level2d=(unsigned char*)level;
	      memset(tmp_set2d,0,(size_t)(two_pow_n*two_pow_n*sizeof(unsigned char)));
	      for(y=0;y<Ny;y++)
		for(x=0;x<Nx;x++)
		  *(tmp_set2d+two_pow_n*y+x)=(unsigned char)*(tmp_level2d+Nx*y+x);
	    }
	  else
	    fprintf(stderr,"error\n");
	  break;   
	default:
	  fprintf(stderr,"MFAM_qtree_level_convert arguments error\n");
	  break;
	}
      return 1;
    }
  else
    {
      fprintf(stderr,"MFAM_qtree_level_convert arguments error\n");
      return 0;
   } 
}

/*****************************************************************************/

void* MFAM_qtree_level_get(MFAMt_qtree* tree,MFAMt_level level_type,short n)
{
  if((tree!=NULL)||(n>0))
    { 
      void* level=NULL;
      switch (level_type)
	{	
	case MFAM_CAPACITY:
	  level=(void*)*(tree->c_level+n); 
	  break; 
	case MFAM_REFERENCE:
	  level=(void*)*(tree->r_level+n); 
	  break;    
	case MFAM_HOELDER:
	  level=(void*)*(tree->h_level+n);
	  break;    
	case MFAM_SET:  
	  level=(void*)*(tree->s_level+n);
	  break;   
	default:
	  fprintf(stderr,"MFAM_qtree_level_get arguments error\n");
	  break;
	}
      return level;
    }
  else
    {
      fprintf(stderr,"MFAM_qtree_level_get arguments error\n");
      return NULL;
   }   
}

/*****************************************************************************/

int MFAM_qtree_level_print(MFAMt_qtree *tree,MFAMt_level level_type,short n_ini,short n_fin)
{ 
  if(tree!=NULL)
    {
     int n=n_ini;
     switch (level_type)
	{	
	case MFAM_CAPACITY:
	  while(n<=n_fin)
	    {
	      int k=0;
	      int four_pow_n=(int)pow(4.,(double)n); 
	      fprintf(stderr,"printing capacities of level # %d\n",n);
	      for(k=0;k<four_pow_n;k++)
		fprintf(stderr,"%d %g\n",k,*(*(tree->c_level+n)+k));
	      n++;
	    }
	  break;  
	case MFAM_REFERENCE:
	  while(n<=n_fin)
	    {
	      int k=0; 
	      int four_pow_n=(int)pow(4.,(double)n); 
	      fprintf(stderr,"printing reference measures of level # %d\n",n);
	      for(k=0;k<four_pow_n;k++)
		fprintf(stderr,"%d %g\n",k,*(*(tree->r_level+n)+k));
	      n++;
	    }
	  break;  
	case MFAM_HOELDER:
	  while(n<=n_fin)
	    {
	      int k=0; 
	      int four_pow_n=(int)pow(4.,(double)n); 
	      fprintf(stderr,"printing hoelder exponents of level # %d\n",n);
	      for(k=0;k<four_pow_n;k++)
		fprintf(stderr,"%d %g\n",k,*(*(tree->h_level+n)+k));
	      n++;
	    }
	  break;   
	case MFAM_SET:
	  while(n<=n_fin)
	    {
	      int k=0; 
	      int four_pow_n=(int)pow(4.,(double)n); 
	      fprintf(stderr,"printing set of level # %d\n",n);
	      for(k=0;k<four_pow_n;k++)
		fprintf(stderr,"%d %d\n",k,*(*(tree->s_level+n)+k));
	      n++;
	    }
	  break;
	default:
	  fprintf(stderr,"MFAM_qtree_level_print arguments error\n");
	  break;
	}
      return 1;
    }
  else
    { 
      fprintf(stderr,"MFAM_qtree_level_print error\n");
      return 0;
    }
}

/*****************************************************************************/

double MFAM_qnode_cget(MFAMt_qnode* node,MFAMt_qtree* tree)
{
  int two_pow_n=(int)pow(2.,(double)node->n);
  if((*(tree->c_level+node->n)!=NULL)&&(*(tree->c_level+node->n)+two_pow_n*node->j+node->i!=NULL))
    return *(*(tree->c_level+node->n)+two_pow_n*node->j+node->i);
  else
    {
      fprintf(stderr,"MFAM_qnode_cget error\n");
      return 0.;
    }
}

/*****************************************************************************/

void MFAM_qnode_cset(MFAMt_qnode* node,MFAMt_qtree* tree,double c)
{  
  int two_pow_n=(int)pow(2.,(double)node->n);
  if((*(tree->c_level+node->n)!=NULL)&&(*(tree->c_level+node->n)+two_pow_n*node->j+node->i!=NULL))
    *(*(tree->c_level+node->n)+two_pow_n*node->j+node->i)=c;
  else 
    fprintf(stderr,"MFAM_qnode_cset error\n");
}

/*****************************************************************************/

double MFAM_qnode_rget(MFAMt_qnode* node,MFAMt_qtree* tree)
{
  int two_pow_n=(int)pow(2.,(double)node->n);
  if((*(tree->r_level+node->n)!=NULL)&&(*(tree->r_level+node->n)+two_pow_n*node->j+node->i!=NULL))
    return *(*(tree->r_level+node->n)+two_pow_n*node->j+node->i);
  else
    {
      fprintf(stderr,"MFAM_qnode_rget error\n");
      return 0.;
    }
}

/*****************************************************************************/

double MFAM_qnode_rset(MFAMt_qnode* node,MFAMt_qtree* tree,double r)
{  
  int two_pow_n=(int)pow(2.,(double)node->n);
  if((*(tree->r_level+node->n)!=NULL)&&(*(tree->r_level+node->n)+two_pow_n*node->j+node->i!=NULL))
    return (*(*(tree->r_level+node->n)+two_pow_n*node->j+node->i)=r);
  else
    {
      fprintf(stderr,"MFAM_qnode_rset error\n");
      return 0.;
    }
}

/*****************************************************************************/

double MFAM_qnode_hget(MFAMt_qnode* node,MFAMt_qtree* tree)
{
  double h=0.;  
  int two_pow_n=(int)pow(2.,(double)node->n);
  if ((*(tree->h_level+node->n)!=NULL)&&(*(tree->h_level+node->n)+two_pow_n*node->j+node->i!=NULL))
    h=*(*(tree->h_level+node->n)+two_pow_n*node->j+node->i);
  else
     fprintf(stderr,"MFAM_qnode_hget error\n");
  return h;
}

/*****************************************************************************/

void MFAM_qnode_hset(MFAMt_qnode* node,MFAMt_qtree* tree,double h)
{  
  int two_pow_n=(int)pow(2.,(double)node->n);
  if((*(tree->h_level+node->n)!=NULL)&&(*(tree->h_level+node->n)+two_pow_n*node->j+node->i!=NULL))
    *(*(tree->h_level+node->n)+two_pow_n*node->j+node->i)=h;
  else 
    fprintf(stderr,"MFAM_qnode_hset error\n");
}

/*****************************************************************************/

unsigned char MFAM_qnode_sget(MFAMt_qnode* node,MFAMt_qtree* tree)
{
  unsigned char s=0;
  int two_pow_n=pow(2.,(double)node->n);
  if((*(tree->s_level+node->n)!=NULL)&&(*(tree->s_level+node->n)+two_pow_n*node->j+node->i!=NULL))
    s=*(*(tree->s_level+node->n)+two_pow_n*node->j+node->i);
  else
     fprintf(stderr,"MFAM_qnode_sget error\n");
  return s;
}

/*****************************************************************************/

void MFAM_qnode_sset(MFAMt_qnode* node,MFAMt_qtree* tree, unsigned char s)
{
  int two_pow_n=(int)pow(2.,(double)node->n);
  if((*(tree->s_level+node->n)!=NULL)&&(*(tree->s_level+node->n)+two_pow_n*node->j+node->i!=NULL))
    *(*(tree->s_level+node->n)+two_pow_n*node->j+node->i)=s;
  else 
    fprintf(stderr,"MFAM_qnode_sset error\n");
}

/*****************************************************************************/

double MFAM_qnode_c_level_decimate(MFAMt_qnode* node,MFAMt_qtree* tree,double (*capa)(double,double),short n_ini,short n_fin)
{
  if((node!=NULL)&&(tree!=NULL))
    {
      if(node->n<n_ini)
	{
	  MFAM_qnode_c_level_decimate(node->cnode,tree,capa,n_ini,n_fin);
	  MFAM_qnode_c_level_decimate(node->hnode,tree,capa,n_ini,n_fin);
	  MFAM_qnode_c_level_decimate(node->vnode,tree,capa,n_ini,n_fin);
	  MFAM_qnode_c_level_decimate(node->hvnode,tree,capa,n_ini,n_fin);
	  return 0.;
	}
      else if(node->n<n_fin)
	{
	  double c=0.;	 
	  double c_tmp=0.; 
	  double sum_c=0.; 
	  sum_c+=c_tmp=MFAM_qnode_c_level_decimate(node->cnode,tree,capa,n_ini,n_fin);
	  c=c_ref=c_tmp;
	  sum_c+=c_tmp=MFAM_qnode_c_level_decimate(node->hnode,tree,capa,n_ini,n_fin);
	  c=capa(c,c_tmp); 
	  sum_c+=c_tmp=MFAM_qnode_c_level_decimate(node->vnode,tree,capa,n_ini,n_fin);
	  c=capa(c,c_tmp);
	  sum_c+=c_tmp=MFAM_qnode_c_level_decimate(node->hvnode,tree,capa,n_ini,n_fin);
	  c=capa(c,c_tmp);
	  MFAM_qnode_cset(node,tree,c);
	  return sum_c;
	}
      else /* if(node->n=n_fin) */
	return MFAM_qnode_cget(node,tree);
    }
  else
    {
      fprintf(stderr,"MFAM_qnode_c_level_decimate arguments error\n");
      return 0.;
    }
}

/*****************************************************************************/

int MFAM_qtree_c_level_decimate(MFAMt_qtree* tree,MFAMt_capa t_capa,short n_ini,short n_fin)
{
 if(tree!=NULL)
   {  

#ifndef __STDC__
     double (*capa)(); 
#else /* __STDC__ */
     double (*capa)(double,double);
#endif /* __STDC__ */

     switch(t_capa)
       { 
       case MFAM_SUM:
	 norm_meas=1.;
	 capa=MFAM_sum;
	 break; 
       case MFAM_LPSUM:
	 norm_meas=1.;
	 p=2;
	 capa=MFAM_lpsum;
	 break;
       case MFAM_MIN:
	 capa=MFAM_min;
	 break;   
       case MFAM_MAX:
	 capa=MFAM_max;
	 break; 
       case MFAM_ISO:
	 {
	   double Q=20;
	   double c_min=0,c_max=-HUGE;
	   int four_pow_n=(int)pow(4.,(double)n_fin);
	   double* c=NULL;
	   c=(double*)MFAM_qtree_level_get(tree,MFAM_CAPACITY,n_fin);
	   if(MFAM_min_max(four_pow_n,c,&c_min,&c_max)==0)
	     { 
	       fprintf(stderr,"MFAM_min_fin error\n"); 
	       return 0; 
	     }
	   q=(c_max-c_min)/Q;
#ifdef MFAMDEBUG 
	   fprintf(stderr,"c_min=%e\n",c_min); 
	   fprintf(stderr,"c_max=%e\n",c_max);
	   fprintf(stderr,"q=%e\n",q);
#endif /* MFAMDEBUG */
	   capa=MFAM_iso;
	   break;
	 }
       case MFAM_ADAPISO:
	 {
	   int four_pow_n=(int)pow(4.,(double)n_fin);
	   double* c=NULL;
	   c=(double*)MFAM_qtree_level_get(tree,MFAM_CAPACITY,n_fin);
	   if((sigma2=MFAM_variance(four_pow_n,c,MFAM_mean(four_pow_n,c)))==HUGE)
	     {
	       fprintf(stderr,"MFAM_variance error\n");
	       return 0;
	     }
#ifdef MFAMDEBUG 
	   fprintf(stderr,"mean_c=%e\n",MFAM_mean(four_pow_n,c)); 
	   fprintf(stderr,"sigma2=%e\n",sigma2);
#endif /* MFAMDEBUG */ 
	   capa=MFAM_adapiso;
	   break;
	 } 
       default:
	 return 0;
       } 
     MFAM_qnode_c_level_decimate(tree->root,tree,capa,n_ini,n_fin);
     return 1;
   }
 else
   {
      fprintf(stderr,"MFAM_qtree_c_level_decimate arguments error\n");
      return 0;
   } 
}

/*****************************************************************************/

double MFAM_qnode_r_level_decimate(MFAMt_qnode* node,MFAMt_qtree* tree,short n_ini,short n_fin)
{
  if((node!=NULL)&&(tree!=NULL))
    {
      if(node->n<n_ini)
	{
	  MFAM_qnode_r_level_decimate(node->cnode,tree,n_ini,n_fin);
	  MFAM_qnode_r_level_decimate(node->hnode,tree,n_ini,n_fin);
	  MFAM_qnode_r_level_decimate(node->vnode,tree,n_ini,n_fin);
	  MFAM_qnode_r_level_decimate(node->hvnode,tree,n_ini,n_fin);
	  return 0.;
	}
      else if(node->n<n_fin)
	return MFAM_qnode_rset(node,tree,
			       MFAM_qnode_r_level_decimate(node->cnode,tree,n_ini,n_fin)+
			       MFAM_qnode_r_level_decimate(node->hnode,tree,n_ini,n_fin)+
			       MFAM_qnode_r_level_decimate(node->vnode,tree,n_ini,n_fin)+
			       MFAM_qnode_r_level_decimate(node->vnode,tree,n_ini,n_fin));
      else /* if(node->n=n_fin) */
	return MFAM_qnode_rget(node,tree);
    }
  else
    {
      fprintf(stderr,"MFAM_qnode_r_level_decimate arguments error\n");
      return 0.;
    }
}

/*****************************************************************************/

int MFAM_qtree_r_level_decimate(MFAMt_qtree* tree,short n_ini,short n_fin)
{
 if(tree!=NULL)
   {  
     MFAM_qnode_r_level_decimate(tree->root,tree,n_ini,n_fin);
     return 1;
   }
 else
   {
     fprintf(stderr,"MFAM_qtree_r_level_decimate arguments error\n");
     return 0;
   } 
}

/*****************************************************************************/

unsigned char MFAM_qnode_s_level_decimate(MFAMt_qnode* node,MFAMt_qtree* tree,short n_ini,short n_fin)
{
  if((node!=NULL)&&(tree!=NULL))
    {
      if(node->n<n_ini)
	{ 
	  MFAM_qnode_s_level_decimate(node->cnode,tree,n_ini,n_fin);
	  MFAM_qnode_s_level_decimate(node->hnode,tree,n_ini,n_fin);
	  MFAM_qnode_s_level_decimate(node->vnode,tree,n_ini,n_fin);
	  MFAM_qnode_s_level_decimate(node->hvnode,tree,n_ini,n_fin);
	  return 0;
	}
      else if(node->n<n_fin)
	{
	  unsigned char s=0;
	  if(MFAM_qnode_s_level_decimate(node->cnode,tree,n_ini,n_fin))
	    s=(unsigned char)1; 
	  if(MFAM_qnode_s_level_decimate(node->hnode,tree,n_ini,n_fin))
	    s=(unsigned char)1; 
	  if(MFAM_qnode_s_level_decimate(node->vnode,tree,n_ini,n_fin))
	    s=(unsigned char)1; 
	  if(MFAM_qnode_s_level_decimate(node->hvnode,tree,n_ini,n_fin))
	    s=(unsigned char)1;
	  MFAM_qnode_sset(node,tree,s);
	  return s;
	}
      else /* if(node->n=n_fin) */
	return MFAM_qnode_sget(node,tree);
    }
  else
    {
      fprintf(stderr,"MFAM_qnode_s_level_decimate arguments error\n");
      return 0;
    }
}

/*****************************************************************************/

int MFAM_qtree_s_level_decimate(MFAMt_qtree* tree,short n_ini,short n_fin)
{
  if(tree!=NULL)
    {  
      MFAM_qnode_s_level_decimate(tree->root,tree,n_ini,n_fin);
      return 1;
    }
  else
    {
      fprintf(stderr,"MFAM_qtree_s_level_decimate arguments error\n");
      return 0;
    } 
}

/*****************************************************************************/

MFAMt_qtree* MFAM_qtree_s_level_init(int Nx,int Ny,unsigned char* s2d,void* tree,short* n_opt)
{
  if((Nx>0)&&(Ny>0)&&(s2d!=NULL)&&(0<=*n_opt))
    { 
      int two_pow_n=0;
      MFAMt_qtree* s_tree=NULL;

      /* find optimal resolution: n */ 
      if(*n_opt==0)
	*n_opt=(short)ceil(log((double)MAX(Nx,Ny))/log(2.));

#ifdef MFAMDEBUG	
      fprintf(stderr,"Nx=%d\n",Nx); 
      fprintf(stderr,"Ny=%d\n",Ny);
      fprintf(stderr,"optimal resolution: n_opt=%d\n",*n_opt);
#endif /* MFAMDEBUG */

      if(tree==NULL) /* tree's not already created */
	{  
	
#ifdef MFAMDEBUG
	  fprintf(stderr,"tree's not already created\n"); 
#endif /* MFAMDEBUG */

	  s_tree=MFAM_qtree_create(*n_opt);

#ifdef MFAMDEBUG
	  fprintf(stderr,"tree's created\n");
#endif /* MFAMDEBUG */

	  two_pow_n=(int)pow(2.,(double)*n_opt);
	  if((Nx==two_pow_n)||(Ny==two_pow_n))
	    {  
	      fprintf(stderr,"tree set\n");

	      MFAM_qtree_level_create(s_tree,MFAM_SET,0,*n_opt-1); 
	      MFAM_qtree_level_set(s_tree,MFAM_SET,*n_opt,(void*)s2d);
	    }
	  else
	    {  
	      fprintf(stderr,"tree convert\n");

	      MFAM_qtree_level_create(s_tree,MFAM_SET,0,*n_opt);
	      MFAM_qtree_level_convert(s_tree,MFAM_SET,*n_opt,Nx,Ny,(void*)s2d);
	    }
	 
	  MFAM_qtree_s_level_decimate(s_tree,0,*n_opt); 
	}  
      else /* tree's already created */
	{ 

#ifdef MFAMDEBUG
	  fprintf(stderr,"tree's already created\n");
#endif /* MFAMDEBUG */

	  /* pointers' affectation */
	  s_tree=(MFAMt_qtree*)tree;
	}
      return s_tree;
    }
  else
    {
      fprintf(stderr,"MFAM_qtree_s_level_init arguments error\n");
      fprintf(stderr,"\tNx=[7m%d[m\n",Nx);
      fprintf(stderr,"\tNy=[7m%d[m\n",Ny);
      fprintf(stderr,"\ts2d=[7m%p[m\n",s2d);
      fprintf(stderr,"\ttree=[7m%p[m\n",tree);
      fprintf(stderr,"\tn_opt=[7m%p[m\n",n_opt);  
      return NULL;
    }
}

/*****************************************************************************/

int MFAM_qtree_s_level_update(int Nx,int Ny,unsigned char* s2d,MFAMt_qtree* tree,short* n_opt)
{
  if((Nx>0)&&(Ny>0)&&(s2d!=NULL)&&(0<=*n_opt))
    { 
      int two_pow_n=0;
      two_pow_n=(int)pow(2.,(double)*n_opt);
      if((Nx==two_pow_n)||(Ny==two_pow_n)) 
	{  
	  MFAM_qtree_level_set(tree,MFAM_SET,*n_opt,(void*)NULL);
	  MFAM_qtree_level_set(tree,MFAM_SET,*n_opt,(void*)s2d);
	}
      else
	MFAM_qtree_level_convert(tree,MFAM_SET,*n_opt,Nx,Ny,(void*)s2d);
      
      MFAM_qtree_s_level_decimate(tree,0,*n_opt); 
 
#ifdef MFAMDEBUG
      fprintf(stderr,"tree's udpated\n"); 
#endif /* MFAMDEBUG */ 

      return 1;
    } 
  else
    {
      fprintf(stderr,"MFAM_qtree_s_level_update arguments error\n");
      fprintf(stderr,"\tNx=[7m%d[m\n",Nx); 
      fprintf(stderr,"\tNy=[7m%d[m\n",Ny);
      fprintf(stderr,"\ts2d=[7m%p[m\n",s2d);
      fprintf(stderr,"\ttree=[7m%p[m\n",tree);
      fprintf(stderr,"\tn_opt=[7m%p[m\n",n_opt);  
      return 0;
    }
}

/*****************************************************************************/

int MFAM_qtree_s_level_clear(int Nx,int Ny,unsigned char* s2d,MFAMt_qtree* s_tree,void* tree,short* n_opt)
{
  if((Nx>0)&&(Ny>0)&&(s2d!=NULL)&&(s_tree!=NULL)&&(0<=*n_opt))
    {  
      if(tree==NULL) /* tree was not already created */
	{ 
	  int two_pow_n=0; 
	  two_pow_n=(int)pow(2.,(double)*n_opt);
	  if((Nx==two_pow_n)||(Ny==two_pow_n))
	    {
	      /* set levels deletion */  
	      MFAM_qtree_level_set(s_tree,MFAM_SET,*n_opt,(void*)NULL); 
	      MFAM_qtree_level_delete(s_tree,MFAM_SET,0,*n_opt-1);
	    }
	  else
	    MFAM_qtree_level_delete(s_tree,MFAM_SET,0,*n_opt);
	  
	  /* tree deletion */
	  MFAM_qtree_delete(s_tree);
	  s_tree=NULL; 

#ifdef MFAMDEBUG
	  fprintf(stderr,"tree's cleared\n"); 
#endif /* MFAMDEBUG */ 
	} 
      else /* tree was already created */
	s_tree=NULL;
      return 1;
    }
  else
    {
      fprintf(stderr,"MFAM_qtree_s_level_clear arguments error\n");  
      fprintf(stderr,"\tNx=[7m%d[m\n",Nx);
      fprintf(stderr,"\tNy=[7m%d[m\n",Ny);
      fprintf(stderr,"\ts2d=[7m%p[m\n",s2d);
      fprintf(stderr,"\ts_tmp=[7m%p[m\n",s_tree);
      fprintf(stderr,"\ttree=[7m%p[m\n",tree);
      fprintf(stderr,"\tn_opt=[7m%p[m\n",n_opt);  
     return 0;
    }
}
