/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/FRACLAB-WKSP/FRACLAB/MFAM/SRC/MFAM_qtree.h
* File type        : C header file
* Revision         : $Revision$
* RCS status       : $Id$
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sat Mar 30 18:57:23 2002
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Wed Nov 20 23:10:51 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/
/* Christophe Canus 02 */

#ifndef _MFAM_qtree_h_
#define _MFAM_qtree_h_
#include "MFAM_include.h"
#include "MFAM_misc.h"
#include "MFAM_stats.h"
#include "MFAM_btree.h"
#include "MFAM_capacity.h"

typedef struct _qnode
{
  char* x;
  short n;
  int i;
  int j;
  struct _qnode* cnode;
  struct _qnode* hnode;
  struct _qnode* vnode;
  struct _qnode* hvnode;
} MFAMt_qnode;

typedef struct _qtree
{
  short N;
  double** c_level;
  double** r_level;
  double** h_level;
  unsigned char** s_level;
  MFAMt_qnode* root;
} MFAMt_qtree;

#ifndef __STDC__ 
MFAMt_qnode* MFAM_qnode_create();
void MFAM_qnode_delete();
void MFAM_qnode_print();
#else /* __STDC__ */
MFAMt_qnode* MFAM_qnode_create(MFAMt_qnode*,char*,short,int,int,short);
void MFAM_qnode_delete(MFAMt_qnode*);
void MFAM_qnode_print(MFAMt_qnode*,int);
#endif /* __STDC__ */

#ifndef __STDC__
MFAMt_qtree* MFAM_qtree_create();
void MFAM_qtree_print();
void MFAM_qtree_delete();
#else /* __STDC__ */
MFAMt_qtree* MFAM_qtree_create(short);
void MFAM_qtree_print(MFAMt_qtree*);
void MFAM_qtree_delete(MFAMt_qtree*);
#endif /* __STDC__ */

#ifndef __STDC__
int MFAM_qtree_level_create();
int MFAM_qtree_level_delete();
int MFAM_qtree_level_set();
int MFAM_qtree_level_convert();
void* MFAM_qtree_level_get();
int MFAM_qtree_level_print();
#else /* __STDC__ */
int MFAM_qtree_level_create(MFAMt_qtree*,MFAMt_level,short,short);
int MFAM_qtree_level_delete(MFAMt_qtree*,MFAMt_level,short,short);
int MFAM_qtree_level_set(MFAMt_qtree*,MFAMt_level,short,void*);
int MFAM_qtree_level_convert(MFAMt_qtree*,MFAMt_level,short,int,int,void*);
void* MFAM_qtree_level_get(MFAMt_qtree*,MFAMt_level,short);
int MFAM_qtree_level_print(MFAMt_qtree*,MFAMt_level,short,short);
#endif /* __STDC__ */

#ifndef __STDC__
double MFAM_qnode_cget();
void MFAM_qnode_cset();
double MFAM_qnode_rget();
double MFAM_qnode_rset();
double MFAM_qnode_hget();
void MFAM_qnode_hset();
unsigned char MFAM_qnode_sget();
void MFAM_qnode_sset();
#else /* __STDC__ */
double MFAM_qnode_cget(MFAMt_qnode*,MFAMt_qtree*);
void MFAM_qnode_cset(MFAMt_qnode*,MFAMt_qtree*,double);
double MFAM_qnode_rget(MFAMt_qnode*,MFAMt_qtree*);
double MFAM_qnode_rset(MFAMt_qnode*,MFAMt_qtree*,double);
double MFAM_qnode_hget(MFAMt_qnode*,MFAMt_qtree*);
void MFAM_qnode_hset(MFAMt_qnode*,MFAMt_qtree*,double);
unsigned char MFAM_qnode_sget(MFAMt_qnode*,MFAMt_qtree*);
void MFAM_qnode_sset(MFAMt_qnode*,MFAMt_qtree*,unsigned char);
#endif /* __STDC__ */

#ifndef __STDC__
double MFAM_qnode_c_level_decimate();
int MFAM_qtree_c_level_decimate();
double MFAM_qnode_r_level_decimate();
int MFAM_qtree_r_level_decimate();
unsigned char MFAM_qnode_s_level_decimate();
int MFAM_qtree_s_level_decimate();
#else /* __STDC__ */
double MFAM_qnode_c_level_decimate(MFAMt_qnode*,MFAMt_qtree*,double (*)(double,double),short,short);
int MFAM_qtree_c_level_decimate(MFAMt_qtree*,MFAMt_capa,short,short);
double MFAM_qnode_r_level_decimate(MFAMt_qnode*,MFAMt_qtree*,short,short);
int MFAM_qtree_r_level_decimate(MFAMt_qtree*,short,short);
unsigned char MFAM_qnode_s_level_decimate(MFAMt_qnode*,MFAMt_qtree*,short,short);
int MFAM_qtree_s_level_decimate(MFAMt_qtree*,short,short);
#endif /* __STDC__ */

#ifndef __STDC__
MFAMt_qtree* MFAM_qtree_c_level_init();
int MFAM_qtree_c_level_update();
int MFAM_qtree_c_level_clear();
MFAMt_qtree* MFAM_qtree_s_level_init();
int MFAM_qtree_s_level_update();
int MFAM_qtree_s_level_clear();
#else /* __STDC__ */
MFAMt_qtree* MFAM_qtree_c_level_init(int,int,unsigned char*,void*,short*);
int MFAM_qtree_c_level_update(int,int,unsigned char*,MFAMt_qtree*,short*);
int MFAM_qtree_c_level_clear(int,int,unsigned char*,MFAMt_qtree*,void*,short*);
MFAMt_qtree* MFAM_qtree_s_level_init(int,int,unsigned char*,void*,short*);
int MFAM_qtree_s_level_update(int,int,unsigned char*,MFAMt_qtree*,short*);
int MFAM_qtree_s_level_clear(int,int,unsigned char*,MFAMt_qtree*,void*,short*);
#endif /* __STDC__ */

#endif /* _MFAM_qtree_h_ */
