/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/canus/MFA.CMP/MFAM.CL/src/MFAM_stats.c
* File type        : C source file
* Revision         : $Revision: 1.1.1.1 $
* RCS status       : $Id: MFAM_stats.c,v 1.1.1.1 1999/02/10 13:29:04 guiheneu Exp $
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sun Feb  7 15:53:42 1999
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Sun Feb  7 15:53:43 1999
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 1997-98 */

#include "MFAM_stats.h"

/*****************************************************************************/

double MFAM_mean(int n,double* x)
{
  if((n>0)&&(x!=NULL))
  {
    register int k=0;
    double m_x=0.;
    for(k=0;k<n;k++)
      m_x+=*(x+k);
    m_x*=1./(double)n; /* biased estimator */
    return m_x;
  }
  else
  {
    fprintf(stderr,"MFAM_mean arguments error\n");
    return HUGE;
  }
}

/*****************************************************************************/

double MFAM_variance(int n,double* x,double m_x)
{
  if((n>0)&&(x!=NULL)&&(m_x!=HUGE))
  {
    register int k=0;
    double sigma_2_x=0.;
    for(k=0;k<n;k++) 
      sigma_2_x+=(*(x+k)-m_x)*(*(x+k)-m_x); 
    sigma_2_x*=1./(double)n; /* biased estimator */
    return sigma_2_x;
  }
  else
  {
    fprintf(stderr,"MFAM_variance arguments error\n");
    return HUGE;
  }
}

/*****************************************************************************/
double MFAM_covariance(int n,double* x,double* y,double m_x,double m_y)
{
  if((n>0)&&(x!=NULL)&&(y!=NULL)&&(m_x!=HUGE)&&(m_y!=HUGE))
  {
    register int k=0;
    double sigma_2_xy=0.;
    for(k=0;k<n;k++)
      sigma_2_xy+=(*(x+k)-m_x)*(*(y+k)-m_y);
    sigma_2_xy*=1./(double)n; /* biased estimator */
    return sigma_2_xy;
  }
  else
  {
    fprintf(stderr,"MFAM_covariance arguments error\n");
    return HUGE;
  }
}

/*****************************************************************************/
double MFAM_bias(int n,double* theta_hat,double theta)
{
  if((n>0)&&(theta_hat!=NULL)&&(theta!=HUGE))
  {
    double bias_n=MFAM_mean(n,theta_hat)-theta;
    return bias_n;
  }
  else
  {
    fprintf(stderr,"MFAM_bias arguments error\n");
    return HUGE;
  }
}

/*****************************************************************************/
double MFAM_generalized_bias(int n,double* theta_hat,double theta)
{
  if((n>0)&&(theta_hat!=NULL)&&(theta!=HUGE))
  {
    double generalized_bias_n=MFAM_variance(n,theta_hat,theta);
    return generalized_bias_n;
  }
  else
  {
    fprintf(stderr,"MFAM_generalized_bias arguments error\n");
    return HUGE;
  }
}
