/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/canus/MFA.CMP/MFAS.CL/src/MFAS_kiesswetter.c
* File type        : C source file
* Revision         : $Revision: 1.1.1.1 $
* RCS status       : $Id: MFAS_kiesswetter.c,v 1.1.1.1 1999/02/10 13:43:28 guiheneu Exp $
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sun Feb  7 16:14:50 1999
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Sun Feb  7 16:14:51 1999
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

#include "MFAS_kiesswetter.h"

static void rkf(int,int,double,double,double,double*);

/***************/
int MFAS_kf(int n,double p,double *f_x)
{
  if((n>0)&&(p>0.)&&(f_x!=NULL))
  {
    register int k=0;
    int four_pow_n=(int)pow(4.,(double)n),cfour_pow_n=0;
    *(f_x+four_pow_n)=1.;
    while(n>0)
    { 
      n--;
      cfour_pow_n=(int)pow(4.,(double)n);
      for(k=cfour_pow_n;k<four_pow_n;k+=4*cfour_pow_n)
      {
	*(f_x+k)=*(f_x+k-cfour_pow_n)+p*(*(f_x+k+cfour_pow_n)-*(f_x+k-cfour_pow_n));
	*(f_x+k+1)=*(f_x+k)-(p*(*(f_x+k+cfour_pow_n)-*(f_x+k-cfour_pow_n)));
	*(f_x+k+2)=*(f_x+k+1)+p*(*(f_x+k+cfour_pow_n)-*(f_x+k-cfour_pow_n));
      }
    }
    return 1;
  }
  else
  {
    fprintf(stderr,"MFAS_kf arguments error\n");
    return 0;
  }
}


/*******************/
int MFAS_rkf(int n,double p,double *f_x)
{
  if((n>0)&&(p>0.)&&(f_x!=NULL))
  {
    int four_pow_n=(int)pow(4.,(double)n);
    *(f_x)=0.;
    *(f_x+four_pow_n)=1.;
    rkf(0,four_pow_n,p,0,1,f_x);
    return 1;
  }
  else
  {
    fprintf(stderr,"MFAS_rkf arguments error\n");
    return 0;
  }
}

/*****************/
static void rkf(int k,int nu_n,double p,double a,double b,double *f_x)
{
  if(nu_n>=4)
  {
    double slope=b-a;
    *(f_x+k+nu_n/4)=*(f_x+k)+p*slope;
    if(slope>0.)
    {
      *(f_x+k+2*nu_n/4)=*(f_x+k+nu_n/4)-p*slope; 
      *(f_x+k+3*nu_n/4)=*(f_x+k+2*nu_n/4)+p*slope;   
    }
    else
    {
      *(f_x+k+2*nu_n/4)=*(f_x+k+nu_n/4)+p*slope;  
      *(f_x+k+3*nu_n/4)=*(f_x+k+2*nu_n/4)-p*slope;
    }   
    rkf(k,nu_n/4,p,*(f_x+k),*(f_x+k+nu_n/4),f_x); 
    rkf(k+nu_n/4,nu_n/4,p,*(f_x+k+nu_n/4),*(f_x+k+2*nu_n/4),f_x); 
    rkf(k+2*nu_n/4,nu_n/4,p,*(f_x+k+2*nu_n/4),*(f_x+k+3*nu_n/4),f_x);  
    rkf(k+3*nu_n/4,nu_n/4,p,*(f_x+k+3*nu_n/4),*(f_x+k+4*nu_n/4),f_x);  
  }
}
