/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/MFA.CMP/MFAS.CL/src/MFAS_spectrum.c
* File type        : C source file
* Revision         : $Revision: 1.1.1.1 $
* RCS status       : $Id: MFAS_spectrum.c,v 1.1.1.1 1999/02/10 13:43:28 guiheneu Exp $
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sun Feb  7 16:13:26 1999
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Sun May  5 18:23:05 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

#include "MFAS_spectrum.h" 

/******************/
int MFAS_bm1ds(int n,double p0,double* alpha,double* f_alpha)
{
  if((n>0)&&(p0>0.)&&(alpha!=NULL)&&(f_alpha!=NULL))
  {
    register int i=0;
    double alpha_min=-log(1-p0)/log(2.);
    double alpha_max=-log(p0)/log(2.);
    double d_alpha=(alpha_max-alpha_min)/(n-1);
    double phi_alpha=0.;  
    for(i=0;i<n;i++)
    {
      *(alpha+i)=alpha_min+i*d_alpha; 
      phi_alpha=(*(alpha+i)+log(1-p0)/log(2.))/(log((1-p0)/p0)/log(2.));
      if((fabs(phi_alpha-0.)>1e-6)&&(fabs(phi_alpha-1.)>1e-6))
	*(f_alpha+i)=-phi_alpha*log(phi_alpha)/log(2.)-(1-phi_alpha)*log(1-phi_alpha)/log(2.); 
      else
	*(f_alpha+i)=0.;
    }
    return 1;
  }
  else
  {
    fprintf(stderr,"MFAS_bm1ds arguments error\n");
    return 0;
  }
}

/******************/
int MFAS_sumbm1ds(int n,double p01,double p02,double* alpha,double* f_alpha)
{
  if((n>0)&&(p01>0.)&&(p02>0.)&&(p01<p02)&&(alpha!=NULL)&&(f_alpha!=NULL))
  {
    register int i=0; 
    int toggle=1;
    double alpha_min_1=-log(1-p01)/log(2.),alpha_min_2=-log(1-p02)/log(2.),alpha_max_2=-log(p02)/log(2.);
    double d_alpha=(alpha_max_2-alpha_min_1)/(n-1),phi_alpha_1=0.,phi_alpha_2=0.,f_alpha_1=0.,f_alpha_2=0.;  
    for(i=0;i<n;i++)
    {
      *(alpha+i)=alpha_min_1+i*d_alpha; 
      if((alpha_min_1<=*(alpha+i))&&(*(alpha+i)<=alpha_min_2))
      {
	phi_alpha_1=(*(alpha+i)+log(1-p01)/log(2.))/(log((1-p01)/p01)/log(2.)); 
	if((fabs(phi_alpha_1-0.)>1e-6)&&(fabs(phi_alpha_1-1.)>1e-6))
	  f_alpha_1=-phi_alpha_1*log(phi_alpha_1)/log(2.)-(1-phi_alpha_1)*log(1-phi_alpha_1)/log(2.); 
	else
	  f_alpha_1=0.;
	*(f_alpha+i)=f_alpha_1;
      }
      else 
	if((alpha_min_2<*(alpha+i))&&(*(alpha+i)<=alpha_max_2))
	{   
	  if(toggle)
	  {
	    phi_alpha_1=(*(alpha+i)+log(1-p01)/log(2.))/(log((1-p01)/p01)/log(2.)); 
	    if((fabs(phi_alpha_1-0.)>1e-6)&&(fabs(phi_alpha_1-1.)>1e-6))
	      f_alpha_1=-phi_alpha_1*log(phi_alpha_1)/log(2.)-(1-phi_alpha_1)*log(1-phi_alpha_1)/log(2.); 
	    else
	      f_alpha_1=0.; 
	    phi_alpha_2=(*(alpha+i)+log(1-p02)/log(2.))/(log((1-p02)/p02)/log(2.));
	    if((fabs(phi_alpha_2-0.)>1e-6)&&(fabs(phi_alpha_2-1.)>1e-6))
	      f_alpha_2=-phi_alpha_2*log(phi_alpha_2)/log(2.)-(1-phi_alpha_2)*log(1-phi_alpha_2)/log(2.); 
	    else
	      f_alpha_2=0.;
	    *(f_alpha+i)=(f_alpha_1>f_alpha_2?f_alpha_1:f_alpha_2);
	    if(f_alpha_1<f_alpha_2)
	      toggle=0;
	  }   
	  else
	  {
	    phi_alpha_2=(*(alpha+i)+log(1-p02)/log(2.))/(log((1-p02)/p02)/log(2.));
	    if((fabs(phi_alpha_2-0.)>1e-6)&&(fabs(phi_alpha_2-1.)>1e-6))
	      f_alpha_2=-phi_alpha_2*log(phi_alpha_2)/log(2.)-(1-phi_alpha_2)*log(1-phi_alpha_2)/log(2.); 
	    else
	      f_alpha_2=0.;
	    *(f_alpha+i)=f_alpha_2;
	  }
	}
    }   
    return 1;
  }
  else
  {
    fprintf(stderr,"MFAS_sumbm1ds arguments error\n");
    return 0;
  }
}

/******************/
int MFAS_lumpbm1ds(int n,double p01,double p02,double* alpha,double* f_alpha)
{
   if((n>0)&&(p01>0.)&&(p02>0.)&&(alpha!=NULL)&&(f_alpha!=NULL))
   {
   register int i=0;
   double alpha_min_1=-log(1-p01)/log(2.),alpha_max_1=-log(p01)/log(2.),alpha_min_2=-log(1-p02)/log(2.),alpha_max_2=-log(p02)/log(2.);
   double alpha_min=(alpha_min_1<alpha_min_2?alpha_min_1:alpha_min_2),alpha_max=(alpha_max_1>alpha_max_2?alpha_max_1:alpha_max_2);
   double d_alpha=(alpha_max-alpha_min)/(n-1),phi_alpha_1=0.,phi_alpha_2=0.,f_alpha_1=0.,f_alpha_2=0.;  
   for(i=0;i<n;i++)
   {
      *(alpha+i)=alpha_min+i*d_alpha; 
      if((alpha_min_1<=*(alpha+i))&&(*(alpha+i)<=alpha_max_1))
      {
	 phi_alpha_1=(*(alpha+i)+log(1-p01)/log(2.))/(log((1-p01)/p01)/log(2.)); 
	 if((fabs(phi_alpha_1-0.)>1e-6)&&(fabs(phi_alpha_1-1.)>1e-6))
	    f_alpha_1=-phi_alpha_1*log(phi_alpha_1)/log(2.)-(1-phi_alpha_1)*log(1-phi_alpha_1)/log(2.); 
	 else
	   f_alpha_1=0.;
      }  
      else
	 f_alpha_1=0.;
      if((alpha_min_2<=*(alpha+i))&&(*(alpha+i)<=alpha_max_2))
      {  
	 phi_alpha_2=(*(alpha+i)+log(1-p02)/log(2.))/(log((1-p02)/p02)/log(2.));
	 if((fabs(phi_alpha_2-0.)>1e-6)&&(fabs(phi_alpha_2-1.)>1e-6))
	    f_alpha_2=-phi_alpha_2*log(phi_alpha_2)/log(2.)-(1-phi_alpha_2)*log(1-phi_alpha_2)/log(2.); 
	 else
	    f_alpha_2=0.;
      }   
      else
	 f_alpha_2=0.;
      *(f_alpha+i)=(f_alpha_1>f_alpha_2?f_alpha_1:f_alpha_2);
   }
   return 1;
   }
   else
   {
      fprintf(stderr,"MFAS_lumpbm1ds arguments error\n");
      return 0;
   }
}

/******************/
int MFAS_ubm1ds(int n,double* alpha,double* f_alpha)
{
   if((n>0)&&(alpha!=NULL)&&(f_alpha!=NULL))
   {
   register int i=0;
   double alpha_min=.3346,alpha_max=3.3864,d_alpha=(alpha_max-alpha_min)/(n-1),rho_alpha=0.,delta=log(2.);  
   for(i=0;i<n;i++)
   {
      *(alpha+i)=alpha_min+i*d_alpha;
      rho_alpha=log(*(alpha+i)*delta)/log(2.)-*(alpha+i)+1./delta;
      *(f_alpha+i)=1.+rho_alpha;
   }
   return 1;
   }
   else
   {
      fprintf(stderr,"MFAS_ubm1ds arguments error\n");
      return 0;
   }
}

/******************/
int MFAS_lnm1ds(int n,double nu,double* alpha,double* f_alpha)
{
   if((n>0)&&(nu>0.)&&(alpha!=NULL)&&(f_alpha!=NULL))
   {
   register int i=0;
   double alpha_min=-3.,alpha_max=+7.,d_alpha=(alpha_max-alpha_min)/(n-1),rho_alpha=0.;  
   for(i=0;i<n;i++)
   {
      if((*(alpha+i)=alpha_min+i*d_alpha)<-1.+nu)
	 rho_alpha=*(alpha+i)/nu-1.;
      else
	 rho_alpha=-(*(alpha+i)-1.-nu)*(*(alpha+i)-1.-nu)/(4.*nu);
      *(f_alpha+i)=1.+rho_alpha;
   }
   return 1;
   }
   else
   {
      fprintf(stderr,"MFAS_lnm1ds arguments error\n");
      return 0;
   }
}

/*****************************************************************************/

int MFAS_bmc3s(int n,double p0,double* alpha,double* f_alpha)
{
  if((n>0)&&(n/2!=0)&&(p0>0.)&&(p0<.5)&&(alpha!=NULL)&&(f_alpha!=NULL))
  {
    register int k=0;
    int m=(n-9)/2;
    double p1=1.-p0;
    double alpha_min=-log(pow(p1,3.))/(3*log(2.));
    double beta_3_1=-log(p0*pow(p1,2.))/(3*log(2.));
    double beta_3_3=-log(pow(p0,2.)*p1)/(3*log(2.)); 
    double alpha_max=-log(pow(p0,3.))/(3*log(2.));
    double alpha_0=-log(.5*p0*p1)/(3*log(2.));
    double d_alpha=0.;
    *alpha=alpha_min;
    *f_alpha=0.;
    *(alpha+1)=beta_3_1;
    *(f_alpha+1)=0.; 
    *(alpha+2)=beta_3_1;
    *(f_alpha+2)=log(3.)/(3*log(2.)); 
    *(alpha+3)=beta_3_1;
    *(f_alpha+3)=1./3.;
    d_alpha=(alpha_0-beta_3_1)/m;
    for(k=4;k<(n-1)/2;k++)
    {
      *(alpha+k)=beta_3_1+(k-4)*d_alpha;
      *(f_alpha+k)=1./3.+1./3.*log(1+(pow(2.,-3.**(alpha+k))-p0*pow(p1,2.))/(p0*p1*(p0-p1)))/log(2.);
    }
    *(alpha+(n+1)/2-1)=alpha_0;
    *(f_alpha+(n+1)/2-1)=log(3.)/(3.*log(2.));
    d_alpha=(beta_3_3-alpha_0)/m;
    for(k=(n+1)/2;k<n-4;k++)
    {
      *(alpha+k)=alpha_0+(k-(n+1)/2)*d_alpha;
      *(f_alpha+k)=1./3.+1./3.*log(1+(pow((p0),2.)*p1-pow(2.,-3.**(alpha+k)))/(p0*p1*(p0-p1)))/log(2.);
    }
    *(alpha+n-4)=beta_3_3; 
    *(f_alpha+n-4)=1./3.;
    *(alpha+n-3)=beta_3_3; 
    *(f_alpha+n-3)=log(3)/(3*log(2.));
    *(alpha+n-2)=beta_3_3; 
    *(f_alpha+n-2)=0.;
    *(alpha+n-1)=alpha_max; 
    *(f_alpha+n-1)=0.;
    return 1;
  }
  else 
  { 
    fprintf(stderr,"MFAS_bm3cs arguments error\n");
    return 0;
  }
}
