/* This Software is ( Copyright INRIA . 1998  1 )                    */
/*                                                                   */
/* INRIA  holds all the ownership rights on the Software.            */
/* The scientific community is asked to use the SOFTWARE             */
/* in order to test and evaluate it.                                 */
/*                                                                   */
/* INRIA freely grants the right to use modify the Software,         */
/* integrate it in another Software.                                 */
/* Any use or reproduction of this Software to obtain profit or      */
/* for commercial ends being subject to obtaining the prior express  */
/* authorization of INRIA.                                           */
/*                                                                   */
/* INRIA authorizes any reproduction of this Software.               */
/*                                                                   */
/*    - in limits defined in clauses 9 and 10 of the Berne           */
/*    agreement for the protection of literary and artistic works    */
/*    respectively specify in their paragraphs 2 and 3 authorizing   */
/*    only the reproduction and quoting of works on the condition    */
/*    that :                                                         */
/*                                                                   */
/*    - "this reproduction does not adversely affect the normal      */
/*    exploitation of the work or cause any unjustified prejudice    */
/*    to the legitimate interests of the author".                    */
/*                                                                   */
/*    - that the quotations given by way of illustration and/or      */
/*    tuition conform to the proper uses and that it mentions        */
/*    the source and name of the author if this name features        */
/*    in the source",                                                */
/*                                                                   */
/*    - under the condition that this file is included with          */
/*    any reproduction.                                              */
/*                                                                   */
/* Any commercial use made without obtaining the prior express       */
/* agreement of INRIA would therefore constitute a fraudulent        */
/* imitation.                                                        */
/*                                                                   */
/* The Software beeing currently developed, INRIA is assuming no     */
/* liability, and should not be responsible, in any manner or any    */
/* case, for any direct or indirect dammages sustained by the user.  */
/*                                                                   */
/* Any user of the software shall notify at INRIA any comments       */
/* concerning the use of the Sofware (e-mail : FracLab@inria.fr)     */
/*                                                                   */
/* This file is part of FracLab, a Fractal Analysis Software         */

/* Module de convolution de la transformee ondelettes continue */

/* Bertrand GUIHENEUF - FRACTALES 10/1996 */ 



#include "CMPT1D_Filter.h"


double CMPT1D_max(
		  double a,
		  double b
		  )
{
  if (a>b) return(a);
  else return(b);
}


double CMPT1D_min(
		  double a,
		  double b
		  )
{
  if (a<b) return(a);
  else return(b);
}



void CMPT1D_Filtrage_max(double *ent, /* pointeur sur le tableau d'entree */
			 double *sor, /* pointeur sur le tableau de sortie */
			 int tailleX, /* Taille du tableau d'entree */
			 CMPT1D_t_Filtre *filtre /* filtre de la convolution  */
			 )
  /* Convolution entre un signal et un filtre discret */
{
  
  /* Variables locales */
  int i, j, ligne;
  double tmp, *s_ptr, *e_deb_ligne, *s_deb_ligne, *conv_ptr, *ent_ptr, mirroir;
  double max;
  /*-------------------------------------------------------------------------*/
  
  /* Convention : Le prefixe e_ pour entree */
  /*              Le prefixe s_ pour sortie */

  /* i est le point ou se fait la convolution dans  */
  /* le signal d'entree */
  
  
  e_deb_ligne=ent;
  s_deb_ligne=sor; /* pour la lisibilite */
  
  
  
  
  s_ptr=s_deb_ligne;
  /* premiere partie, le filtre deborde a gauche */
  for (i=0; i<filtre->taille_neg; i++, s_ptr++)
    {
      conv_ptr=e_deb_ligne+(filtre->taille_neg-i);
      max=filtre->valeur[0]+((*conv_ptr));
      
      for (j=0; j<filtre->taille_neg-i; j++, conv_ptr--)
	max=CMPT1D_max((filtre->valeur[j]+(*conv_ptr)),max);
      
      for (j=filtre->taille_neg-i; j<filtre->taille; j++, conv_ptr++)
	max=CMPT1D_max((filtre->valeur[j]+(*conv_ptr)),max);
      *s_ptr=max;
    }
  
  
  /* deuxieme partie, roule mon gars, roule ! */
  ent_ptr=e_deb_ligne+i-filtre->taille_neg;
  for (; i<tailleX-filtre->taille_pos; i++, s_ptr++, ent_ptr++)
      {
	conv_ptr=ent_ptr;
	max=filtre->valeur[0]+((*conv_ptr));
	for (j=0; j<filtre->taille; j++, conv_ptr++)
	  max=CMPT1D_max((filtre->valeur[j]+(*conv_ptr)),max);
	*s_ptr=max;
      }
  
  
  /* troisieme partie, Ouppps! ca deborde a droite */	   
  for (; i<tailleX; i++, s_ptr++, ent_ptr++)
      {
	conv_ptr=ent_ptr;
	max=filtre->valeur[0]+((*conv_ptr));
	for (j=0; j<(tailleX-i+filtre->taille_neg); j++, conv_ptr++)
	  max=CMPT1D_max((filtre->valeur[j]+(*conv_ptr)),max);
	
	conv_ptr-=2;
	
	for (j=(tailleX-i+filtre->taille_neg); j<(filtre->taille); j++, conv_ptr--)
	  max=CMPT1D_max((filtre->valeur[j]+(*conv_ptr)),max);
	
	*s_ptr=max;
      }
  
}








void CMPT1D_Filtrage_min(double *ent, /* pointeur sur le tableau d'entree */
			 double *sor, /* pointeur sur le tableau de sortie */
			 int tailleX, /* Taille du tableau d'entree */
			 CMPT1D_t_Filtre *filtre /* filtre de la convolution  */
			 )
  /* Convolution entre un signal et un filtre discret */
{
  
  /* Variables locales */
  int i, j, ligne;
  double tmp, *s_ptr, *e_deb_ligne, *s_deb_ligne, *conv_ptr, *ent_ptr, mirroir;
  double min;
  /*-------------------------------------------------------------------------*/
  
  /* Convention : Le prefixe e_ pour entree */
  /*              Le prefixe s_ pour sortie */

  /* i est le point ou se fait la convolution dans  */
  /* le signal d'entree */
  
  
  e_deb_ligne=ent;
  s_deb_ligne=sor; /* pour la lisibilite */
  
  
  
  
  s_ptr=s_deb_ligne;
  /* printf("filtre->taille_neg:%d\n", filtre->taille_neg); */
 /*  printf(":%d\n", filtre->taille_neg); */
  
  /* premiere partie, le filtre deborde a gauche */
  for (i=0; i<filtre->taille_neg; i++, s_ptr++)
    {
      conv_ptr=e_deb_ligne+(filtre->taille_neg-i);
      min=-filtre->valeur[0]+((*conv_ptr));
      
      for (j=0; j<filtre->taille_neg-i; j++, conv_ptr--)
	min=CMPT1D_min((-filtre->valeur[j]+(*conv_ptr)),min);
      
      for (j=filtre->taille_neg-i; j<filtre->taille; j++, conv_ptr++)
	min=CMPT1D_min((-filtre->valeur[j]+(*conv_ptr)),min);
      *s_ptr=min;
    }
  
  
  /* deuxieme partie, roule mon gars, roule ! */
  ent_ptr=e_deb_ligne+i-filtre->taille_neg;
  for (; i<tailleX-filtre->taille_pos; i++, s_ptr++, ent_ptr++)
      {
	conv_ptr=ent_ptr;
	min=-filtre->valeur[0]+((*conv_ptr));
	for (j=0; j<filtre->taille; j++, conv_ptr++)
	  min=CMPT1D_min((-filtre->valeur[j]+(*conv_ptr)),min);
	*s_ptr=min;
      }
  
  
  /* troisieme partie, Ouppps! ca deborde a droite */	   
  for (; i<tailleX; i++, s_ptr++, ent_ptr++)
      {
	conv_ptr=ent_ptr;
	min=-filtre->valeur[0]+((*conv_ptr));
	for (j=0; j<(tailleX-i+filtre->taille_neg); j++, conv_ptr++)
	  min=CMPT1D_min((-filtre->valeur[j]+(*conv_ptr)),min);
	
	conv_ptr-=2;
	
	for (j=(tailleX-i+filtre->taille_neg); j<(filtre->taille); j++, conv_ptr--)
	  min=CMPT1D_min((-filtre->valeur[j]+(*conv_ptr)),min);
	
	*s_ptr=min;
      }
  
}



