/* This Software is ( Copyright INRIA . 1998  2009 )                 */
/*                                                                   */
/* INRIA  holds all the ownership rights on the Software.            */
/* The scientific community is asked to use the SOFTWARE             */
/* in order to test and evaluate it.                                 */
/*                                                                   */
/* INRIA freely grants the right to use modify the Software,         */
/* integrate it in another Software.                                 */
/* Any use or reproduction of this Software to obtain profit or      */
/* for commercial ends being subject to obtaining the prior express  */
/* authorization of INRIA.                                           */
/*                                                                   */
/* INRIA authorizes any reproduction of this Software.               */
/*                                                                   */
/*    - in limits defined in clauses 9 and 10 of the Berne           */
/*    agreement for the protection of literary and artistic works    */
/*    respectively specify in their paragraphs 2 and 3 authorizing   */
/*    only the reproduction and quoting of works on the condition    */
/*    that :                                                         */
/*                                                                   */
/*    - "this reproduction does not adversely affect the normal      */
/*    exploitation of the work or cause any unjustified prejudice    */
/*    to the legitimate interests of the author".                    */
/*                                                                   */
/*    - that the quotations given by way of illustration and/or      */
/*    tuition conform to the proper uses and that it mentions        */
/*    the source and name of the author if this name features        */
/*    in the source",                                                */
/*                                                                   */
/*    - under the condition that this file is included with          */
/*    any reproduction.                                              */
/*                                                                   */
/* Any commercial use made without obtaining the prior express       */
/* agreement of INRIA would therefore constitute a fraudulent        */
/* imitation.                                                        */
/*                                                                   */
/* The Software beeing currently developed, INRIA is assuming no     */
/* liability, and should not be responsible, in any manner or any    */
/* case, for any direct or indirect dammages sustained by the user.  */
/*                                                                   */
/* Any user of the software shall notify at INRIA any comments       */
/* concerning the use of the Sofware (e-mail : FracLab@inria.fr)     */
/*                                                                   */
/* This file is part of FracLab, a Fractal Analysis Software         */

#include "C_LAB_Interf.h"
#include <string.h>
#include<stdio.h>
#include<math.h>
#define pi 3.1416

void LAB_prescalpha()
{

        /*** Inputs****/

        Matrix *MInt;  /*** Matrice contenant les points d'interpolation ***/
        double *Int;  
        Matrix *Mcoefs;  /*** Matrice contenant les coefficients de contractions***/
        double *coefs; 
        Matrix *MIter;  /*** Matrice contenant le nombre d'iterations ***/
        double Iter; 

        /*** Outputs ***/

        Matrix *Mx;  /*** Matrice contenant les abscisses de l'attracteur du GIFS ***/
        double *x;  
        Matrix *My;  /*** Matrice contenant les ordonnees de l'attracteur du GIFS ***/
        double *y;

        /*** Autres variables ***/
    int size_Int = 3;
	int i , k , j , n , m , J , N , j_N  , a_j , flag=0 , p=0;
	double a , b , c , e , f , pt1_abs , pt1_ord , pt2_abs, pt2_ord , imag_pt1_abs , imag_pt1_ord , imag_pt2_abs , imag_pt2_ord , xd , yd , t;

	MInt = Interf.Param[0];
	Mcoefs = Interf.Param[1];
    MIter = Interf.Param[2];
    Iter = MatrixGetScalar(MIter);    
	Int = MatrixGetPr(MInt);
	coefs = MatrixGetPr(Mcoefs);

	/***** Pour se remettre dans les notations du programme originale ***/
	N = size_Int;
	J = (int)Iter;

	Mx = MatrixCreate(GIFS_pow_nj(N-1,J)+1,1,"real");
	x = MatrixGetPr(Mx);
	My = MatrixCreate(GIFS_pow_nj(N-1,J)+1,1,"real");
	y = MatrixGetPr(My);

        for(m=0 ; m<N ; m++) {
	  x[m*GIFS_pow_nj(N-1,J-1)] = Int[m];
	  y[m*GIFS_pow_nj(N-1,J-1)] = Int[m+N];
	}

	N = N-1; /***** N devient le nombre de fonctions a la premiere iteration *****/

	for(j_N=N , j=1 ; j<J ; j++ , j_N *= N) {
		a_j = GIFS_pow_nj(N,J-j);

		for(k=0 ; k<j_N ; k++) {
			if((k%N)==0){
				pt1_abs  = x[k*a_j];
				pt1_ord  = y[k*a_j];
				pt2_abs  = x[(k+2)*a_j];
				pt2_ord  = y[(k+2)*a_j];
				imag_pt1_abs = pt1_abs;
				imag_pt1_ord = pt1_ord;
				imag_pt2_abs = x[(k+1)*a_j];
				imag_pt2_ord = y[(k+1)*a_j];
				xd = imag_pt2_abs;
				yd = imag_pt2_ord;
			}
			else{
				pt1_abs  = x[(k-1)*a_j];
				pt1_ord  = y[(k-1)*a_j];
				pt2_abs  = x[(k+1)*a_j];
				pt2_ord  = y[(k+1)*a_j];
				imag_pt1_abs = x[k*a_j];
				imag_pt1_ord = y[k*a_j];
				imag_pt2_abs = pt2_abs;
				imag_pt2_ord = pt2_ord;
				xd = imag_pt1_abs;
				yd = imag_pt1_ord;				
			}				
			a = (imag_pt1_abs - imag_pt2_abs)/(pt1_abs-pt2_abs);
			e = imag_pt1_abs - a*pt1_abs;

			/******** Determination des c_k^j selon la fonction choisie ****/
			c = coefs[p];
			p++;
			
			c = pow(-1,k)/pow((double)N,c);
			
			b = (imag_pt1_ord-imag_pt2_ord - c*(pt1_ord-pt2_ord))/(pt1_abs-pt2_abs);
			f = imag_pt1_ord - b*pt1_abs - c*pt1_ord;
			x[k*a_j + a_j/N] = a*xd + e;
			y[k*a_j + a_j/N] = b*xd + c*yd + f;	
		}
	}
	
	ReturnParam(Mx);
	ReturnParam(My);

}